#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDAssert.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from ALAssert import ALAssert
from ALVerbose import ALVerbose
from ALString import ALString


class EDAssert( ALAssert ):
    """
    This class is an object encapsulation of the python assert utilities
    """

    def equal( _oExpected, _oObject ):
        """Fail if the two objects are unequal as determined by the '==' operator.
        """
        if ( ALAssert.isAssertionMode() == False ):
            return
            
        ALAssert.m_oalThreadingConditionAssert.synchronizeOn()
        
        try:
        
            if  ( _oExpected != _oObject ):
            
                EQUAL_ERROR_ASSERT_MESSAGE = \
                    EDAssert.getFailureMessage( ALString( _oExpected ), ALString( _oObject ) )
                ALVerbose.ASSERT( EQUAL_ERROR_ASSERT_MESSAGE )
                raise AssertionError, EQUAL_ERROR_ASSERT_MESSAGE
            else:
                ALVerbose.ASSERT( EDAssert.getSuccessMessage( ALString( _oExpected ), ALString( _oObject)))
        finally:
            ALAssert.m_oalThreadingConditionAssert.synchronizeOff()
    equal = staticmethod( equal )

    def getFailureMessage( _expected, _obtained ):
        return ALString( "*** FAILURE: Expected = " + _expected + " / Obtained = " + _obtained )
    getFailureMessage = staticmethod( getFailureMessage )


    def getSuccessMessage( _expected, _obtained ):
        return ALString( "OK" )
        #return ALString( "OK: Expected = " + _expected + " / Obtained = " + _obtained )
    getSuccessMessage = staticmethod( getSuccessMessage )
