#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsFile.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDObject
from EDImportLib                         import EDString
from EDImportLib                         import EDVerbose

from EDImportSystem                      import PyOs

class EDUtilsFile ( EDObject ):
    """
    """
    
    def readFile( _edStringFileName ):
        """
        """
        edString = None
        try:
            edString = EDString( open( _edStringFileName, "r" ).read() )
        except:
            EDVerbose.error( "*** EDUtilsFile.readFile: Problem when reading " + _edStringFileName )
            raise        
        return edString
    readFile = staticmethod( readFile )
    
    
    def writeFile( _edStringFileName, _edStringContent ):
        """
        """
        try:
            open( _edStringFileName, "w" ).write( _edStringContent )
        except:
            EDVerbose.error( "*** EDUtilsFile.writeFile: Problem when writing " + _edStringFileName )
            raise        
    writeFile = staticmethod( writeFile )    
    
    
    def readFileAndParseVariable( _edStringFileName, _edStringKey, _edStringValue ):
        """
        Reads a file and parses potential existing environment variables such as:
        Returns the content of this file as a string
        """
        edString = None
        try:
            edString = EDUtilsFile.readFile ( _edStringFileName )
        except:
            raise        
        
        if( edString.contains( _edStringKey ) ):
            edString = EDString( edString ).replace( _edStringKey , _edStringValue )

        return edString
    readFileAndParseVariable = staticmethod( readFileAndParseVariable )
    
    
    def readFileAndParseVariables( _edStringFileName, _edDictionary ):
        """
        Returns the content of this file as a string
        """
        edString = None
        try:
            edString = EDUtilsFile.readFile ( _edStringFileName )
        except:
            raise        
        
        for key in _edDictionary.keys():
            if( edString.contains( key ) ):
                edString = EDString( edString ).replace( key , _edDictionary.getValue( key ) )

        return edString
    readFileAndParseVariables = staticmethod( readFileAndParseVariables )
        
    
    def getFileExtension( _edStringFileName ):
        """
        Returns the file extension, e.g. "img" for "testscale_1_001.img"
        """
        edStringFileExtension = None
        edStringFileSplit = PyOs.path.splitext( _edStringFileName )
        if ( len( edStringFileSplit ) > 1):
            edStringExtensionWithSeparator = edStringFileSplit[1]
            if ( len( edStringExtensionWithSeparator ) > 1):
                # Remove the separator
                edStringFileExtension = edStringExtensionWithSeparator[1:]
        return edStringFileExtension
    getFileExtension = staticmethod( getFileExtension )