#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDUtilsPath.py 708 2009-05-13 13:20:02Z incardon $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDObject
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString

from EDImportSystem                      import PyOs


class EDUtilsPath (EDObject):
    
    m_edStringEDNA_HOME = None
    m_edStringEDNA_SITE = None


    def appendListOfPaths( _edStringPath, _pyListOfPaths ):
        """
        Appends a list of paths to a path, for example [ "..", "..", "..", "..", "src" ],
        and returns the absolute path.
        """
        edStringNewPath = _edStringPath
        for pyStrPath in _pyListOfPaths:
            edStringNewPath = EDUtilsPath.mergePath( edStringNewPath, EDString( pyStrPath ) )
        return edStringNewPath
    appendListOfPaths = staticmethod( appendListOfPaths )
            
            
    def mergePath( _edStringPath1, _edStringPath2 ):
        """
        Merges two paths and returns the absolute path.
        """
        edStringNewPath = PyOs.path.join( _edStringPath1, _edStringPath2 )
        return EDUtilsPath.getAbsolutePath( edStringNewPath )
    mergePath = staticmethod( mergePath )
    
    
    def getAbsolutePath( _edStringPath ):
        """
        Returns the absolute path.
        """
        return PyOs.path.abspath( _edStringPath )
    getAbsolutePath = staticmethod( getAbsolutePath )
    
    
    def existPath( _edStringPath ):
        """
        Checks if a folder exists.
        """
        return PyOs.path.exists( _edStringPath )
    existPath = staticmethod( existPath )
        
    
    def getEdnaHome():
        """
        Returns the EDNA_HOME variable from environment
        If environment variable not set, computes it relatively from <edna_home>/prototype/tmp
        """
        # EDNA Home
        
        if( EDUtilsPath.m_edStringEDNA_HOME == None ):
            edStringEdnaHome = PyOs.environ.get("EDNA_HOME")
            if( edStringEdnaHome != None):
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME is set: " + edStringEdnaHome )
            else:
                # if EDNA_HOME is not set, let's assume that the tests are lanched from <edna_home>/prototype/tmp
                edStringEdnaHome = "../../../edna"
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME not set: take relative path... " )
            
            EDUtilsPath.m_edStringEDNA_HOME = EDUtilsPath.getAbsolutePath( edStringEdnaHome )
            EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaHome: EDNA_HOME: " + EDUtilsPath.m_edStringEDNA_HOME )
        
            if ( EDUtilsPath.existPath( EDUtilsPath.m_edStringEDNA_HOME ) != True ):
                EDVerbose.error( "!!!" + EDUtilsPath.m_edStringEDNA_HOME + " Not Found!!!" )
            
        return EDUtilsPath.m_edStringEDNA_HOME
    getEdnaHome = staticmethod( getEdnaHome )
    
    
    def getEdnaSite():
        """
        Returns the EDNA_SITE variable from environment
        """    
        # EDNA Site
        if( EDUtilsPath.m_edStringEDNA_SITE == None ):
            EDUtilsPath.m_edStringEDNA_SITE = PyOs.environ.get("EDNA_SITE")
            if( EDUtilsPath.m_edStringEDNA_SITE != None):
                EDVerbose.DEBUG( "*** EDUtilsEnv.getEdnaSite: EDNA_SITE is set: " + EDUtilsPath.m_edStringEDNA_SITE )
            else:
                EDVerbose.warning( "*** EDUtilsEnv.getEdnaSite: EDNA_SITE not set!!" )

        return EDUtilsPath.m_edStringEDNA_SITE
    getEdnaSite = staticmethod( getEdnaSite )
    

