#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCase.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from ALTestCase                       import ALTestCase
from ALInformationTest                import ALInformationTest
from ALManagerTest                    import ALManagerTest as EDManagerTest

from EDImportLib                      import EDVerbose
from EDImportLib                      import EDDebug
from EDImportLib                      import EDString
from EDImportLib                      import EDLogFile
from EDImportLib                      import EDDiskExplorer

from EDAssert                         import EDAssert
from EDUtilsPath                      import EDUtilsPath
from EDUtilsTest                      import EDUtilsTest


class EDTestCase( ALTestCase ):
    """
    The main test class for all TestCase (Unit and execution Test cases)
    """
    
    def __init__( self, _oalStringTestName = None ):
        """
        Initialize the test case:
        - Traces
        - Home directories
        """
        ALTestCase.__init__( self, _oalStringTestName )
        if ( _oalStringTestName is None ):
            self.m_oalStringTestName = EDString( self.getClassName() )
            
        self.m_oLogFile = None
        if (_oalStringTestName != None ):
            oalStringNameLog = self.m_oalStringTestName + EDString( ".log" )
            self.m_oLogFile = EDLogFile( oalStringNameLog )
            
        EDVerbose.setVerboseOn()
        EDAssert.setAssertionModeOn()

        self.m_edStringTestsHome           = None 
        self.m_edStringTestsDataHome       = None   
        self.m_edStringTestsDataImagesHome = None
        
        EDUtilsTest.getFactoryPluginTest().loadModule( self.getClassName() )
        

    def getTestsHome( self ):
        """
        Returns the Test home directory
        """
        if ( self.m_edStringTestsHome is None ):
            self.m_edStringTestsHome = EDUtilsTest.getTestsHome()
        return self.m_edStringTestsHome
    
    
    def getTestsDataHome( self ):
        """
        Returns the Test data home directory
        """
        if ( self.m_edStringTestsDataHome is None ):
            self.m_edStringTestsDataHome = EDUtilsTest.getTestsDataHome()
        return self.m_edStringTestsDataHome
    
    
    def getTestsDataImagesHome( self ):
        """
        Returns the Test data home directory
        """
        if ( self.m_edStringTestsDataImagesHome is None ):
            self.m_edStringTestsDataImagesHome = EDUtilsTest.getTestsDataImagesHome()
        return self.m_edStringTestsDataImagesHome

    
    def processKernel( self ):
        """
        Test execution method
        """

        bAssertion = False
        bException = False

        iNumberMethods = self.m_oalListTest.getNumberObjects( )
        EDVerbose.unitTest( "===================================================================" )
        EDVerbose.unitTest( "TestCase Total Number of Tests = [ " + EDString( iNumberMethods ) + " ]" )
        EDVerbose.unitTest( " " )
        iTestCaseNumber = 0
        for oTestMethod in self.m_oalListTest:
            iTestCaseNumber = iTestCaseNumber + 1
            edStringMethodName = EDString( self.patchMethodName( oTestMethod ) )

            oalInformationTest = ALInformationTest( self.patchMethodName( oTestMethod ), iTestCaseNumber )
            EDManagerTest.addInformationTestMethod( oalInformationTest )

            EDVerbose.unitTest( "===================================================================" )
            EDVerbose.unitTest( "TestCase Method [ " + edStringMethodName  + " ]" )
            EDVerbose.unitTest( "TestCase Method [ " + EDString( iTestCaseNumber )  + " ]" )
            EDVerbose.unitTest( "-------------------------------------------------------------------" )

            try:
                oTestMethod()
                oalInformationTest.setTerminate()
                oalInformationTest.setSuccess()
                EDVerbose.unitTest( "[ " + edStringMethodName + " ] : [SUCCESS]" )
                EDVerbose.unitTest( " " )
            except AssertionError, opyException:
                oalInformationTest.setTerminate()
                oalInformationTest.setFailure()
                oalInformationTest.setException( EDString(opyException) )
                EDVerbose.unitTest( "TestCase = Assertion Error Raised"  )
                EDVerbose.unitTest( "[ " + edStringMethodName + " ] : [FAILURE]" )
                EDVerbose.unitTest( " " )
                bAssertion = True
                EDDebug.controlDebug( self )

            except Exception, opyException:
                oalInformationTest.setTerminate()
                oalInformationTest.setFailure()
                oalInformationTest.setException( EDString(opyException) )
                EDVerbose.unitTest( "TestCase = Unexpected Error Raised"  )
                EDVerbose.unitTest( "[ " + edStringMethodName + " ] : [FAILURE]" )
                EDVerbose.unitTest( " " )
                bException = True
                EDDebug.controlDebug( self )

        if(( bAssertion == True ) or ( bException == True )):
            raise
 

        
    def execute( self ):
        """
        This method overrides the ALTest execute method to raise the exception to the TestSuite
        """
        EDVerbose.DEBUG( "Execution: EDTestCase.execute()" )
        try:
            self.executeKernel()
        except:
            raise
        