#
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePlugin.py 664 2009-05-06 13:05:34Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                    import EDVerbose
from EDImportLib                    import EDString
from EDImportLib                    import EDDiskExplorer
from EDUtilsPath                    import EDUtilsPath
from EDTestCase                     import EDTestCase
from EDUtilsTest                    import EDUtilsTest



class EDTestCasePlugin( EDTestCase ):
    """
    This is the main test class to test a plugin (Unit and Execution)
    """
    
    def __init__( self, _edStringPluginName, _edStringPluginDir = None, _edStringTestName = None ):
        """
        Initialize the test case:
        - Plugin Home directories: <edna>/prototype/plugins/<MyPlugin>
        - Plugin test data home directory: <edna>/prototype/plugins/<MyPlugin>/tests/data
        """
        EDTestCase.__init__( self, _edStringTestName )
        
        self.m_edStringPluginName = _edStringPluginName
                        
        self.m_edStringTestsDataDir = None
        self.m_edStringPluginHome = EDUtilsTest.getFactoryPluginTest().getModuleLocation( _edStringPluginName )
        self.m_edStringPluginTestsDataHome = EDUtilsTest.getPluginTestDataDirectory( self.getClassName())
                       
            
    def getPluginName( self ):
        """
        Returns the plugin name
        """
        return self.m_edStringPluginName
    
    
    def getPluginHome( self ):
        """
        Returns the plugin home directory
        """
        return self.m_edStringPluginHome
    
    
    def setPluginTestsDataDir( self, _edStringPluginDataDir ):
        """
        Sets the plugin test data directory
        """
        self.m_edStringPluginTestsDataDir = _edStringPluginDataDir
        
        
    def getPluginTestsDataHome( self ):
        """
        Returns the plugin test data home directory
        """
        return self.m_edStringPluginTestsDataHome
        
        
    def setPluginTestsDataHome( self, _edStringPluginTestsDataHome ):
        """
        Sets the plugin test data home directory
        """
        self.m_edStringPluginTestsDataHome = _edStringPluginTestsDataHome
        
        
           
                
