#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDApplicationDeprecated.py 586 2008-12-11 10:28:54Z incardon $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDObject
from EDImportLib import EDVerbose
from EDImportLib import EDCommandLine
from EDImportLib import EDCommandLineArgument
from EDImportLib import EDLogFile

from EDUtilsTest import EDUtilsTest


class EDTestLauncher( EDObject ):

    m_edStringTEST_LABEL = "--test"

    
    """
    """
    
    def __init__ ( self ):
        EDObject.__init__( self )
        self.m_edTestCase       = None


    def preProcess( self ):
        """
        Scans the command line.
        """
        EDVerbose.setLogFileOff()
        EDVerbose.DEBUG( " *** EDTestLauncher.preProcess" )
        edCommandLine = EDCommandLine( EDCommandLineArgument() )
        EDVerbose.setVerboseOn()

        # The check for --quiet and --DEBUG should ideally be placed elsewhere,
        # for example in EDApplication.
        if ( edCommandLine.existCommand( "--quiet" ) ):
            EDVerbose.setVerboseOff()

        if ( edCommandLine.existCommand( "--DEBUG" ) ):
            EDVerbose.setVerboseDebugOn()
            EDVerbose.DEBUG( " *** EDTestLauncher.preProcess: Debug mode" )

        edStringTestName = edCommandLine.getArgument( EDTestLauncher.m_edStringTEST_LABEL )
        EDVerbose.DEBUG( " *** EDTestLauncher.preProcess: test name = %r" % edStringTestName )
        if ( edStringTestName is None ):
            EDVerbose.screen( "ERROR - no --test argument found" )
        else:
            self.m_edTestCase = EDUtilsTest.getFactoryPluginTest().loadPlugin( edStringTestName )
            EDVerbose.m_aolLogFile = EDLogFile( edStringTestName + ".log")
            EDVerbose.setLogFileOn()
            

    def process( self ):
        EDVerbose.DEBUG( " *** EDTestLauncher.process" )
        if ( self.m_edTestCase is not None ):
            EDVerbose.DEBUG( " *** EDTestLauncher.process: launching test" )
            self.m_edTestCase.execute()
    
    
    def execute( self ):
        """
        This method executes the EDTestlauncher.
        """
        EDVerbose.DEBUG( " *** EDTestLauncher.execute" )
        # This code should ideally be placed elsewhere, e.g. in an EDApplication class.
        self.preProcess()
        self.process()