##
#    Project: The EDNA Kernel
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDUtilsImage.py 671 2009-05-06 14:24:21Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#



from EDImportLib import EDCompiler
from EDImportLib import EDVerbose
from EDImportLib import EDDiskExplorer
from EDImportLib import EDList
from EDImportLib import EDDictionary
from EDImportLib import EDString 

from EDUtilsImage import EDUtilsImage
from EDTestCase   import EDTestCase
from EDAssert     import EDAssert


class EDTestCaseEDUtilsImage( EDTestCase ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _edStringTestName )
        self.m_edConfiguration = None
        edStrDataDir = "images"
        self.m_edStrDataPath = EDDiskExplorer.mergePath( self.getTestsDataHome(), edStrDataDir )
        self.m_edStringFileName = "ref-testscale_1_001.img"
        self.m_edStringFilePath = EDDiskExplorer.mergePath( self.m_edStrDataPath, self.m_edStringFileName )
        
        
    def testGetImageNumber( self ):
        """
        Testing retrieving the image number from a image filename
        """
        iImageNumber = EDUtilsImage.getImageNumber( self.m_edStringFileName )
        iReference = 1
        EDAssert.equal( iReference, iImageNumber )
                

    def testGetTemplateHash( self ):
        """
        Testing retrieving the image number from a image filename
        """
        edStringSymbol = "#"
        edStringTemplate = EDUtilsImage.getTemplate( self.m_edStringFileName )
        edStringTemplateReference = EDString( "ref-testscale_1_###.img" )
        EDAssert.equal( edStringTemplateReference, edStringTemplate )


    def testGetTemplateQuestionMark( self ):
        """
        Testing retrieving the image number from a image filename
        """
        edStringSymbol = "?"
        edStringTemplate = EDUtilsImage.getTemplate( self.m_edStringFileName, edStringSymbol )
        edStringTemplateReference = EDString( "ref-testscale_1_???.img" )
        EDAssert.equal( edStringTemplateReference, edStringTemplate )


        
##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testGetImageNumber )
        self.addTestMethod( self.testGetTemplateHash )
        self.addTestMethod( self.testGetTemplateQuestionMark )



##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCaseEDUtilsImage = EDTestCaseEDUtilsImage( "TestCase EDTestCaseEDUtilsImage" )
    edTestCaseEDUtilsImage.execute()
