#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition                  import  *
from ALSystem                      import  ALSystem
from ALString                      import  ALString
from ALVerbose                     import  ALVerbose
from ALTime                        import  ALTime
from ALDiskExplorer                import  ALDiskExplorer
from ALSlot                        import  ALSlot
from ALSignal                      import  ALSignal
from ALDebug                       import  ALDebug
from ALTrace                       import  ALTrace
from ALThreadControl               import  ALThreadControl
from ALException                   import  ALException
from ALExceptionSignal             import  ALExceptionSignal
from ALApplicationActionControl    import  ALApplicationActionControl



class ALAction( ALThreadControl ):
    """
    Parent class for all ALAction objects.

    #################################################################
    ALL Slot connection point available for all ALAction Tree
    self.connectInit( self, _oMethod ):
    self.connectPause( self, _oMethod ):
    self.connectResume( self, _oMethod ):
    self.connectAbort( self, _oMethod ):
    self.connectTerminate( self, _oMethod ):
    self.connectException( self, _oMethod ):
    self.connectError( self, _oMethod ):
    self.connectEnd( self, _oMethod ):
    self.connectExecute( self, _oMethod ):
    self.connectRun( self, _oMethod ):
    self.connectExecute( _oMethod )
    """
    def __init__( self, _oBoolPriority = False ):
        """
        Constructor for ALAction.

        """
        ALThreadControl.__init__( self )
        self.m_oalSlotPreProcess              = ALSlot()
        self.m_oalSlotProcess                 = ALSlot()
        self.m_oalSlotPostProcess             = ALSlot()
        self.m_oalSlotSUCCESS                 = ALSlot()
        self.m_oalSlotFAILURE                 = ALSlot()
        self.m_oalSlotScreen                  = ALSlot()
        self.m_oalSlotDEBUG                   = ALSlot()
        self.m_oalBoolHighPriority            = _oBoolPriority
        self.m_oalBoolSuccess                 = False
        self.m_oalStrWorkingDirectory         = ALDiskExplorer.getCurrentFolder()
        self.m_oalStrWorkingDirectoryOriginal = self.m_oalStrWorkingDirectory
        self.m_iTimeDelaySystemLoadBalance    = AL_TIME_MINIMUM / 10.0



        self.connectSUCCESS( self.traceSuccessStatusLog )
        self.connectFAILURE( self.traceFailureStatusLog )




    def setTimeDelaySystemLoadBalance( self, _iTimeDelay ):
        self.m_iTimeDelaySystemLoadBalance  = _iTimeDelay


    def getTimeDelaySystemLoadBalance( self  ):
        return self.m_iTimeDelaySystemLoadBalance


    def setHighPriority( self, _oBoolPriority = True ):
        self.m_oalBoolHighPriority  = _oBoolPriority


    def getHighPriority( self ):
        return self.m_oalBoolHighPriority

    def isHighPriority( self ):
        return self.m_oalBoolHighPriority


    def getWorkingFolder( self ):
        return self.m_oalStrWorkingDirectory


    def getTerminateSuccess(self):
        return self.m_oalBoolSuccess


    def setWorkingFolder( self, _oalStrWorkingDirectory ):
        self.m_oalStrWorkingDirectoryOriginal = ALDiskExplorer.getCurrentFolder()
        self.m_oalStrWorkingDirectory = ALDiskExplorer.absolutePath( _oalStrWorkingDirectory )
        ALDiskExplorer.createFolder( self.m_oalStrWorkingDirectory )


    def traceSuccessStatusLog( self, _oalObject = None ):
        ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] [ActionEndState] :: [SUCCESS]")


    def traceFailureStatusLog( self, _oalObject = None ):
        ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] [ActionEndState] :: [FAILURE]")



    def __setRegisterOn(self):
        # Thread Register in the ALApplication
        if (self.getRegisterAction()==True):
            ALApplicationActionControl.startAction( self )


    def __setRegisterOff(self):
        # Thread Register in the ALApplication
        if (self.getRegisterAction()==True):
            ALApplicationActionControl.endAction( self )



    def run( self ):
        """
        This is the method started by the thread model.
        """
        self.executeKernel()


    def actionControl( self ):
        self.threadControl()


    def __raiseControl( self ):
        if (ALDebug.isDebugModeActionOn()==True or ALDebug.isDebugModeForceOn()==True):
            ALDebug.debugForce( self, False )
        else:
            oBoolStatus =  ALDebug.isDebugModeOn() # Here because it is a bug in jython
            ALTrace.trace( self, oBoolStatus )



    def executeAction( self, _oalObject = None ):
        """
        This method starts the thread.
        Only here to remember the thread action in the framework
        """
        self.execute( _oalObject )


    def executeActionSynchronous( self, _oalObject = None ):
        """
        This method starts the Action in Synchronous  Mode.
        """
        self.executeSynchronous()



    def connectPreProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotPreProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectPostProcess( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotPostProcess.connect( _oMethod )
        self.synchronizeOff()


    def connectSUCCESS( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotSUCCESS.connect( _oMethod )
        self.synchronizeOff()


    def connectFAILURE( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotFAILURE.connect( _oMethod )
        self.synchronizeOff()



    def connectScreen( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotScreen.connect( _oMethod )
        self.synchronizeOff()



    def connectDEBUG( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotDEBUG.connect( _oMethod )
        self.synchronizeOff()



    def connectConsole( self, _oMethod ):
        self.connectScreen( _oMethod )
        self.connectDEBUG( _oMethod )



    def screen( self, _oalString ):
        self.m_oalSlotScreen.call( _oalString )
        ALVerbose.screenID( self.getID(), _oalString )


    def DEBUG( self, _oalString, _oalStringTag = AL_DEBUG_TAG):
        oalString = ALString( "[DEBUG]: " ) + ALString( _oalString )
        if (ALDebug.isDebugModeOn()==True):
            self.m_oalSlotDEBUG.call( oalString )
        ALVerbose.DEBUGID( self.getID(), _oalString, _oalStringTag )




    def terminate(self, _oalObject = None ):
        self.abort()
        self.m_oalSlotAbort.callSignal( ALSignal.abort() )


    def connectTerminate( self, _oMethod ):
        self.connectAbort( _oMethod )



    def executeActionTimer( self ):
        """
        This is the main connection method to be used with Timer Plugins and methods.
        """
        self.executeKernel( )




    def executeKernel( self ):
        """
        This is the main kernel of the Action class.
        """
        try:
            try:
                # Controls the Succes of the excecute
                self.m_oalBoolSuccess = False

                self.m_oalTime = ALTime()
                ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] [ThreadStart]  :: " + ALTime.timeString() )

                # Thread Register in the ALApplication
                self.__setRegisterOn()

                # Thread Init
                self.actionControl()
                self.m_oalSlotInit.call( self )

                # PreProcess Sequence
                self.actionControl()
                self.preProcess( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotPreProcess.call( self )

                # Process Sequence
                self.actionControl()
                self.process( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotProcess.call( self )

                # PostProcess Sequence
                self.actionControl()
                self.postProcess( self.m_oalObject )
                self.actionControl()
                self.m_oalSlotPostProcess.call( self )


                #Thread end
                self.actionControl()
                self.m_bIsEnded = True
                self.m_oalSlotEnd.call( self )

                # Controls the Succes of the excecute
                self.m_oalBoolSuccess = True

            except ALExceptionSignal, _oalExceptionSignal:
                oalSignal = _oalExceptionSignal.getSignal()
#                self.m_oalSlotAbort.call()
                if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT ):
                    self.m_oalSlotAbort.callSignal( ALSignal.abort() )
                elif ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ):
                    self.m_oalSlotError.callSignal( ALSignal.error() )
                    #self.__raiseControl() # It generates a bug in Jython....
                    #This code instead to bypass the Jython bug
                    if (ALDebug.isDebugModeActionOn()==True or ALDebug.isDebugModeForceOn()==True):
                        ALDebug.debugForce( self, False )
                    else:
                        oBoolStatus =  ALDebug.isDebugModeOn() # Here because it is a bug in jython
                        ALTrace.trace( self, oBoolStatus )
                else:
                    ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] Action stopped under Unknown signal: "  + ALString( oalSignal.getSignal() ) )


            except:
                # Priority control to avoid the Application interruption = complete stop
                if (self.isHighPriority()==True):
                    self.m_oalSlotException.callSignal( ALSignal.exception() )
                    ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] Action exception with HIGH Priority" )
                    ALVerbose.aalibDEBUGID( self.getID(), "           Action: " + ALString( self.getID() ) )
                else:
                    ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] Action exception with LOW Priority" )
                    ALVerbose.aalibDEBUGID( self.getID(), "           Action: " + ALString( self.getID() ) )
                    self.m_oalSlotError.callSignal( ALSignal.error() )
                #self.__raiseControl() # It generates a bug in Jython....
                #This code instead to bypass the Jython bug
                if (ALDebug.isDebugModeActionOn()==True or ALDebug.isDebugModeForceOn()==True):
                    ALDebug.debugForce( self, False )
                else:
                    oBoolStatus =  ALDebug.isDebugModeOn() # Here because it is a bug in jython
                    ALTrace.trace( self, oBoolStatus )

        finally:
            #Thread end
            self.m_bIsEnded = True
            self.m_oalTimeDuration = self.m_oalTime.getDuration()
            ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] [ThreadEnd]    :: " + ALTime.timeString() )
            ALVerbose.aalibDEBUGID( self.getID(), "[ALAction] [ThreadRuntime]:: " + ALString( self.m_oalTimeDuration ) )

            if (self.getTerminateSuccess() == True):
                self.m_oalSlotSUCCESS.call( self )
            elif (self.getTerminateSuccess() == False):
                self.m_oalSlotFAILURE.call( self )

            # Thread Register in the ALApplication
            self.__setRegisterOff()






############################################################################
# To be modified by the user

    def preProcess( self, _oalObject = None ):
        """
        Any processing that must occur before the action should go into
        this method.
        """
        self.synchronizeOn()
        ALVerbose.aalibDEBUGID( self.getID(),"Execution: ALAction.preProcess()" )
        self.synchronizeOff()


    def process( self, _oalObject = None ):
        """
        This is the main method of the action.
        """
        self.synchronizeOn()
        ALVerbose.aalibDEBUGID( self.getID(),"Execution: ALAction.process()" )
        self.synchronizeOff()


    def postProcess( self, _oalObject = None ):
        """
        Similar to preProcess, any processing
        """
        self.synchronizeOn()
        ALVerbose.aalibDEBUGID( self.getID(),"Execution: ALAction.postProcess()" )
        self.synchronizeOff()


##############################################################################

