#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition       import *
from ALVerbose          import ALVerbose
from ALTime             import ALTime
from ALActionMethod     import ALActionMethod


class ALActionMethodTimer( ALActionMethod ):
    """
    Interface for ALActionMethodTimer classes.

    The methods to be modified by the user (overload):
        def preProcess( self, _oalObject = None )
        def postProcess( self, _oalObject = None )

    """
    def __init__( self, _oalClassMethod = None,
                 _ofMethodTimeDelay = 0.1, _ofMethodTimeDuration = AL_TIME_INFINITE, _obRegisterAction = False ):
        ALActionMethod.__init__( self, _oalClassMethod )
        self.m_oalMethodTimeDelay      = _ofMethodTimeDelay
        self.m_oalMethodTimeDuration   = _ofMethodTimeDuration

        if (_obRegisterAction == False):
            self.setNoRegisterAction()


    def getMethodTimeDelay( self ):
        return self.m_oalMethodTimeDelay

    def getMethodTimeDuration( self ):
        return self.m_oalMethodTimeDuration


    def setMethodTimeDelay( self, _oalMethodTimeDelay ):
        self.m_oalMethodTimeDelay = _oalMethodTimeDelay

    def setMethodTimeDuration( self, _oalMethodTimeDuration ):
        self.m_oalMethodTimeDuration = _oalMethodTimeDuration


    # NOTE: avoid to set methods started on the thread.run() method
    # It will generate an error starting the run() method on the loop
    def process( self, _oalObject = None ):
        ALVerbose.aalibDEBUGID( self.getID(), "Execution: ALActionMethodTimer.process()" )
        oalTime = ALTime()
        while ( oalTime.getDuration()<self.getMethodTimeDuration() ):
            self.actionControl()
            ALActionMethod.process( self, _oalObject )
            self.actionControl()
            self.threadControlTimer( self.getMethodTimeDelay() )


##############################################################################
