#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition         import *
from ALActionController   import ALActionController
from ALActionMethod       import ALActionMethod
from ALList               import ALList
from ALManagerProcess     import ALManagerProcess
from ALString             import ALString
from ALSubProcess         import ALSubProcess
from ALSystem             import ALSystem
from ALTime               import ALTime
from ALToken              import ALToken
from ALVerbose            import ALVerbose


class ALActionProcess( ALActionController ):
    """
    Interface for ALActionProcess classes.

    The method to be modified by the user (overload):
        def preProcess(  self, _oalObject = None )
        def postProcess( self, _oalObject = None )

    """
    def __init__( self, _oCommand, _oalStringFolderName = "./" ):
        ALActionController.__init__( self )
        self.m_oalSubProcess     = None
        self.m_oalActionMethod     = None
        self.m_oPid              = None
        self.m_oListRead         = None
        self.m_oListWrite        = None
        self.m_oListError        = None
        self.m_oCommand          = ALString( _oCommand )
        self.m_obIsStarted       = False
        self.m_oalManagerProcess = ALManagerProcess()
        self.setWorkingFolder( _oalStringFolderName )
        self.m_oalListDataProcess = None

        self.m_oExecutable = None
        self.__getExecutableName()



    def __getExecutableName(self):
        if(ALSystem.getOSName()=="Windows"):
            oalList = self.m_oCommand.split()
            if (oalList[0]!=""):
                self.m_oExecutable = ALString( oalList[0] ).replace( "/", "\\" )
            else:
                self.m_oExecutable = ALString( oalList[1] ).replace( "/", "\\" )
        else:
            self.m_oExecutable = self.m_oCommand
        self.m_oExecutable = ALString( self.m_oExecutable.capitalize() )


    def getCommandLine( self ):
        return self.m_oCommand


    def getInput( self ):
        self.synchronizeOn()
        oalList = ALList()
        oalList.add( self.m_oCommand )
        self.synchronizeOff()
        return oalList


    def getOutput( self ):
        self.synchronizeOn()
        oalList = ALList()
        if (self.m_oalSubProcess!=None):
            oalList = self.m_oalSubProcess.getOutput()
        self.synchronizeOff()
        return oalList


    def getOutputError( self ):
        self.synchronizeOn()
        oalList = ALList()
        if (self.m_oalSubProcess!=None):
            oalList = self.m_oalSubProcess.getOutputError()
        self.synchronizeOff()
        return oalList



    def executeProcess( self, _oalObject = None ):
        """
        This method starts the thread.
        """
        self.executeAction( _oalObject )




    def process( self, _oalObject = None ):
        """
        This is the main method of the action.
        """
        ALVerbose.aalibDEBUGID( self.getID(), "Execution: ALActionProcess.process()" )

        self.__controlListDataProcess()

        self.synchronizeOn()
        # ALL folder change is deprecated to avoid Python mistakes in massive parallel actions
        self.m_oalSubProcess = ALSubProcess( self.m_oCommand, self.getWorkingFolder() )


        # It takes time to start the child process on Lazy systems - Problem of Load Balance on the system
        # Default time is AL_TIME_MINIMUM / 10.0 - 0.08 s
        ALTime.wait( self.getTimeDelaySystemLoadBalance() )
        self.m_oalActionMethod = ALActionMethod( self.m_oalSubProcess.executeProcess )
        self.m_oalActionMethod.setTimeOut( self.getTimeOut() )
        self.m_oalActionMethod.execute( _oalObject )

        self.synchronizeOff()

        self.__controlListDataProcess()
        self.__synchronizeActionMethod()



    def __controlListDataProcess( self ):

        oalUserName = self.m_oalManagerProcess.getUserName()
#        print oalUserName
        self.m_oalManagerProcess.control()
        oalListDataProcess = self.m_oalManagerProcess.getListDataProcess()
        if (self.m_oalListDataProcess==None):
            self.m_oalListDataProcess = ALList( oalListDataProcess )
#            print "New List: " + ALString( self.m_oalListDataProcess.getNumberObjects() )
            for iProcOld in self.m_oalListDataProcess:
                if (oalUserName!=iProcOld.getUserName()):
                    iProcOld.setPid( -1 )
        else:
            for iProc in oalListDataProcess:
                if (oalUserName==iProc.getUserName()):
                    obStatus = True
                    for iProcOld in self.m_oalListDataProcess:
                        if ( int( iProc.getPid()) == int( iProcOld.getPid() )):
                            obStatus = False
                            iProcOld.setPid( -1 )
                    if(obStatus == True and oalUserName==iProc.getUserName()):
#                        print "adding: " + ALString( iProc.getPid() )
                        self.m_oalListDataProcess.append( iProc )


            if (self.m_oPid!=None):
                oiPidOld = self.m_oPid
            else:
                oiPidOld = 0
#            print self.m_oPid
            for iProc in self.m_oalListDataProcess:
                if (int( iProc.getPid() )!=-1):

                    oalString = ALString( iProc.getCommand().capitalize() )
#                    print oalString
#                    if(self.m_oExecutable.contains( oalString )==True):
                    if( oalString.contains( self.m_oExecutable )==True and
                        oalUserName==iProc.getUserName() and
                        oiPidOld < int(iProc.getPid()) ):
#                        print iProc.getImageName()
#                        print iProc.getPid()
#                        print iProc.getSessionName()
#                        print iProc.getSession()
#                        print iProc.getMemoryUsage()
#                        print iProc.getStatus()
#                        print iProc.getUserName()
#                        print iProc.getCpuTime()
#                        print iProc.getWindowTitle()
#                        print iProc.getCommand()
#                        print "\n\n\n"
                        self.m_oPid = iProc.getPid()
                        oiPidOld = iProc.getPid()




    def __synchronizePid( self, _oTimeOut = 2.0 ):
        """
        It controls the lazy startup on charged systems and try to
        catch the pid number
        """
        ALVerbose.aalibDEBUGID( self.getID(), "Execution: ALActionProcess.__synchronizePid()" )

        oalTime = ALTime()
        while (self.m_oPid==None):
            self.actionControl()
            ALTime.wait( AL_TIME_MINIMUM )
            self.__controlListDataProcess()
            if (oalTime.getDuration()>_oTimeOut):
                self.m_oPid = -1

        # To verify lazy startup
        if (self.m_oPid == -1):
            self.__controlListDataProcess()



    def __synchronizeActionMethod( self ):
        ALVerbose.aalibDEBUGID( self.getID(), "Execution: ALActionProcess.synchronize()" )
        self.__synchronizePid()
        oalTime = ALTime()
        while (self.__isRunningActionMethod()==True):
#            print oalTime.getDuration()
            self.actionControl()
            ALTime.wait( AL_TIME_MINIMUM )
            if (oalTime.getDuration()>self.getTimeOut()):
                self.abort()


    def synchronize( self ):
        self.__synchronizeActionMethod()


    def synchronizeAction( self ):
        self.synchronize()


    def __patchLinuxBashProcess( self ):
        #print ">>>>>>>>>>>>>>>>>>>>> teste do if not"
        obIsRunnig = self.m_oalManagerProcess.isRunning( self.m_oCommand )
        if (obIsRunnig==True):
            self.m_oPid = self.m_oalManagerProcess.getPid( self.m_oCommand )
        return obIsRunnig



    def __isRunningActionMethod( self ):
        self.synchronizeOn()
        obIsRunnig = False
        if (ALSystem.getInterpreter()=="Python"):
            #Patch to control if the process is created or Not
            if (self.m_oalActionMethod==None):
                ALVerbose.warningID( self.getID(), "Problem creating the SubProcess: Please inspect your Python SubProcess Module" )
            # The system will explose if the python subprocess crash the child subprocess
            if (self.m_oalActionMethod.getTerminateSuccess()!=True):
                obIsRunnig = self.m_oalManagerProcess.isRunningPid( self.m_oPid )
                if (obIsRunnig==False):
                    obIsRunnig = self.__patchLinuxBashProcess()
                ALVerbose.aalibDEBUGID( self.getID(), "ALActionProcess.isRunning(): " + ALString( obIsRunnig ) + "   PID: " + ALString( self.m_oPid ) )
            else:
                ALVerbose.aalibDEBUGID( self.getID(),"ALActionMethod class finished the execution of the subprocess" )
        else:
            obIsRunnig = self.m_oalManagerProcess.isRunningPid( self.m_oPid )
            if (obIsRunnig==False):
                obIsRunnig = self.__patchLinuxBashProcess()
            ALVerbose.aalibDEBUGID( self.getID(), "ALActionProcess.isRunning(): " + ALString( obIsRunnig ) + "   PID: " + ALString( self.m_oPid ) )
        self.synchronizeOff()
        return obIsRunnig



    def abort( self, _oalObject = None ):
        self.synchronizeOn()
#        print "abort"
        self.m_oalManagerProcess.killProcess( self.getCommandLine() )
        self.m_bIsAbort = True
        self.synchronizeOff()



##############################################################################
