#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALList                import  ALList
from ALSignal              import  ALSignal
from ALActionController    import  ALActionController


class ALActionSet( ALActionController ):
    """
    Parent class for all action set objects.

    The methods to be modified by the user (overload):
        def preProcess( self, _oalObject = None )
        def process( self, _oalObject = None )
        def postProcess( self, _oalObject = None )


    NOTE: this class is ASynchronous
    """
    def __init__( self ):
        """
        Constructor for ALActionSet.

        """
        ALActionController.__init__( self  )
        self.m_oalListAction = ALList()


    def deleteActions(self):
        """
        Delete all actions - Warning: it deletes all actions!!!
        """
        self.synchronizeOn()
        if ( self.m_oalListAction!=None ):
            self.m_oalListAction.deleteObjects()
        self.synchronizeOff()



    def addAction( self, _oalAction ):
        """
        Add actions
        """
        self.synchronizeOn()
        if ( _oalAction!=None ):
            self.m_oalListAction.append( _oalAction )
            self.m_oalSlotAbort.connect( _oalAction.abort, ALSignal.abort() )
            self.m_oalSlotException.connect( _oalAction.abort, ALSignal.exception() )
        self.synchronizeOff()



    def abort( self, _oalObject = None ):
        ALActionController.abort( self )
        for action in self.m_oalListAction :
            action.abort()


    def pause( self, _oalObject = None ):
        ALActionController.pause( self )
        for action in self.m_oalListAction :
            action.pause()


    def resume( self, _oalObject = None ):
        ALActionController.resume( self )
        for action in self.m_oalListAction :
            action.resume()



##############################################################################


