#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition               import *
from ALVerbose                  import ALVerbose
from ALString                   import ALString
from ALSignal                   import ALSignal
from ALList                     import ALList
from ALDiskExplorer             import ALDiskExplorer
from ALToken                    import ALToken
from ALCommandLineInterface     import ALCommandLineInterface
from ALAppDefinition            import ALAppDefinition


class ALAppDefinitionCommandLine( ALAppDefinition ):
    """
    Interface for ALAppDefinitionOutput objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinition.__init__( self, _oalString, _dversion, _obPlugin )

        ALAppDefinitionCommandLine.m_oalCommandLine.m_oalSlotException.connect( self.exit, ALSignal.exception() ) # not used??


    def setCommandLineInterfaceFile( self, _oalStringFile ):
        oalStringFile = ALDiskExplorer.getAbsolutePath( _oalStringFile )
        ALAppDefinitionCommandLine.m_oalCommandLineInterface = ALCommandLineInterface( oalStringFile , ALAppDefinitionCommandLine.isFactoryPluginDynamic() )


    def getCommandLineArguments():
        ALAppDefinitionCommandLine.synchronizeOn()
        oalArguments = ALAppDefinitionCommandLine.m_oalCommandLine.getCommandLine()
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalArguments
    getCommandLineArguments = staticmethod( getCommandLineArguments )


    def setCommandLine( _oalStringCommand ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalArguments = ALAppDefinitionCommandLine.m_oalCommandLine.setCommandLine( _oalStringCommand )
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalArguments
    setCommandLine = staticmethod( setCommandLine )



    def addCommand( self, _oalStringToken, _oalObjectMethod ):
        ALAppDefinitionCommandLine.synchronizeOn()
        ALAppDefinitionCommandLine.m_oalCommandLine.addCommand( _oalStringToken, _oalObjectMethod )
        ALAppDefinitionCommandLine.synchronizeOff()



    def addCommandLineArguments( _oalListArgument ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalListArgument = ALList( _oalListArgument )
        for i in oalListArgument:
            ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( i )
        ALAppDefinitionCommandLine.synchronizeOff()
    addCommandLineArguments = staticmethod( addCommandLineArguments )


    def addListArgument( self, _oalListArgument ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalListArgument = ALList( _oalListArgument )
        for i in oalListArgument:
            ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( i )
        ALAppDefinitionCommandLine.synchronizeOff()


    def addCommandLineArgument( _oalStringToken ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalStringToken = ALToken( _oalStringToken )
        ALAppDefinitionCommandLine.m_oalCommandLine.addCommandArgument( oalStringToken )
        ALAppDefinitionCommandLine.synchronizeOff()
    addCommandLineArgument = staticmethod( addCommandLineArgument )


    def getCommandLineArgument( _oalStringToken ):
        ALAppDefinitionCommandLine.synchronizeOn()
        oalCommandLineArg = ALAppDefinitionCommandLine.m_oalCommandLine.getArgument( _oalStringToken )
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalCommandLineArg
    getCommandLineArgument = staticmethod( getCommandLineArgument )



    def getCommandLine():
        ALAppDefinitionCommandLine.synchronizeOn()
        oalCommandLine = ALAppDefinitionCommandLine.m_oalCommandLine.getCommand()
        ALAppDefinitionCommandLine.synchronizeOff()
        return oalCommandLine
    getCommandLine = staticmethod( getCommandLine )


    def existCommand( _oalStringCommand ):
        ALAppDefinitionCommandLine.synchronizeOn()
        bExist = ALAppDefinitionCommandLine.m_oalCommandLine.existCommand( _oalStringCommand )
        ALAppDefinitionCommandLine.synchronizeOff()
        return bExist
    existCommand = staticmethod( existCommand )



    def processCommandLine( self ):
        """
        This is the command line method of the ALAppDefinitionCommandLine.
        It is started Synchronized to execute the command before the kernel.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCommandLine.processCommandLine()" )
#        ALAppDefinitionCommandLine.m_oalCommandLine.executeCommand()
#        ALAppDefinitionCommandLine.m_oalCommandLine.synchronize()
        ALAppDefinitionCommandLine.m_oalCommandLine.executeCommandSynchronous()




    def processCommandLineInterface( self ):
        """
        This is the command line Interface (from xml file definition) method of the ALAppDefinitionCommandLine.
        It is started Synchronized to execute the command before the kernel.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCommandLine.processCommandLineInterface()" )
        if (ALAppDefinitionCommandLine.m_oalCommandLineInterface==None):
            return
        oalCommandLine = ALAppDefinitionCommandLine.m_oalCommandLine.getCommandLine()
        ALAppDefinitionCommandLine.m_oalCommandLineInterface.setCommandLine( oalCommandLine )
        ALAppDefinitionCommandLine.m_oalCommandLineInterface.executeCommandSynchronous()
#        ALAppDefinitionCommandLine.m_oalCommandLineInterface.executeCommand()
#        ALAppDefinitionCommandLine.m_oalCommandLineInterface.synchronize()



##############################################################################
