#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition               import *
from ALDebug                    import ALDebug
from ALSystem                   import ALSystem
from ALString                   import ALString
from ALTime                     import ALTime
from ALTrace                    import ALTrace
from ALVerbose                  import ALVerbose
from ALAppDefinitionOutput      import ALAppDefinitionOutput
from ALApplicationActionControl import ALApplicationActionControl
from ALApplicationCommunicator  import ALApplicationCommunicator



class ALAppDefinitionControl( ALAppDefinitionOutput ):
    """
    Interface for ALAppDefinitionControl objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionOutput.__init__( self, _oalString, _dversion, _obPlugin )



    def signalControl( self ):
        """
        This ALAppDefinitionControl Signal control.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionControl.signalControl()" )
        ALAppDefinitionControl.synchronizeOn( )
        oalSignalExt = ALApplicationCommunicator.getSignal()
        if (oalSignalExt != None):
            oalSignal = oalSignalExt.clone()
            ALAppDefinitionControl.m_oalSlotSignal.callSignal( oalSignal )
        ALAppDefinitionControl.synchronizeOff( )



    def __patchBugPythonThread(self):
        # workaround to avoid problem of the thread starting
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionControl.__patchBugPythonThread()" )
        ALTime.wait( 0.5 )



    def __actionControlExecutionSpeed( self, _fTimeOut ):
        """
        This ALAppDefinitionControl Action __actionControlExecutionSpeed.
        The Standard time out corresponds to: 11d:13h:46m:39s:000ms = AL_TIME_INFINITE
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionControl.__actionControlExecutionSpeed()" )
        # It must add the timeout control....in the future
        oalTime = ALTime()
        obStatusActionControl = True
        while (obStatusActionControl):
            if ( ALApplicationActionControl.isRunningControl()==True ):
                ALTime.wait( AL_TIME_MINIMUM )
                if (ALVerbose.isVerboseDebug()==True):
                    ALApplicationActionControl.getStatusRunning()
            else:
                obStatusActionControl = False
            # Controls the ALApplication ALAction time out
            # The end will be executed by the end method of the next class
            if (oalTime.getDuration()>_fTimeOut):
                obStatusActionControl = False
                ALVerbose.aalibDEBUG( "ALApplication:: ALAction-ALPlugin control time out - REACHED: " + ALString( oalTime.getDuration() ) )



    def actionControlExecution( self ):
        """
        This ALAppDefinitionControl Action Control.
        The Standard time out corresponds to: 11d:13h:46m:39s:000ms = AL_TIME_INFINITE
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionControl.actionControl()" )
        self.__patchBugPythonThread()
        self.__actionControlExecutionSpeed( self.getTimeOutPlugins() )




    def raiseControl( self ):
        """
        This ALAppDefinitionControl Raise Control.
        """
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )



##############################################################################
