#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition            import *
from ALVerbose               import ALVerbose
from ALString                import ALString
from ALSignal                import ALSignal
from ALDebug                 import ALDebug
from ALException             import ALException
from ALExceptionSignal       import ALExceptionSignal
from ALAppDefinitionEnd      import ALAppDefinitionEnd


class ALAppDefinitionCore( ALAppDefinitionEnd ):
    """
    Interface for ALAppDefinitionCore objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionEnd.__init__( self, _oalString, _dversion, _obPlugin )



    def __executeCoreInit( self, _bInit ):
        """
        This is the Initial Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreInit()" )
        oBoolStatus = self.init( _bInit )
        if (oBoolStatus==False):
            return oBoolStatus
        self.signalControl()
        self.callbackSlotInit( self )
        return oBoolStatus


    def __executeCoreCommandLine( self ):
        """
        This is the CommandLine Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreCommandLine()" )
        # Process Sequence
        self.signalControl()
        self.processCommandLine()
        self.signalControl()
        self.processCommandLineInterface()



    def __executeCorePreProcess( self ):
        """
        This is the PreProcessing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCorePreProcess()" )
        # PreProcess Sequence
        self.signalControl()
        self.preProcess()
        self.signalControl()
        self.callbackSlotPreProcess( self )



    def __executeCoreProcess( self ):
        """
        This is the Processing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreProcess()" )
        # Process Sequence
        self.signalControl()
        self.process()
        self.signalControl()
        self.callbackSlotProcess( self )


    def __executeCorePostProcess( self ):
        """
        This is the Post Processing Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCorePostProcess()" )
        # PostProcess Sequence
        self.signalControl()
        self.postProcess()
        self.signalControl()
        self.callbackSlotPostProcess( self )



    def __executeCoreEnd( self ):
        """
        This is the Final Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreEnd()" )
        # End
        self.signalControl()
        self.actionControlExecution()
        self.callbackSlotEnd( self )


    def __executeCoreClose( self ):
        """
        This is the Final Close Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreClose()" )
        # Closing
        self.endApplication()


    def __executeCoreRaise( self ):
        """
        This is the Raise Core Element started by the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCoreRaise()" )
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def __executeCore( self ):
        """
        This is the Main Core Element of the ALAppDefinitionCore.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.__executeCore()" )
        self.__executeCoreCommandLine()
        self.__executeCorePreProcess()
        self.__executeCoreProcess()
        self.__executeCorePostProcess()
        self.__executeCoreEnd()



############################################################################

    def preProcess( self ):
        """
        Any processing that must occur before the main ALAppDefinitionCore loop
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.preProcess()" )
        ALAppDefinitionCore.synchronizeOff()


    def process( self ):
        """
        This is the main method of the ALAppDefinitionCore.
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.process()" )
        ALAppDefinitionCore.synchronizeOff()


    def postProcess( self ):
        """
        This is the final method executed by the ALAppDefinitionCore
        """
        ALAppDefinitionCore.synchronizeOn()
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.postProcess()" )
        ALAppDefinitionCore.synchronizeOff()


##############################################################################


    def execute( self, _bInit = True ):
        """
        This is the Main ALAppDefinitionCore Loop Control.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.execute()" )
        try:
            try:
                if (self.__executeCoreInit( _bInit )==True):
                    self.__executeCore()


            except ALExceptionSignal, _oalExceptionSignal:
                oalSignal = _oalExceptionSignal.getSignal()
                ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.execute() - AL_FRAMEWORK_FORCE_CLOSE_APPLICATION" )
                if ( oalSignal!=None ):
                    ALVerbose.aalibDEBUG("ALAppDefinitionCore.execute() - stopped by Signal: "  + ALString( oalSignal.getSignal() ) )
                else:
                    ALVerbose.aalibDEBUG("ALAppDefinitionCore.execute() - stopped under Unknown signal: "  + ALString( oalSignal.getSignal() ) )
                self.__executeCoreRaise()


            except:
                ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionCore.execute() - UNEXPECTED CONTROL RAISED" )
                self.__executeCoreRaise()

        finally:
            self.__executeCoreClose()


##############################################################################
