#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALDefinition               import *
from ALImportPython             import PySys
from ALVerbose                  import ALVerbose
from ALString                   import ALString
from ALTime                     import ALTime
from ALLog                      import ALLog
from ALSystem                   import ALSystem
from ALSignal                   import ALSignal
from ALExceptionSignal          import ALExceptionSignal
from ALAppDefinitionInit        import ALAppDefinitionInit
from ALApplicationActionControl import ALApplicationActionControl


class ALAppDefinitionEnd( ALAppDefinitionInit ):
    """
    Interface for ALAppDefinitionEnd objects.

    """
    m_oBoolTerminateApplication = False

    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionInit.__init__( self, _oalString, _dversion, _obPlugin )

        ALAppDefinitionEnd.m_oalSlotSignal.connect( self.exit, ALSignal.error() )
        ALAppDefinitionEnd.m_oalSlotSignal.connect( self.exit, ALSignal.exception() )
        ALAppDefinitionEnd.m_oalSlotSignal.connect( self.exit, ALSignal.exit() )
        ALAppDefinitionEnd.m_oalSlotSignal.connect( self.endApplication, ALSignal.terminateApplication() )




    def __forceExit( self, _iArg = 1 ):
        """
        This ALAppDefinitionEnd __forceExit
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionEnd.__forceExit()" )
        raise ALExceptionSignal, ALSignal.exit()




    def __forceCloseTimerActions( self ):
        """
        This ALAppDefinitionEnd force Close Timer Actions.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionEnd.__forceCloseTimerActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        for iAction in ALAppDefinitionEnd.m_oalListTimerActions:
            iAction.close()
        ALAppDefinitionEnd.synchronizeOff()


    def __forceCloseActions( self ):
        """
        This ALAppDefinitionEnd force Close All Actions.
        """
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionEnd.__forceCloseActions()" )
        ALAppDefinitionEnd.synchronizeOn()
        ALApplicationActionControl.forceClose()
        ALAppDefinitionEnd.synchronizeOff()



    def exit( self, _oalObject = None ):
        """
        This ALAppDefinitionEnd calling force exit.
        """
        # patch to avoid multiples signal calls to end the application
        if (ALAppDefinitionEnd.m_oBoolTerminateApplication == True):
            return
        if (ALAppDefinitionEnd.m_oBoolTerminateApplication == False):
            ALAppDefinitionEnd.m_oBoolTerminateApplication = True
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionEnd.exit() - calling  __forceExit" )
        if (_oalObject != None and isinstance( _oalObject, ALSignal )):
            oalSignal = _oalObject.getSignal()
            oalString = _oalObject.getInformation()
            if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ) :
                ALVerbose.error( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
            elif ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT  ) :
                ALVerbose.error( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
            else:
                ALVerbose.log( "Application::" + ALString( oalSignal.getSignal() ) + "::" + oalString )
        self.__forceExit()




    def __getActionLog( self ):
        ALApplicationActionControl.getActionLogInformation()



    def end( self, _oalObject = None ):
        """
        End The application framework...closing and registering the resources
        """
        ALAppDefinitionEnd.synchronizeOff()
        self.__forceCloseActions()
        ALAppDefinitionEnd.synchronizeOff()
        self.__forceCloseTimerActions()
        ALAppDefinitionEnd.synchronizeOff()
        ALAppDefinitionEnd.setResource( AL_FRAMEWORK_LAST_RUN, ALTime.timeString() )
        ALAppDefinitionEnd.outputResource()
        self.__getActionLog()



        ofAppRunTime     = self.getRunTime()
        ALAppDefinitionEnd.synchronizeOff()
        oalStrAppName    = ALAppDefinitionEnd.getApplicationName()
        oalStrAppRunTime = ALString( ofAppRunTime )
        ALLog.setRuntime( ofAppRunTime )
        ALVerbose.outputActionLog()
        ALVerbose.screen( oalStrAppName + "::Closing" )
        ALVerbose.screen( oalStrAppName + "::Runtime: " + oalStrAppRunTime + " [s]" )
        ALVerbose.screen( oalStrAppName + "::Runtime: " + ALTime.getTimeStringPSP( ofAppRunTime ) )


        # Used to generate a XML profile log file to analise the performance
        ALAppDefinitionEnd.outputProfile()



    def endApplication( self, _oalObject = None ):
        ALVerbose.aalibDEBUG( "Execution: ALAppDefinitionEnd.endApplication()" )
        # patch to avoid multiples signal calls to end the application
        if (ALAppDefinitionEnd.m_oBoolTerminateApplication == False):
            ALAppDefinitionEnd.m_oBoolTerminateApplication = True
            self.end()


    def terminateApplication( _oalObject = False ):
        ALAppDefinitionEnd.setSignal( ALSignal.terminateApplication() )
        if (_oalObject==False):
            PySys.exit() #Bug in Python 2.4 - it raises an exception !!!!!!!!!!
    terminateApplication = staticmethod( terminateApplication )



##############################################################################
