#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALDefinition                    import *
from ALVerbose                       import ALVerbose
from ALString                        import ALString
from ALDebug                         import ALDebug
from ALAppDefinitionCommandLine      import ALAppDefinitionCommandLine


class ALAppDefinitionOutput( ALAppDefinitionCommandLine ):
    """
    Interface for ALAppDefinitionOutput objects.

    """
    def __init__( self, _oalString = AL_FRAMEWORK_APPLICATION_NAME, _dversion = AL_VERSION, _obPlugin = True ):
        ALAppDefinitionCommandLine.__init__( self, _oalString, _dversion, _obPlugin )
        self.connectCommandLineBasis()


    def connectCommandLineBasis( self ):
        # Tricks for the command line help
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin", ALAppDefinitionOutput.outputScreenPlugin )
        # Tricks for the command line help - new standart comand Line
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin-group", ALAppDefinitionOutput.outputScreenPluginGroup )
        ALAppDefinitionOutput.m_oalCommandLine.addCommand( "--plugin-group-tree", ALAppDefinitionOutput.outputScreenPluginGroupTree )


    def outputScreenPlugin( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
#        Bug in Jython version for this command - to verify ???
#        if (ALAppDefinitionOutput.m_oalFactoryPlugin!=None):
#            ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPlugin()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPlugin()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPlugin = staticmethod( outputScreenPlugin )


    def outputScreenPluginHelp( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        bStatus = ALAppDefinitionOutput.m_oalFactoryPlugin.processPluginContainingStringName( "Help" )
        ALAppDefinitionOutput.synchronizeOff()
        return bStatus
    outputScreenPluginHelp = staticmethod( outputScreenPluginHelp )


    def outputScreenPluginMan( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        bStatus = ALAppDefinitionOutput.m_oalFactoryPlugin.processPluginContainingStringName( "Man" )
        ALAppDefinitionOutput.synchronizeOff()
        return bStatus
    outputScreenPluginMan = staticmethod( outputScreenPluginMan )


    def outputScreenPluginGroup( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPluginGroup()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPluginGroup = staticmethod( outputScreenPluginGroup )


    def outputScreenPluginGroupTree( _oalObject = None ):
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalFactoryPlugin.outputScreenPluginGroupTree()
        ALAppDefinitionOutput.synchronizeOff()
    outputScreenPluginGroupTree = staticmethod( outputScreenPluginGroupTree )




    def outputVersion():
        ALAppDefinitionOutput.synchronizeOn()
        ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Version " + ALAppDefinitionOutput.m_oalApplicationVersion )
        ALAppDefinitionOutput.synchronizeOff()
    outputVersion = staticmethod( outputVersion )



    def ouputOption():
        ALAppDefinitionOutput.synchronizeOn()
        ALVerbose.screen( "" )
        ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Options" )
        ALVerbose.screen( "Standard Options:" )
        ALVerbose.screen( "" )
        ALVerbose.screen( "            --help or -h  [option] : to show help if available" )
        ALVerbose.screen( "            --man  or -m  [option] : to show manual if available" )
        ALVerbose.screen( "                      --help --all : it shows the main command list help" )
        ALVerbose.screen( "                       --man --all : it shows the main command list man" )
        ALVerbose.screen( "                         --verbose : to iterative output" )
        ALVerbose.screen( "                           --quiet : NO output (default option)" )
        ALVerbose.screen( "                          --no-log : NO log file output" )
        ALVerbose.screen( "                         --profile : Massive Parallel profile output for trace actions" )
        ALVerbose.screen( "                   --version or -v : executable version info" )
        ALVerbose.screen( "                          --plugin : show the plug-in connected to the application" )
        ALVerbose.screen( "                    --plugin-group : show the plug-in groups" )
        ALVerbose.screen( "               --plugin-group-tree : show the plug-in group tree" )
        ALVerbose.screen( "                           --DEBUG : output debug info with no stops" )
        ALVerbose.screen( "                 --DEBUG-TAG [tag] : output only user Tag debug - all, aalib or others" )
        ALVerbose.screen( "                     --DEBUG-FORCE : output default Python-Java debug info and" )
        ALVerbose.screen( "                                     FORCE the application to stop" )
        ALVerbose.screen( "              --DEBUG-FORCE-PLUGIN : output default Python-Java debug only from a PLUGIN and " )
        ALVerbose.screen( "                                     FORCE the application to stop" )
        ALVerbose.screen( "                          --ASSERT : output assert info for debug" )
        ALVerbose.screen( "      --import-resource [filename] : import a file application resource file" )
        ALVerbose.screen( "                       --systemLog : output in the log file informations about the system" )
        ALVerbose.screen( "                         --license : output the PyAALib-JyAALib Library license" )
        ALVerbose.screen( "" )
        ALAppDefinitionOutput.synchronizeOff()
    ouputOption = staticmethod( ouputOption )



    def outputHelp():
       
        if (ALAppDefinitionOutput.m_oalHelp != None):
            ALAppDefinitionOutput.m_oalHelp.outputScreen()

        if( ALAppDefinitionOutput.getCommandLineArgument( "--help" )!=None ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "--help" )
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Help option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringHelp ) )
        elif( ALAppDefinitionOutput.getCommandLineArgument( "-h" )!=None  ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "-h" )
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Help option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringHelp ) )
        else:
            oalStringHelp = ALAppDefinitionOutput.m_oalCommandLineInterface.getHelp( )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalStringHelp ) )

        ALAppDefinitionOutput.ouputOption()
        if (ALAppDefinitionOutput.getCommandLineArgument( "--console" )==None ):
            ALAppDefinitionOutput.outputScreenPluginHelp()
    outputHelp = staticmethod( outputHelp )



    def outputMan():
        if (ALAppDefinitionOutput.m_oalMan != None):
            ALAppDefinitionOutput.m_oalMan.outputScreen()

        if( ALAppDefinitionOutput.getCommandLineArgument( "--man" )!=None ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "--man" )
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Manual option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringManual ) )
        elif( ALAppDefinitionOutput.getCommandLineArgument( "-m" )!=None  ):
            oalString = ALAppDefinitionOutput.getCommandLineArgument( "-m" )
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( oalString )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALAppDefinitionOutput.m_oalApplicationName + "::Manual option" )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalString + ": " + oalStringManual ) )
        else:
            oalStringManual = ALAppDefinitionOutput.m_oalCommandLineInterface.getManual( )
            ALVerbose.screen( "" )
            ALVerbose.screen( ALString( oalStringManual ) )

        ALAppDefinitionOutput.ouputOption()
        if (ALAppDefinitionOutput.getCommandLineArgument( "--console" )==None ):
            ALAppDefinitionOutput.outputScreenPluginMan()

    outputMan = staticmethod( outputMan )


    # Output control ==========================================================
    def setDebugMode():
        ALAppDefinitionOutput.synchronizeOn( )
        ALAppDefinitionOutput.m_oalDebug = True
        ALVerbose.setVerboseDebugOn()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugMode = staticmethod( setDebugMode )


    def setDebugModeON():
        ALAppDefinitionOutput.setDebugMode()
    setDebugModeON = staticmethod( setDebugModeON )


    def setDebugModeOFF():
        ALAppDefinitionOutput.synchronizeOn()
        ALAppDefinitionOutput.m_oalDebug = False
        ALVerbose.setVerboseDebugOff()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugModeOFF = staticmethod( setDebugModeOFF )



    def isDebugMode():
        ALAppDefinitionOutput.synchronizeOn()
        bDebug = ALAppDefinitionOutput.m_oalDebug
        ALAppDefinitionOutput.synchronizeOff()
        return bDebug
    isDebugMode = staticmethod( isDebugMode )


    def setDebugProfileOn():
        ALAppDefinitionOutput.synchronizeOn( )
        ALVerbose.setProfileOn()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugProfileOn = staticmethod( setDebugProfileOn )


    def setDebugProfileOff():
        ALAppDefinitionOutput.synchronizeOn( )
        ALVerbose.setProfileOff()
        ALAppDefinitionOutput.synchronizeOff( )
    setDebugProfileOff = staticmethod( setDebugProfileOff )


    def setDebugVerboseOn():
        ALAppDefinitionOutput.synchronizeOn()
        ALDebug.setDebugModeVerboseOn()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugVerboseOn = staticmethod( setDebugVerboseOn )


    def setDebugPluginOn():
        ALAppDefinitionOutput.synchronizeOn()
        ALDebug.setDebugModeActionOn()
        ALAppDefinitionOutput.synchronizeOff()
    setDebugPluginOn = staticmethod( setDebugPluginOn )

##############################################################################
