#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDictionary             import ALDictionary
from ALInformationAction      import ALInformationAction
from ALObject                 import ALObject
from ALSemaphore              import ALSemaphore
from ALString                 import ALString
from ALVerbose                import ALVerbose


class ALApplicationActionControl( ALObject ):
    """
    Parent class.
    """

    m_oalDictAction              = ALDictionary()
    m_oalDictActionFinished      = ALDictionary()
    m_oalSemaphoreActionControl  = ALSemaphore()
    m_oalNumberActions           = 0
    m_oalNumberActionsRunning    = 0
    m_oalNumberActionsFinished   = 0


    def __init__( self,  ):
        ALObject.__init__( self )


    def reset():
        ALApplicationActionControl.synchronizeOn()
        del ALApplicationActionControl.m_oalDictAction
        del ALApplicationActionControl.m_oalDictActionFinished
        ALApplicationActionControl.m_oalDictAction              = ALDictionary()
        ALApplicationActionControl.m_oalDictActionFinished      = ALDictionary()
        ALApplicationActionControl.m_oalNumberActions           = 0
        ALApplicationActionControl.m_oalNumberActionsRunning    = 0
        ALApplicationActionControl.m_oalNumberActionsFinished   = 0
        ALApplicationActionControl.synchronizeOff()
    reset = staticmethod( reset )


    def synchronizeOn():
        ALApplicationActionControl.m_oalSemaphoreActionControl.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        ALApplicationActionControl.m_oalSemaphoreActionControl.release()
    synchronizeOff = staticmethod( synchronizeOff )


    def startAction( _oalAction ):
        ALApplicationActionControl.synchronizeOn()
        ALApplicationActionControl.m_oalNumberActions += 1
        oalInformationAction = ALInformationAction( _oalAction.getID(), _oalAction )
        ALApplicationActionControl.m_oalDictAction.addObject( oalInformationAction.getActionID(), oalInformationAction )
        ALApplicationActionControl.synchronizeOff()
    startAction = staticmethod( startAction )


    def endAction( _oalAction ):
        ALApplicationActionControl.synchronizeOn()
        oalInfoAction = ALApplicationActionControl.m_oalDictAction.getObject( _oalAction.getID() )
        if (oalInfoAction!=None):
            oalInfoAction.setActionEnd()
            ALApplicationActionControl.m_oalNumberActionsFinished += 1
            ALApplicationActionControl.m_oalDictActionFinished.addObject( _oalAction.getID(), oalInfoAction.getRunTime() )
            ALApplicationActionControl.m_oalDictAction.removeObject( _oalAction.getID() )
#        else:
#            ALVerbose.warning( "Object ALInformationAction doesn't exist" )
        ALApplicationActionControl.synchronizeOff()
    endAction = staticmethod( endAction )



    def getNumberActiveActions():
        ALApplicationActionControl.synchronizeOn()
        oiActive = ALApplicationActionControl.m_oalNumberActions
        oiActive = oiActive - ALApplicationActionControl.m_oalNumberActionsFinished
        ALApplicationActionControl.synchronizeOff()
        return oiActive
    getNumberActiveActions = staticmethod( getNumberActiveActions )


    def isRunningControl():
        obStatus = False
        if (ALApplicationActionControl.getNumberActiveActions()!=0):
            obStatus = True
        return obStatus
    isRunningControl = staticmethod( isRunningControl )



    def forceClose():
        ALVerbose.aalibDEBUG( "ALApplicationActionControl::forceClose()" )
        ALApplicationActionControl.synchronizeOn()
        for iAction in ALApplicationActionControl.m_oalDictAction.ordered():
            oalInfo = ALApplicationActionControl.m_oalDictAction[ iAction ]
            oalInfo.forceActionClose()
        ALApplicationActionControl.synchronizeOff()


    forceClose = staticmethod( forceClose )



    def getStatusRunning():
        iActionsTotal = ALApplicationActionControl.m_oalNumberActions
        iActions      = ALApplicationActionControl.getNumberActiveActions()
        ALVerbose.aalibDEBUG( "   Total Actions: " + ALString( iActionsTotal ) )
        ALVerbose.aalibDEBUG( " Running Actions: " + ALString( iActions ) )
    getStatusRunning = staticmethod( getStatusRunning )


    def getActionInformation( _oalActionID ):
        ALApplicationActionControl.synchronizeOn()
        oalInfo = None
        try:
            oalInfo = ALApplicationActionControl.m_oalDictAction.getObject( _oalActionID )
        except:
            pass
        ALApplicationActionControl.synchronizeOff()
        return oalInfo
    getActionInformation = staticmethod( getActionInformation )




    def getActionLogInformation():
        ALApplicationActionControl.synchronizeOn()
        if(ALVerbose.isVerboseDebug()==True):
            ALVerbose.aalibDEBUG( "                [Total Actions] - " + ALString( ALApplicationActionControl.m_oalNumberActions ) )
            for iAction in ALApplicationActionControl.m_oalDictActionFinished.ordered():
                ofTime = ALApplicationActionControl.m_oalDictActionFinished[ iAction ]
                ALVerbose.aalibDEBUG( "                [" + iAction +"] - Run Time [s]: " + ALString( ofTime ) )
        ALApplicationActionControl.synchronizeOff()
        ALApplicationActionControl.reset()
    getActionLogInformation = staticmethod( getActionLogInformation )


##############################################################################


