#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALString                  import ALString
from ALVerbose                 import ALVerbose
from ALSystem                  import ALSystem
from ALFile                    import ALFile
from ALCommandLine             import ALCommandLine
from ALXsd                     import ALXsd
from ALDiskExplorer            import ALDiskExplorer
from ALDictionary              import ALDictionary
from ALDebug                   import ALDebug


try:
    import ALCommandLineInterfaceDefinition  as ALCommandLineInterfaceDefinition
    import ALCommandLineInterfaceElement     as ALCommandLineInterfaceElement
except ImportError:
    try:
        # You must run this module on the m/modules folder to generate the specific code
        ALXsd().generateModule( "ALCommandLineInterfaceDefinition.py", "ALCommandLineInterfaceDefinition.xsd", "xsd:" )
        ALXsd().generateModule( "ALCommandLineInterfaceElement.py", "ALCommandLineInterfaceElement.xsd", "xsd:" )
    except:
        ALVerbose.warning( "Problems to generate class interfaces for: ALCommandLineInterfaceElement and ALCommandLineInterfaceDefinition" )


class ALCommandLineInterface( ALCommandLine ):
    """
    Base class for command line interface objects.
    """
    def __init__( self, _oStrXmlFileName = None, _oBoolFactoryPluginDynamic = True ):
        ALCommandLine.__init__( self )
        self.m_oalDictCmdInterface       = ALDictionary()
        self.m_oStrXmlFileName           = _oStrXmlFileName
        self.m_oBoolFactoryPluginDynamic = _oBoolFactoryPluginDynamic
        self.createCommandLine( self.m_oStrXmlFileName )


    def createCommandLine( self, _oStrXmlFileName ):
        if (_oStrXmlFileName==None or _oStrXmlFileName==ALString.empty()) :
            ALVerbose.aalibDEBUG( "No CommandLineInterface generated :: [ Empty file name ]" )
            return
        if (ALDiskExplorer.existFile( _oStrXmlFileName )==False):
            ALVerbose.aalibDEBUG( "[NO] CommandLineInterface Generated From File :: [" + ALString( _oStrXmlFileName ) + "]" )
            ALVerbose.aalibDEBUG( "                       The file doesn't exist :: [" + ALString( _oStrXmlFileName ) + "]" )
            return

        ALVerbose.aalibDEBUG( "[OK] CommandLineInterface Generated From File :: [" + ALString( _oStrXmlFileName ) + "]" )

        oalCmdLineDefinition = ALCommandLineInterfaceDefinition.unMarshalling( _oStrXmlFileName )
        oalList = oalCmdLineDefinition.getCommandLineElement()

        for iElem in oalList:
            self.m_oalDictCmdInterface.addObject( iElem.getCommand(), iElem )

        self.__createCommandLineCallBack()



    def __createCommandLineCallBack( self ):
        for iElem in self.m_oalDictCmdInterface.ordered():
            oalCmdElement = self.m_oalDictCmdInterface[iElem]
            oalCmdClass = oalCmdElement.getClass()
            if (oalCmdClass!=ALString.empty()):
                self.__importClassObjectCallBack( oalCmdElement )
            else:
                self.__importPluginObjectCallBack( oalCmdElement )



    def __importClassObjectCallBack( self, _oalCmdElement ):
        if (self.m_oBoolFactoryPluginDynamic==False):
            return
        oalCmdName          = _oalCmdElement.getCommand()
        oalCmdClass         = _oalCmdElement.getClass()
        oalCmdClassMethod   = _oalCmdElement.getClassMethod()
        oalCmdClassFileName = ALDiskExplorer.normPath( _oalCmdElement.getClassFileName() )

        self.__importClassObject( oalCmdName, oalCmdClass, oalCmdClassMethod, oalCmdClassFileName )



    def __importClassObject( self, _oalCmdName, _oalCmdClass, _oalCmdClassMethod, _oalCmdClassFileName ):
        if (self.m_oBoolFactoryPluginDynamic==False):
            return
        oalCmdName          = _oalCmdName
        oalCmdClass         = _oalCmdClass
        oalCmdClassMethod   = _oalCmdClassMethod
        oalCmdClassFileName = _oalCmdClassFileName

        if (oalCmdClassFileName==ALString.empty()):
            oalCmdClassFileName = oalCmdClass + ".py"

        if (ALDiskExplorer.existFile( oalCmdClassFileName )==False):
            ALVerbose.aalibDEBUG( "        The Defined Plugin File doesn't exist :: [" + oalCmdClassFileName + "]" )
            return

        if ALFile.isExtension( oalCmdClassFileName.upper(), ".PY" ):
            oalPath =  ALFile.getFolderName( oalCmdClassFileName )
            if (oalPath!=ALString.empty()):
                ALSystem.setPath( oalPath )
            oNameModule = ALFile.getBaseName( oalCmdClassFileName )
            try:
                oModule = __import__( oNameModule )
                oBoolStatus = False
                for iKey in oModule.__dict__:
                    if (iKey==oalCmdClass):
                        oBoolStatus = True
                if (oBoolStatus==True):
                    try:
                        oClass  = oModule.__dict__[ oalCmdClass ]()
                        oMethod = getattr( oClass, oalCmdClassMethod )
                        self.addCommand( oalCmdName, oMethod )
                    except:
                        self.__raiseControl()
                else:
                    ALVerbose.aalibDEBUG( "The defined class was not found :: [" + oNameModule + "]" )
            except:
                ALVerbose.aalibDEBUG( "Problem Compiling and Loading Class :: [" + oalCmdClassFileName + "]" )
                ALVerbose.aalibDEBUG( "Please DEBUG the plug-in code to fix the problem." )
                self.__raiseControl()

        else:
             ALVerbose.aalibDEBUG( "Not valid Python file: " + oalCmdClassFileName )



    def __importPluginObjectCallBack( self, _oalCmdElement ):
        oalCmdName           = _oalCmdElement.getCommand()
        oalCmdPlugin         = _oalCmdElement.getPlugin()
        oalCmdPluginMethod   = _oalCmdElement.getPluginMethod()
        oalCmdPluginFileName = _oalCmdElement.getPluginFileName()
        if (oalCmdPluginFileName!=ALString.empty()):
            oalCmdPluginFileName = ALDiskExplorer.normPath( _oalCmdElement.getPluginFileName() )

        if (oalCmdPluginMethod==ALString.empty()):
            oalCmdPluginMethod = ALString( "executePlugin" )
        
        from ALAppDefinition  import ALAppDefinition
        oalPlugin = None
       # oalPlugin = ALApplication.getPluginObject( oalCmdPlugin )
        oalPlugin = ALAppDefinition.getPluginObject( oalCmdPlugin )

        if (oalPlugin == None):
            self.__importClassObject( oalCmdName, oalCmdPlugin, oalCmdPluginMethod, oalCmdPluginFileName )
            return


        if (oalPlugin!=None):
#            if (oalCmdPluginMethod!=ALString.empty()):
#                oMethod = getattr( oalPlugin, oalCmdPluginMethod )
#            else:
#                oMethod = oalPlugin.executePlugin
            if (oalCmdPluginMethod!=ALString.empty()):
                if (oalCmdPluginMethod=="executePlugin"):
                    #print oalCmdPlugin
                    oMethod = oalPlugin.executePlugin
                elif (oalCmdPluginMethod=="executePluginSynchronous"):
                    oMethod = oalPlugin.executePluginSynchronous
                else:
                    oMethod = getattr( oalPlugin, oalCmdPluginMethod )
            else:
                oMethod = oalPlugin.executePlugin
            self.addCommand( oalCmdName, oMethod )
        else:
            ALVerbose.aalibDEBUG( "Problem Loading plug-in :: [ " + ALString( oalCmdPlugin ) + "]" )


    def getHelp( self, oalStrCommand = None ):
        oalString = ALString( "" )
        if ( oalStrCommand == None):
            for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStringHelp = self.m_oalDictCmdInterface[ oalStrCommand ].getHelp()
                if (oalStringHelp!=ALString.empty()):
                    oalString = oalString + oalStringHelp
        elif ( oalStrCommand == "--all" ):
            for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStrCmdDescription = self.m_oalDictCmdInterface[ oalStrCommand ].getCommandDescription()
                oalStrCmd = self.m_oalDictCmdInterface[ oalStrCommand ].getCommand()
                if (oalStrCmdDescription==ALString.empty()):
                    oalStrCmdDescription = "No Command Description Avaiable"
                if (oalStrCmd!=ALString.empty()):
                    oalString = oalString + ALString( "\n          " + oalStrCmd + " : " +  oalStrCmdDescription )
        else:
            if (oalStrCommand in self.m_oalDictCmdInterface):
                oalStringHelp = self.m_oalDictCmdInterface[ oalStrCommand ].getHelp()
                if (oalStringHelp!=ALString.empty()):
                    oalString = oalStringHelp
        return oalString


    def getManual( self, oalStrCommand = None ):
        oalString = ALString( "" )
        if ( oalStrCommand == None):
            for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStringManual = self.m_oalDictCmdInterface[ oalStrCommand ].getManual()
                if (oalStringManual!=ALString.empty()):
                    oalString = oalString + oalStringManual
        elif ( oalStrCommand == "--all" ):
            for oalStrCommand in self.m_oalDictCmdInterface.getSequenceOrdered():
                oalStrCmdDescription = self.m_oalDictCmdInterface[ oalStrCommand ].getCommandDescription()
                oalStrCmd = self.m_oalDictCmdInterface[ oalStrCommand ].getCommand()
                if (oalStrCmdDescription==ALString.empty()):
                    oalStrCmdDescription = "No Command Description Available"
                if (oalStrCmd!=ALString.empty()):
                    oalString = oalString + ALString( "\n          " + oalStrCmd + " : " +  oalStrCmdDescription )
        else:
            if (oalStrCommand in self.m_oalDictCmdInterface):
                oalStringManual = self.m_oalDictCmdInterface[ oalStrCommand ].getManual()
                if (oalStringManual!=ALString.empty()):
                    oalString = oalStringManual
        return oalString


    def getWebAdress( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getWebAdress()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getWebUpDate( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getWebUpDate()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getCopyright( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getCopyright()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getAuthor( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getAuthor()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


    def getDate( self, oalStrCommand ):
        oalString = None
        if oalStrCommand in self.m_oalDictCmdInterface:
            oalStringData = self.m_oalDictCmdInterface[ oalStrCommand ].getDate()
            if (oalStringData!=ALString.empty()):
                oalString = oalStringData
        return oalString


##############################################################################
