#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDebug           import ALDebug
from ALDictionary      import ALDictionary
from ALDiskExplorer    import ALDiskExplorer
from ALFactory         import ALFactory
from ALFile            import ALFile
from ALList            import ALList
from ALPlugin          import ALPlugin
from ALString          import ALString
from ALSystem          import ALSystem
from ALTrace           import ALTrace
from ALVerbose         import ALVerbose
from ALXsd             import ALXsd


class ALFactoryPlugin( ALFactory ):
    """
    Interface for plugin factory classes.
    """
    def __init__( self ):
        ALFactory.__init__( self )
        self.m_oalPluginFolder    = None
        self.m_oalPluginFile      = None
        self.m_oalDictPluginGroup = ALDictionary()


    def __raiseControl( self ):
        """
        This ALFactoryPlugin Raise Control.
        """
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def __addPluginGroup( self, _oGroup ):
        oGroup = ALString( _oGroup )
        self.m_oalDictPluginGroup.addObject( oGroup, oGroup )



    def __getGroupName( self, _oFile,  _oalStrFolderPluginRoot ):
        oGroupName = _oalStrFolderPluginRoot
        oFile = _oFile
        if (oGroupName!=None):
            oFile = oFile.replace( oGroupName, "" )
        oGroupList = oFile.split( "/" )
        oGroupName = oGroupList[1]
        if ALFile.isExtension( oGroupName.upper(), ".PY" ):
            oGroupName = ALFile.getBaseName( oGroupName )
        return oGroupName




    def __addPlugin( self, _oalString, _oalStrFolderPluginRoot = None):
        ifile = ALString( _oalString )
        if ALDiskExplorer.isFile( ifile ):
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                oNameModule = ALFile.getBaseName( ifile )
                try:
                    oModule = __import__( oNameModule )
                    oBoolStatus = False
                    for iKey in oModule.__dict__:
                        if (iKey==oNameModule):
                            oBoolStatus = True
                    if (oBoolStatus==True):
                        try:
#                            # only to test the import failure of the plug-in = it will be deleted...
#                            oPlugin = oModule.__dict__[ oNameModule ]()
                            oGroupName = self.__getGroupName( ifile, _oalStrFolderPluginRoot )
                            self.__addPluginGroup( oGroupName )
                            self.synchronizeOff()
                            self.addObject( oNameModule, oModule )
                            self.synchronizeOn()
                        except:
                            self.__raiseControl()
                    else:
                        ALVerbose.aalibDEBUG( "It's not an AALib Standard Plug-in :: [" + ifile + "]" )
                        ALVerbose.aalibDEBUG( " The standard class was not found :: [" + oNameModule + "]" )
                except:
                    ALVerbose.aalibDEBUG( "Problem Compiling and Loading Plug-in :: [" + ifile + "]" )
                    ALVerbose.aalibDEBUG( "Please DEBUG the plug-in code to fix the problem." )
                    self.__raiseControl()
        else:
             ALVerbose.aalibDEBUG( "ALFactoryPlugin - Error loading plug-in :: [" + ifile + "]" )





    def __addPluginXsd( self, _oalString, _oalStrFolderPluginRoot = None):
        ifile = ALString( _oalString )
        if ALDiskExplorer.isFile( ifile ):
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                try:
                    oNameModule = ALFile.getBaseName( ifile )

                    oalStrPyOutputFile = ALFile.setExtension( ifile, ".py" )
                    oalStrXsdInputFile = ifile

                    try:
                        ALXsd().generateModuleBuffer( oalStrPyOutputFile, oalStrXsdInputFile )
                    except:
                        self.__raiseControl()

                    oModule = __import__( oNameModule )
                    self.synchronizeOff()
                    self.addObject( oNameModule, oModule )
                    self.synchronizeOn()
                except:
                    ALVerbose.aalibDEBUG( "ALFactoryPluginXSD - Error importing module :: [" + ifile + "]" )
                    self.__raiseControl()
        else:
            ALVerbose.aalibDEBUG( "ALFactoryPluginXSD - Error loading plug-in :: [" + ifile + "]" )



    def __addPluginInteract( self,  _pyArg, _pyFolder, _pyFiles ):
        oalStrCurFolder       = ALDiskExplorer.normPath( _pyFolder )
        oalListFile           = ALList( _pyFiles )
        oalStrFolderPluginRoot= ALString( _pyArg )
        for ifile in oalListFile:
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
#                    ALSystem.setPath( self.m_oalPluginFolder )
                oFile = ALDiskExplorer.mergePath( oalStrCurFolder, ifile )
                self.__addPlugin( oFile, oalStrFolderPluginRoot )
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
#                    ALSystem.setPath( self.m_oalPluginFolder )
                oFile = ALDiskExplorer.mergePath( oalStrCurFolder, ifile )
                self.__addPluginXsd( oFile, oalStrFolderPluginRoot )



    def __addPluginPathModuleInteract( self,  _pyArg, _pyFolder, _pyFiles ):
        oalStrCurFolder       = ALDiskExplorer.normPath( _pyFolder )
        oalListFile           = ALList( _pyFiles )
#        oalStrFolderPluginRoot= ALString( _pyArg )
        for ifile in oalListFile:
            if ALFile.isExtension( ifile.upper(), ".PY" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
                    ALSystem.setPath( self.m_oalPluginFolder )
            if ALFile.isExtension( ifile.upper(), ".XSD" ):
                if ( self.m_oalPluginFolder != oalStrCurFolder ):
                    self.m_oalPluginFolder = oalStrCurFolder
                    ALSystem.setPath( self.m_oalPluginFolder )



    def addPluginFolder( self, _oalString ):
        self.synchronizeOn()
        oalStatus = True
        if (ALDiskExplorer.existFolder( _oalString )):
            # Separate procedure to import modules from other folders in the plug-in
            # directory...
            # First add all folders to the modules import PATH
            ALDiskExplorer.iteractPath( ALDiskExplorer.normPath( _oalString ),
                                   self.__addPluginPathModuleInteract, ALDiskExplorer.normPath( _oalString ) )
            # and loads the plugins
            ALDiskExplorer.iteractPath( ALDiskExplorer.normPath( _oalString ),
                                   self.__addPluginInteract, ALDiskExplorer.normPath( _oalString ) )
        else:
            ALVerbose.aalibDEBUG( "[NO] Standard Plug-in Folder NOT Found :: [" + ALString( _oalString ) +"]" )
            oalStatus = False
        self.synchronizeOff()
        return oalStatus



    def addPluginFolderFromPath( self, _oalString ):
        self.synchronizeOn()
        oalStatus = False
        oalList =  ALSystem.getSystemPath()
        oalAppName = ALFile.getFileName( _oalString )
        for iFolder in oalList:
            oalFolder = ALDiskExplorer.normPath( iFolder )
            oalFile = oalFolder + "/" + oalAppName
            if (ALDiskExplorer.existFile( oalFile )):
                oalFolder = oalFolder + "/plugins"
                self.synchronizeOff()
                self.addPluginFolder( oalFolder )
                self.synchronizeOn()
                oalStatus = True
            elif (ALDiskExplorer.existFile( oalFile + ".exe" )):
                oalFolder = oalFolder + "/plugins"
                self.synchronizeOff()
                self.addPluginFolder( oalFolder )
                self.synchronizeOn()
                oalStatus = True

        if ( oalStatus==False ):
            ALVerbose.aalibDEBUG( "[NO] Standard Plug-in Folder NOT Found :: [*/plugins]" )
        self.synchronizeOff()
        return oalStatus



    def addPluginFile( self, _oalString ):
        self.synchronizeOn()
        oalFolder = ALFile.getFolderName( _oalString )
        if ( self.m_oalPluginFolder != oalFolder ):
            self.m_oalPluginFolder = oalFolder
            ALSystem.setPath( self.m_oalPluginFolder )
        self.m_oalPluginFile = ALString( _oalString )
        self.__addPlugin( _oalString )
        self.synchronizeOff()




    def getPlugin( self, _oalToken, _oalXmlFileName = None, _oBoolXsd = False ):
        self.synchronizeOn()
        oalPlugin = None
        if ( _oalToken!=None ):
            try:
                if (_oalXmlFileName == None  and  _oBoolXsd == False):
                    # Workaround to bypass instance plug-in problem = Must to be fixed for modules import only
                    try:
                        oalModule = self.getObject( _oalToken )
                        oalPlugin = oalModule.__dict__[ _oalToken ]()
                    except:
                        oalPlugin = self.getObject( _oalToken )

                else:
                    # Try to generate an instance of the object based on XSD and an XML file
                    if (_oalXmlFileName != None and  _oBoolXsd == False):
                        try:
                            oalModule = self.getObject( _oalToken )

                            oalStrXmlFileName = ALDiskExplorer.normalize( _oalXmlFileName )
                            if (ALDiskExplorer.existFile( oalStrXmlFileName )!=False):
                                oalPlugin = oalModule.unMarshalling( oalStrXmlFileName )
                            else:
                                ALVerbose.aalibDEBUG( "Plug-in Factory XML file problem: getPlugin() - " + ALString(_oalToken ) + ", " + ALString( _oalXmlFileName )  )
                        except:
                            self.__raiseControl()
                    # Try to generate an empty instance of the object based on XSD
                    elif (_oalXmlFileName == None and  _oBoolXsd == True):
                        try:
                            oalModule = self.getObject( _oalToken )
                            oalPlugin = oalModule.__dict__[ _oalToken ]()
                        except:
                            self.__raiseControl()
            except:
                ALVerbose.aalibDEBUG( "Plug-in Factory problem: getPlugin() - " + ALString(_oalToken ) + ", " + ALString( _oalXmlFileName )  )
                self.__raiseControl()
        self.synchronizeOff()
        return oalPlugin





    def addPlugin( self, _oalPluginInstance ):
        self.synchronizeOn()
        if ( _oalPluginInstance!=None ):
            oalToken  = _oalPluginInstance.getPluginName()
            self.synchronizeOff()
            self.addObject( oalToken, _oalPluginInstance )
            self.synchronizeOn()
        self.synchronizeOff()


    def outputScreenPlugin( self ):
        self.synchronizeOn()
        for ikey in self.getSequence():
            ALVerbose.screen( "" )
            iPlugin = self[ ikey ]
            if (iPlugin!=None):
                try:
                    iPlugin.outputScreenSignature()
                except:
                    try:
                        oalModule = iPlugin
                        oalPlugin = oalModule.__dict__[ ikey ]()
                        oalPlugin.outputScreenSignature()
                    except:
                        ALVerbose.aalibDEBUG( "No information available: a XSD plug-in = " + ALString( ikey ) )
                        self.__raiseControl()

        ALVerbose.screen( "" )
        self.synchronizeOff()


    def outputScreenPluginGroup( self ):
        self.synchronizeOn()
        ALVerbose.screen( "" )
        for ikey in self.m_oalDictPluginGroup:
            ALVerbose.screen( "Plug-in GROUP: " + self.m_oalDictPluginGroup[ ikey ] )
        ALVerbose.screen( "" )
        self.synchronizeOff()


    def outputScreenPluginGroupTree( self ):
        self.synchronizeOn()
        for ikey in self.m_oalDictPluginGroup:
           oalPluginGroup = self.m_oalDictPluginGroup[ ikey ]
           ALVerbose.screen( "" )
           ALVerbose.screen( "" )
           ALVerbose.screen( "#####################################" )
           ALVerbose.screen( "####################### Plug-in GROUP: " + oalPluginGroup )
           for jkey in self:
               jPlugin = self[ jkey ]
               try:
                   try:
                       if (jPlugin.getGroup()==oalPluginGroup):
                           ALVerbose.screen( "" )
                           jPlugin.outputScreenSignature()
                   except:
                       try:
                           jPlugin = jPlugin.__dict__[ jkey ]()
                           if (jPlugin.getGroup()==oalPluginGroup):
                               ALVerbose.screen( "" )
                               jPlugin.outputScreenSignature()
                       except:
                           ALVerbose.aalibDEBUG( "The group information is not available: " + ALString( jPlugin) )
                           self.__raiseControl()
               except:
                    ALVerbose.aalibDEBUG( "It's not a AALib standard plug-in: " + ALString( jPlugin) )
                    self.__raiseControl()
           ALVerbose.screen( "" )
           ALVerbose.screen( "#####################################" )
        ALVerbose.screen( "" )
        self.synchronizeOff()


    def processPluginContainingStringName( self, _oalString = "Help", _bSyncronization = True ):
       self.synchronizeOn()
       bStatus = False
       oalString = ALString( _oalString )


       for ikey in self.getSequence():
            iPlugin = self[ ikey ]
            try:
                ostrPluginName = iPlugin.getPluginName()
                if (ostrPluginName.existSubString( oalString )):
                    iPlugin.executePlugin()
                    if (_bSyncronization==True):
                        iPlugin.synchronizePlugin()
                    bStatus = True
            except:
#                 iPlugin   = iPlugin.__dict__[ ikey ]()
#                 try: # to be improved...
                 try:
                     iPlugin   = iPlugin.__dict__[ ikey ]()
                     ostrPluginName = iPlugin.getPluginName()
                     if (ostrPluginName.existSubString( oalString )):
                        iPlugin.executePlugin()
                        if (_bSyncronization==True):
                            iPlugin.synchronizePlugin()
                        bStatus = True
                 except:
                     pass

       self.synchronizeOff()
       return bStatus


##############################################################################
