#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition         import *
from ALImportPython       import PySys
from ALVerbose            import ALVerbose
from ALInformation        import ALInformation
from ALList               import ALList
from ALString             import ALString



class ALInformationTest( ALInformation ):
    """
    Interface for Information Classes.
    """
    def __init__( self, _oalStringNameTest = "Test", _oiNumberTest = 0 ):
        ALInformation.__init__( self )
        self.m_oalStringNameTest  = _oalStringNameTest
        self.m_oalStringException = ALString.empty()
        self.m_oiNumberTest       = _oiNumberTest
        self.m_oboolSuccess       = True
        self.m_oalListTest        = ALList()


    def isSuccess(self):
        self.synchronizeOn()
        oboolSuccess = True
        if(self.m_oboolSuccess==True):
            for iTest in self.m_oalListTest:
                if (iTest.isSuccess()==False):
                    oboolSuccess = False
        else:
            oboolSuccess = False
        self.synchronizeOff()
        return oboolSuccess


    def getTestNumber(self):
        return self.m_oiNumberTest


    def getTestName(self):
        return self.m_oalStringNameTest



    def setTerminate( self, _oboolSuccess = True):
        self.synchronizeOn()
        self.m_oboolSuccess = _oboolSuccess
        self.synchronizeOff()
        self.setTimeEnd()


    def setFailure( self ):
        self.synchronizeOn()
        self.m_oboolSuccess = False
        self.synchronizeOff()


    def setSuccess( self ):
        self.synchronizeOn()
        self.m_oboolSuccess = True
        self.synchronizeOff()



    def addInformationTest( self, _oalInformationTest ):
        self.synchronizeOn()
        self.m_oalListTest.addObject( _oalInformationTest )
        self.synchronizeOff()



    def setException( self, _oalString ):
        self.synchronizeOn()
        self.m_oalStringException = ALString( _oalString  )
        self.synchronizeOff()
        self.setFailure()



    def getException( self ):
        return self.m_oalStringException



    def getNumberTestMethod( self ):
        return self.m_oalListTest.getNumberObjects()



    def outputString( self, _oalString = "  " ):
        ofTime = self.getRunTime()
        obSuccess = self.isSuccess()
        self.synchronizeOn()

        oalString = ALString()
        if (obSuccess==True):
            oalString += _oalString + "[SUCCESS] [ " +ALString( self.m_oiNumberTest )+" ][ " + self.m_oalStringNameTest +" ]["+ ALString( ofTime )+"]\n"
        else:
            oalString += _oalString + "[FAILURE] [ " +ALString( self.m_oiNumberTest )+" ][ " + self.m_oalStringNameTest +" ]["+ ALString( ofTime )+"]\n"
        if (obSuccess==False):
            if (self.m_oalStringException!=ALString.empty()):
                oalString +=  _oalString + "          [Exception]: " + self.m_oalStringException + "\n"

        for iTest in self.m_oalListTest:
            oalString += iTest.outputString( "            " )
        self.synchronizeOff()
        return oalString






##############################################################################
