#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition            import *
from ALImportPython          import PySys
from ALObject                import ALObject
from ALString                import ALString
from ALTime                  import ALTime
from ALSystem                import ALSystem
from ALDiskExplorer          import ALDiskExplorer



class ALLog( ALObject ):
    """
    Base class to Log data
    This class control the information to lof manipulators
    """
    m_oalTimeInit         = None
    m_oalStringSystemInfo = ALString()
    m_ofRuntime           = 0.0



    def __init__( self ):
        ALObject.__init__( self )
        ALLog.m_oalTimeInit = ALTime.getTimeString()


    def log( self, _oalString = None ):
        self.synchronizeOn()
        if _oalString != None :
            oalString = ALString( ALTime.timeString( ) +  _oalString )
            PySys.stdout.write( oalString )
        self.synchronizeOff()


    def getTimeInit():
        return ALLog.m_oalTimeInit
    getTimeInit = staticmethod( getTimeInit )


    def getTimeEnd():
        return ALTime.getTimeString()
    getTimeEnd = staticmethod( getTimeEnd )


    def setRuntime( _oFloat ):
        ALLog.m_ofRuntime = _oFloat
    setRuntime = staticmethod( setRuntime )


    def getRuntime():
        return ALLog.m_ofRuntime
    getRuntime = staticmethod( getRuntime )



    def createStringSystemInfo( _oalFileName = None ):
        ALLog.m_oalStringSystemInfo =                               "     CoreLibrary    :: Asynchronous Action Library Project - jyaalib.sourceforge.net\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Version        :: " + ALString( AL_VERSION ) + " - Python::Jython\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     VitualMachine  :: " + ALSystem.getMachineVirtuel() + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Log File       :: " + ALTime.getTimeString() + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     OS             :: " + ALSystem.getOSName() + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     OSType         :: " + ALSystem.getOSType()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     OSVersion      :: " + ALSystem.getOSVersion() + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Machine        :: " + ALSystem.getMachineVersion() + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Host           :: " + ALSystem.getHost()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     HostIP         :: " + ALSystem.getHostIP()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     UserName       :: " + ALSystem.getUserName()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Compiler       :: " + ALSystem.getCompilerName()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     Executable     :: " + ALSystem.getProgramName()  + "\n"
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     ExecFolder     :: " + ALSystem.getCwd()  + "\n"
        if (_oalFileName is not None):
            ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     LogFile        :: " + ALDiskExplorer.getAbsolutePath( _oalFileName ) + "\n"
        oalString = ALString()
        oPyArg = PySys.argv
        i = 1
        while (i<len(oPyArg)):
            oalString += ALString( oPyArg[i]) + " "
            i+=1
        ALLog.m_oalStringSystemInfo = ALLog.m_oalStringSystemInfo + "     CommandLine    :: " + oalString  + "\n"




    createStringSystemInfo = staticmethod( createStringSystemInfo )


    def getSystemInfo():
        return ALLog.m_oalStringSystemInfo
    getSystemInfo = staticmethod( getSystemInfo )

##############################################################################


