#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALActionController    import ALActionController
from ALToken               import ALToken
from ALVerbose             import ALVerbose
from ALInformation         import ALInformation
from ALString              import ALString


class ALPlugin( ALActionController ):
    """
    Interface for plug-in classes.

    # To be modified by the user
    def preProcess( self, _oalObject = None ):
    def process( self, _oalObject = None ):
    def postProcess( self, _oalObject = None ):
    """
    def __init__( self, _oalPluginName = None, _oBoolPriority = False  ):
        ALActionController.__init__( self, _oBoolPriority )
        self.m_oalInformation = ALInformation()
        if ( _oalPluginName is not None ):
            self.m_oalPluginName = ALToken( _oalPluginName )
        else:
            self.m_oalPluginName = ALToken( self.getClassName() )
        self.setGroup(  self.m_oalPluginName )
        self.setSource( self.m_oalPluginName + ALToken( ".py" ) )
        self.m_oalStringMethod = ALString()


    # Only here to be used with XMLRPC servers = to define the interfaces of the server
    # as a Plugin
    def _listMethods(self):
        # implement this method so that method.listMethods
        # knows to advertise the strings methods
        # TO BE MODIFIED !!!! - SPIKE
        return list_public_methods(self) + ['method.' + method for method in list_public_methods( self.m_oalStringMethod )]


    def outputScreenInformation( self ):
        self.synchronizeOn()
        ALVerbose.screen( "Plug-in       Name: " + self.getPluginName() )
        ALVerbose.screen( "                ID: " + self.getID() )
        ALVerbose.screen( "             Group: " + self.m_oalInformation.getGroupName() )
        ALVerbose.screen( "            Source: " + self.m_oalInformation.getFileName() )
        ALVerbose.screen( "            Author: " + self.m_oalInformation.getAuthor() )
        ALVerbose.screen( "           Contact: " + self.m_oalInformation.getContact() )
        ALVerbose.screen( "           Version: " + self.m_oalInformation.getVersion() )
        ALVerbose.screen( "       Information: " + self.m_oalInformation.getInformation() )
        ALVerbose.screen( "         Copyright: " + self.m_oalInformation.getCopyright() )
        self.synchronizeOff()


    def outputScreenSignature( self ):
        self.outputScreenInformation()


    def setSource( self, _oFileName ):
        self.synchronizeOn()
        self.m_oalInformation.setFileName( _oFileName )
        self.synchronizeOff()


    def getSource( self ):
        self.synchronizeOn()
        oFileName = self.m_oalInformation.getFileName( )
        self.synchronizeOff()
        return oFileName


    def setGroup( self, _oGroupName ):
        self.synchronizeOn()
        self.m_oalInformation.setGroupName( _oGroupName )
        self.synchronizeOff()


    def getGroup( self ):
        self.synchronizeOn()
        oGroupName = self.m_oalInformation.getGroupName( )
        self.synchronizeOff()
        return oGroupName


    def setInformation( self, _oalInformation ):
        self.synchronizeOn()
        self.m_oalInformation = _oalInformation
        self.synchronizeOff()


    def getInformation( self ):
        self.synchronizeOn()
        oalInfo = self.m_oalInformation
        self.synchronizeOff()
        return oalInfo


    def setToken( self, _oalTokenName ):
        self.synchronizeOn()
        self.m_oalPluginName = ALToken( _oalTokenName )
        self.synchronizeOff()


    def getToken( self ):
        oalToken = ALToken( self.m_oalPluginName )
        return oalToken

    def setPluginName( self, _oalTokenName ):
        self.setToken( _oalTokenName )

    def getPluginName( self ):
        return self.getToken()


    def synchronizePlugin( self, _oalObject = None ):
        self.synchronize( )


    def pluginControl( self ):
        self.actionControl()


    def executePlugin( self, _oalObject = None ):
        """
        This must be called to start the Threaded Action Plugin
        """
        self.executeAction( _oalObject )


    def executePluginTimer( self ):
        """
        This is the main connection method to be used with Timer Plug-ins and methods.
        """
        self.executeActionTimer()



    def executePluginSynchronous( self, _oalObject = None ):
        """
        This method starts the Action in Synchronous  Mode.
        """
        self.executeActionSynchronous()





##############################################################################
