#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALImportPython             import PySubProcess
from ALImportPython             import PyOS
from ALImportPython             import PySignal
from ALImportPython             import PyThreading
from ALImportPython             import PySelect
from ALObject                   import ALObject
from ALString                   import ALString
from ALSystem                   import ALSystem
from ALDebug                    import ALDebug
from ALDiskExplorer             import ALDiskExplorer
from ALList                     import ALList
from ALTrace                    import ALTrace
from ALVerbose                  import ALVerbose



class ALPythonSubProcess( ALObject, PySubProcess.Popen ):
    """
    Interface for ALSubProcess objects.
    Inherits all developments from Python 2.4 subprocess module
    """
    def __init__( self, _oStrCommandArgs, _oStringFolderName = None ):
        ALObject.__init__( self )
        shell = False
        self.m_oReturnCode   = -1
        self.m_oalStrCommand = ALString( _oStrCommandArgs )
        self.m_oListOutput = None
        self.m_oListError  = None
        self.m_oStringFolderName         = ALSystem.getCurrentFolder()
        self.m_oStringFolderNameOriginal = ALSystem.getCurrentFolder()

        # Fix Problem in Linux to call softwares by the system config
        if ( ALSystem.getOSName()!="Windows" ):
            shell = True

        if (_oStringFolderName!=None):
            self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
            ALDiskExplorer.createFolder( self.m_oStringFolderName )

      # Verify how I can change the subprocess folder with no change my main software shell currect folder
      # I must find some scheme on python to change it here on the pipe creation
      #  ALSystem.setCurrentFolder( self.m_oStringFolderName )
        PySubProcess.Popen.__init__( self, self.m_oalStrCommand, 0, None,
                 PySubProcess.PIPE, PySubProcess.PIPE, PySubProcess.PIPE,
                 None, False, shell,
                 None, None, True,
                 None, 0 )
      #  ALSystem.setCurrentFolder( self.m_oStringFolderNameOriginal )



    def setWorkingDirectory( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setDirectory( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setWorkingFolder( self, _oStringFolderName ):
        self.setFolder( _oStringFolderName )


    def setFolder( self, _oStringFolderName ):
        self.synchronizeOn()
        self.m_oStringFolderName = ALDiskExplorer.getAbsolutePath( _oStringFolderName )
        ALSystem.setCurrentFolder( self.m_oStringFolderName )
        self.synchronizeOff()


    def getInput( self ):
        self.synchronizeOn()
        oListStdInput = ALList()
        oListStdInput.add( self.m_oalStrCommand )
        self.synchronizeOff()
        return oListStdInput



    def getOutput( self ):
        self.synchronizeOn()
        oalList = ALList()
        try:
            if (self.m_oListOutput!=None):
                oalString = ALString(  self.m_oListOutput  )
                oalList = oalString.split("\n")
        except:
            self.__raiseControl()
        self.synchronizeOff()
        return oalList



    def getOutputError( self ):
        self.synchronizeOn()
        oalList = ALList()
        try:
            if (self.m_oListError!=None):
                oalString = ALString(  self.m_oListError  )
                oalList = oalString.split("\n")
        except:
            self.__raiseControl()
        self.synchronizeOff()
        return oalList



    def getReturnCode( self ):
        self.synchronizeOn()
        self.m_oReturnCode = self.returncode
        self.synchronizeOff()
        return self.m_oReturnCode



    def __raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )



    def executeProcess( self, _oalObject = None ):
        return self.getCommunication( _oalObject )


#
#    def getCommunication( self, _oalObject = None ):
#        self.synchronizeOn()
#        ALSystem.setCurrentFolder( self.m_oStringFolderName )
#        self.m_oListOutput, self.m_oListError = self.communicate()
#        self.wait()
#        ALSystem.setCurrentFolder( self.m_oStringFolderNameOriginal )
#        self.synchronizeOff()



    def getCommunication( self, _oalObject = None ):
        self.synchronizeOn()
        self.m_oListOutput, self.m_oListError = self.communicate()
        self.wait()
        self.synchronizeOff()


##############################################################################