#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALString                  import ALString
from ALDiskExplorer            import ALDiskExplorer
from ALFile                    import ALFile
from ALSystem                  import ALSystem
from ALVerbose                 import ALVerbose
from ALStreamFile              import ALStreamFile
from ALScript                  import ALScript
from ALCommandLineArgument     import ALCommandLineArgument
from ALToken                   import ALToken
from ALMd5                     import ALMd5


class ALScriptMd5sum( ALScript ):


    def evaluateMD5Sum( self, _oalStrFolder, _oalStrFileExtension ):
        bModified = False

        oalList =  ALDiskExplorer.getFolderList( _oalStrFolder )
        for ifile in oalList:
            if ALDiskExplorer.isFile( ifile ):
                if ALFile.isExtension( ifile, _oalStrFileExtension ):
                    oalMD5 = ALMd5()
                    oalStreamFile = ALStreamFile( ifile, "r" )
                    oalMD5.analyse( oalStreamFile.read() )
                    oalStreamFile.close()
                    if ( self.analyse( ifile, oalMD5.sum() ) ):
                        bModified = True

        if (bModified):
            ALVerbose.screen( "Package::MODIFIED" )
        else:
            ALVerbose.screen( "Package::OK" )


    def analyse( self, _oalStrFile, _oalStrSum ):
        oalResSum = ALScriptMd5sum.getResource( ALFile.getFileName( _oalStrFile ) )
        if (oalResSum == None):
            ALScriptMd5sum.setResource( ALFile.getFileName( _oalStrFile ), _oalStrSum )
            ALVerbose.screen( "File::NEW      - " + _oalStrFile )
        else:
            oalStrSumOrig = ALString( oalResSum );
            if (oalStrSumOrig==_oalStrSum):
                ALVerbose.screen( "File::OK       - " + _oalStrFile )
            else:
                ALVerbose.screen( "File::MODIFIED - " + _oalStrFile )
                return True

        return False


    def reset( self, _oalObject = None ):
        ALScriptMd5sum.resetResource()


    def process( self ):
        ALScriptMd5sum.inputResource()
        self.evaluateMD5Sum( "../modules", ".py" )
        #self.evaluateMD5Sum( "../../modules", ".py" ) # to test the compilated version


##############################################################################


if __name__ == '__main__':

    oalScript = ALScriptMd5sum( "md5sum" )
    oalScript.addCommand( "--reset", oalScript.reset )
    oalScript.execute()


##############################################################################
