#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################



from ALDefinition      import *
from ALObject          import ALObject
from ALString          import ALString
from ALArgument        import ALArgument


class ALSignal( ALObject ):
    """
    Base class for signal objects.
    """


    def __init__( self, _oalString = None, _oalArgument = None ):
        ALObject.__init__( self )
        self.m_oalStringInformation = None
        self.m_oalStringSignal = None
        self.m_oalArgument = None

        self.setSignal( _oalString )
        if isinstance( _oalArgument, ALArgument ) :
            self.setArgument( _oalArgument )
        else :
            self.setInformation( ALString( _oalArgument ) )


    def clone( self ):
        oalSignal = ALSignal( self.getSignal() )
        oalSignal.setArgument( self.getArgument() )
        oalSignal.setInformation( self.getInformation() )
        oalSignal.m_objectID = self.getID()
        return oalSignal



    def setSignal( self, _oalString ):
        if (_oalString==None):
            self.m_oalStringSignal = None
        else:
            self.m_oalStringSignal = ALString( _oalString )


    def setArgument( self, _oalArgument ):
        if _oalArgument is None:
            self.m_oalArgument = None
        else:
            self.m_oalArgument = ALArgument( _oalArgument )


    def getArgument( self ):
        """
        @return: A copy of the signal argument
        """
        return self.m_oalArgument


    def getArgumentNumber( self, _i = 0 ):
        """
        @return: A copy of the  i argument
        """
        if self.m_oalArgument != None:
            return self.m_oalArgument.getArgumentNumber( _i )
        else:
            return None


    def getSignal( self ):
        """
        @return: A copy of the signal string
        """
        return self.m_oalStringSignal # to use with None signal


    def getSignalToken( self ):
        """
        @return: A copy of the signal string
        """
        return self.getSignal()


    def getInformation( self ):
        """
        @return: Get an information string
        """
        return self.m_oalStringInformation


    def setInformation( self, _oalString ):
        """
        @return: Set an information string
        """
        self.m_oalStringInformation  = ALString( _oalString )



    # Static Methods #########################################################
    def ok( _oalString = None ):
        return ALSignal( AL_SIGNAL_OK, _oalString  )
    ok = staticmethod( ok )


    def error( _oalString = None ):
        return ALSignal( AL_ACTION_SIGNAL_ERROR, _oalString )
    error = staticmethod( error )


    def abort( _oalString = None ):
        return ALSignal( AL_ACTION_SIGNAL_ABORT, _oalString )
    abort = staticmethod( abort )


    def exception( _oalString = None ):
        return ALSignal( AL_ACTION_SIGNAL_EXCEPTION, _oalString )
    exception = staticmethod( exception )


    def exit( _oalString = None ):
        return ALSignal( AL_ACTION_SIGNAL_EXIT, _oalString )
    exit = staticmethod( exit )


    def terminateApplication( _oalString = None ):
        return ALSignal( AL_ACTION_SIGNAL_TERMINATE_APPLICATION, _oalString )
    terminateApplication = staticmethod( terminateApplication )




##############################################################################
