#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython        import *
from ALString              import ALString
from ALSystem              import ALSystem
from ALManager             import ALManager
from ALList                import ALList
from ALSignal              import ALSignal
from ALVerbose             import ALVerbose
from ALDebug               import ALDebug
from ALTrace               import ALTrace
from ALExceptionSignal     import ALExceptionSignal



class ALSlot( ALManager ):
    """
    Class for managing signals.
    """


    def __init__( self ):
        ALManager.__init__( self )
        self.m_oalListConnection = ALList()



    def connect(self, _oalClassMethod, _oalSignal = None ):
        """
        Connects a signal between an object and a method in an other object.
        """
        if ( _oalClassMethod == None ):
            ALVerbose.warning( "[Slot] ALSlot.connect : _oalClassMethod is None" )
            return
        self.synchronizeOn()    # Block the thread
        self.m_oalListConnection.append( [ _oalSignal, _oalClassMethod ] )
        self.synchronizeOff()   # Release the thread


    def __raiseControl( self ):
        oBoolStatus =  ALDebug.isDebugModeOn()
        ALDebug.controlDebug( self, oBoolStatus )


    # Blocked by self.call()
    def executeMethod( self, _method, _oalListArguments = None ):
        try:
            if (_oalListArguments == None):
                _method()
            else:
                _method( _oalListArguments )
        except:
            ALVerbose.aalibDEBUG( "[Slot] EXCEPTION - ALSlot::executeMethod - parameter: " + ALString( _oalListArguments ) )
            ALVerbose.aalibDEBUG( "[Slot] EXCEPTION - -------------------------- method: " + ALString( _method ) )
            self.__raiseControl()



    def call( self, _oalListArguments = None, _oalSignal = None ):
        self.synchronizeOn()    # Block the thread
        for connection in self.m_oalListConnection:
            oalSignal = connection[ 0 ]
            omethod   = connection[ 1 ]
            if (_oalSignal == None):
                if (oalSignal == None):
                    if (omethod!=None):
                        self.executeMethod( omethod, _oalListArguments )
            else:
                if (oalSignal != None):
                    oalStringSignalExt = _oalSignal.getSignal()
                    oalStringSignal    =  oalSignal.getSignal()
                    if (oalStringSignal.equals( oalStringSignalExt )):
                        if (_oalListArguments == None and omethod!=None ):
                            self.executeMethod( omethod, _oalSignal )
                        else :
                            self.executeMethod( omethod, _oalListArguments )
        self.synchronizeOff()   # Release the thread


    def callSignal( self, _oalSignal ):
        self.call( None, _oalSignal )

##############################################################################



