#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from  ALImportPython import  PyString
from  ALMetaObject   import  ALMetaObject


class ALString( ALMetaObject, str ):
    """
    Interface for AL string objects.
    """


    def __init__( self, _oString = "" ):
        ALMetaObject.__init__( self )
        str.__init__( _oString )



    def getPythonString( self ):
        """
        #Workaround to Jython and java bugs for multi inheritance
        it allows convert the ALString for jython string!!!!
        """
        return PyString.join( self, "" )


    def duplicate( self ):
        """
        This method is normally provided by the ESObject base class but
        since we don't inherit from ESObject we must define this method
        explicitly.
        """
        return ALString( self )


    def equals( self, _ostr ):
        """
        a.equals(b) corresponds to (a == b) in Python.
        """
        return ( str( self ) == str( _ostr ) )



    def atoi( self ):
        """
        Returns an integer.
        """
        try:
            oValue = float( str( self ) )
        except:
            oValue = int( str( self ) )
        return int( oValue )


    def atof( self ):
        """
        Returns a float.
        """
        try:
            oValue = float( str( self ) )
        except:
            oValue = int( str( self ) )
        return float( oValue )



    def replace( self, _ostr1, _ostr2, _iMaxcount = -1 ):
        """
        Overrides the str replace method.
        """
        return ALString( str.replace( str( self ), _ostr1, _ostr2, _iMaxcount ) )


    def capital( self ):
        return ALString( self.upper() )


    def cleanXml( self ):
        """
        Returns a string with no espaces
        """
        #bug on jython linux version the time in teh format [999] is cutted
        # workaround
        oString2 = ALString( self )
        oString2 = oString2.replace( "\n", "" )
        oString2 = oString2.replace( "\t", "" )
        return ALString( oString2 )



    def split( self, _oCharater = " " ):
        """
        """
        oList = str.split( self, _oCharater  )
        return oList


    def getToken( self, _oPosition = 0,  _oCharater = " " ):
        oList = self.split( _oCharater )
        oalString = None
        if ( len( oList) >_oPosition+1 ):
            oalString = ALString( oList[ _oPosition ] )
        return oalString


    def getNextToken( self, _oRefWord = " ",  _oCharater = " " ):
        oList = self.split( _oCharater )
        oalString = None

        if _oRefWord in oList:
            refWordIndex = oList.index( _oRefWord )
            wordIndex = refWordIndex + 1
            if ( len( oList) > refWordIndex+1 ):
                oalString = ALString( oList[ wordIndex ] )

        return oalString


    def existSubString( self, _oString ):
        result = self.rfind( _oString )
        if (result==-1):
            return False
        else:
            return True


    def contains( self, _oString ):
        return self.existSubString( _oString )



    def empty():
        return ALString( "" )
    empty = staticmethod( empty )


   # only to fix xml log stuffs
    def setLogXmlToString(  _oalString ):
        oalString = ALString( _oalString )
        oalString = oalString.replace( "<", "[" )
        return oalString.replace( ">", "]" )
    setLogXmlToString = staticmethod( setLogXmlToString )



    def setFormat( _oString, _oiSize = 25 ):
        oalStr = ALString( _oString )
        iSizeSpace  = 1
        iSizeString = len( oalStr )

        if (iSizeString<_oiSize):
            iSizeSpace = _oiSize - iSizeString

        oalString = ALString()
        for i in range( 0, iSizeSpace):
            oalString = oalString + " "
        for i in range( 0, iSizeString):
            oalString = oalString + oalStr[i]

        return oalString

    setFormat = staticmethod( setFormat )

##############################################################################
