#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython   import PySys
from ALImportPython   import PyOS
from ALImportPython   import PySocket
from ALDefinition     import *
from ALObject         import ALObject
from ALString         import ALString


class ALSysInfo( ALObject ):
    """
    Used to identify diferent parameters from the OS System
    For each plataform and interpreter the basic layer extracts the main information
    http://java.sun.com/javase/6/docs/api/java/lang/System.html
    """
    def __init__( self ):
       ALObject.__init__( self )
       self.m_interpreter    = None
       self.m_machineVirtuel = None
       self.m_OSName         = None
       self.m_typeOS         = None
       self.m_userName       = None
       self.m_hostName       = None
       self.m_pythonVersion  = None
       self.m_OSVersion      = None
       self.m_machineVersion = None
       self.m_hostIP         = None

       self.__init()


    def __init( self ):
       oSystem = PySys.platform[:4]
       if (oSystem=="java"):
           self.m_interpreter    = ALString( "Jython" )
           self.m_machineVirtuel = ALString( "Java" )
       else:
           self.m_interpreter    = ALString( "Python" )
           self.m_machineVirtuel = ALString( "Python" )
       self.m_pythonVersion  = ALString( PySys.version )
       self.__setOSParameters()



    def __setOSParameters( self ):
       self.__getOSControl()
       self.__setJava()
       self.__setWindows()
       self.__setSun()
       self.__setMacOSX()
       self.__setLinux()
       self.__setBSD()
       self.__setCygwin()
       self.__setUnknown()



    def __setJava( self ):
        if (self.m_typeOS==AL_SYSTEM_JAVA):
           from     ALImportJython   import JySystem
           self.m_OSName         = ALString( JySystem.getProperty( "os.name" ) )
           if (self.m_OSName.contains("Windows")):
               self.m_OSName = "Windows"
           self.m_OSType         = ALString( JySystem.getProperty( "os.name" ) )
           self.m_userName       = ALString( JySystem.getProperty( "user.name" ) )
           self.m_hostName       = ALString( "NoMachineName" )
           self.m_OSVersion      = ALString( JySystem.getProperty( "os.version" ) )
           self.m_machineVersion = ALString( JySystem.getProperty( "os.arch" ) )
           try:
               self.m_hostName       = ALString( PySocket.gethostname() )
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )


    def __setWindows( self ):
        if (self.m_typeOS==AL_SYSTEM_WINDOWS):
           from ALImportPython   import PyPlatform
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Windows" )
           self.m_OSType         = ALString( PyPlatform.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyPlatform.uname()[3] )
           self.m_machineVersion = ALString( "i686" )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )


    def __setSun( self ):
        if (self.m_typeOS==AL_SYSTEM_SOLARIS):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Sun" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setMacOSX( self ):
        if (self.m_typeOS==AL_SYSTEM_MACOSX):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "MacOSX" )
           self.m_OSType         = ALString( PyOS.uname()[0] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )




    def __setLinux( self ):
        if (self.m_typeOS==AL_SYSTEM_LINUX):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Linux" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setBSD( self ):
        if (self.m_typeOS==AL_SYSTEM_BSD):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "BSD" )
           self.m_OSType         = ALString( PyOS.uname()[2] )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setCygwin( self ):
        if (self.m_typeOS==AL_SYSTEM_WINCYGWIN):
           from ALImportPython   import PyGetPass
           self.m_OSName         = ALString( "Cygwin" )
           self.m_OSType         = ALString( "Cygwin" )
           self.m_userName       = ALString( PyGetPass.getuser() )
           self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
           self.m_OSVersion      = ALString( PyOS.uname()[2] )
           self.m_machineVersion = ALString( PyOS.uname()[4] )
           try:
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               self.m_hostIP         = ALString( "Not connected to internet" )



    def __setUnknown( self ):
        if (self.m_typeOS==AL_SYSTEM_UNKNOWN):
           try:
               from ALImportPython   import PyGetPass
               self.m_OSName         = ALString( AL_SYSTEM_UNKNOWN )
               self.m_OSType         = ALString( PyOS.uname()[2] )
               self.m_userName       = ALString( PyGetPass.getuser() )
               self.m_hostName       = ALString( PySocket.getfqdn() ) # PyPlatform.uname()[1]
               self.m_OSVersion      = ALString( PyOS.uname()[2] )
               self.m_machineVersion = ALString( PyOS.uname()[4] )
               self.m_hostIP         = ALString( PySocket.gethostbyname( PySocket.gethostname() ) )
           except:
               print " [Warnning] PyAALib::NOT TESTED AND VALIDATED in this Unknown Operational System\n"


    def __getOSControl( self ):
        oalString = AL_SYSTEM_UNKNOWN
        if (PyOS.name =="java"):
            oalString = ALString( AL_SYSTEM_JAVA )
        elif (PyOS.name =="nt"):
            oalString = ALString( AL_SYSTEM_WINDOWS )
        elif ( PyOS.name == "posix" ):
            oalStrSystem = PyOS.uname()
            if ( oalStrSystem[0] == "Linux" ):
                oalString = ALString( AL_SYSTEM_LINUX )
            elif ( oalStrSystem[0] == "SunOS" ):
                oalString = ALString( AL_SYSTEM_SOLARIS )
            elif ( oalStrSystem[0] == "Darwin" ):
                oalString = ALString( AL_SYSTEM_MACOSX )
            elif ( oalStrSystem[0] == "FreeBSD" ):
                oalString = ALString( AL_SYSTEM_BSD )
            elif ( oalStrSystem[0] != "Linux" ):
                oalStrPath = PyOS.getcwd()
                if ( oalStrPath.count("cygdrive") == True ):
                    oalString = ALString( AL_SYSTEM_WINCYGWIN )
                else :
                    print " [Warnning] PyAALib::NOT TESTED AND VALIDATED in this Unknown Operational System\n"
        self.m_typeOS = oalString

##############################################################################
