#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *


class ClassBT :

    def methodPrintInit( self, _obj = None  ):
        ALVerbose.unitTest( "Class B::printing in INIT: " + ALString( _obj ) )

    def methodPrintEnd( self, _obj = None  ):
        ALVerbose.unitTest( "Class B::printing in END : " + ALString( _obj ) )

    def methodPrintSignal( self, _obj = None  ):
        ALVerbose.unitTest( "Class B::printing in SIGNAL : " + ALString( _obj ) )

    def methodPrintRun( self, _obj = None ):
        ALVerbose.unitTest( "Class B::printing in RUN : " + ALString( _obj ) )


class ClassCT :

    def methodPrintRun( self, _obj = None ):
        ALVerbose.unitTest( "Class C::printing in RUN : " + ALString( _obj ) )

    def methodPrintSignal( self, _obj = None  ):
        ALVerbose.unitTest( "Class C::printing in SIGNAL : " + ALString( _obj ) )

    def methodPrintObjectControl( self, _obj = None  ):
        ALVerbose.unitTest( "--------->The Object Control : " + ALString( _obj ) )

    def methodPrintEndExecution( self, _obj = None  ):
        ALVerbose.unitTest( "--------->The Object Control END: " + ALString( _obj ) )

    def methodPrintObjectExecution( self, _obj = None  ):
        ALVerbose.unitTest( "--------->Action Execution : " + ALString( _obj ) )

    def methodSimulateAnError( self, _obj = None  ):
        ALVerbose.unitTest( "--------->Class C::methodSimulateAnError: " + ALString( _obj ) )
        method__SIMULATE__AN__ERROR()


class TestALActionScreen( ALAction ):

    def __init__( self, _iThreadNo, _fSleepTime ):
        ALAction.__init__( self )
        self.m_iThreadNo  = _iThreadNo
        self.m_fSleepTime = _fSleepTime


    def process( self, _oalObject = None ):
        self.synchronizeOn()
        if (_oalObject!=None):
            ALVerbose.unitTest( "Method Started by :" + ALString( _oalObject ) )

        for i in range( 1, 10 ):
            #To force the execption...
            #al_FRAMEWORK_FORCE_EXCEPTION

            self.m_oalSlotRun.call( ALString( " ----- bip bip : " + self.getName() ) )
            ALVerbose.unitTest( "Output from " + self.getName() )
            ALTime.sleep( self.m_fSleepTime )
            self.threadControl()

#        print "FINISHED"
        self.synchronizeOff()


    def methodTest( self, _oalObject = None  ):
        for i in range( 1, 3 ):
            self.m_oalSlotRun.call( ALString( "bip bip : " + self.getName() ) )
            ALVerbose.unitTest( "Output from " + self.getName() )
            if (_oalObject!=None):
                ALVerbose.unitTest( "Method Started by: " + ALString( _oalObject ) )
            ALTime.sleep( self.m_fSleepTime )




##############################################################################
# Module ALActionMethod

class TestCaseALActionMethod( ALTestCase ):
    def testALActionMethod( self ):
        ALVerbose.unitTest( "Test ALActionMethod" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()


        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "Action in Thread 01" )
#        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
#        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
#        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )


        oalAction1.connectInit( oClassBT.methodPrintInit )
        oalAction1.connectEnd(  oClassBT.methodPrintEnd  )
        oalAction1.connectRun(  oClassCT.methodPrintRun  )


        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "Action in Thread 02" )
        oalAction2.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction2.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction2.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )



        oalActionMethod = ALActionMethod( oalAction2.methodTest )
        oalActionMethod.setName( "MainActionMethod" )


        #Execution
        oalActionMethod.executeMethod()

        while (oalActionMethod.isExecute()):
            ALVerbose.unitTest( " ------------------->oalActionMethod is alive and running" )
            ALTime.sleep(1)
        ALVerbose.unitTest( "oalActionMethod is FINISHED" )


        oalActionMethod = ALActionMethod()
        oalActionMethod.setMethod( oalAction2.methodTest )
        oalActionMethod.setName( "MainActionMethod2" )
        #Execution
        oalActionMethod.executeMethod( oalActionMethod.getName() )

        while (oalActionMethod.isExecute()):
            ALVerbose.unitTest( " ------------------->oalActionMethod is alive and running" )
            ALTime.sleep(1)
        ALVerbose.unitTest( "oalActionMethod is FINISHED" )


        oalMethod = oalActionMethod.getMethod( )
        oalMethod()



        # Prove of the crash when you try to execute an instance of a thread created outside of the ALActionMethod
        ALVerbose.unitTest( "Test of the crash when you try to execute an instance of a thread created outside of the ALActionMethod")
        oalActionMethod = ALActionMethod()
        oalActionMethod.setMethod( oalAction2.executeAction )
        oalActionMethod.executeMethod()
        ALVerbose.unitTest( "ok !!!")


        ALVerbose.unitTest( )



##############################################################################
# Module ALActionSet

class TestCaseALActionSet( ALTestCase ):
    def testALActionSet( self ):
        ALVerbose.unitTest( "Test ALActionSet Synchronous" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "Action in Thread 01" )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "Action in Thread 02" )
        oalAction2.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction2.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction2.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalActionSet = ALActionSet()
        oalActionSet.setName( "MainActionSet" )
        oalActionSet.addAction( oalAction1 )
        oalActionSet.addAction( oalAction2 )

        oalActionSet.execute()
        ALTime.sleep( 2 )
        oalActionSet.pause()
        ALTime.sleep( 1 )
        oalActionSet.resume()
        ALTime.sleep( 2 )
        oalActionSet.abort()

        #oalActionSet.waitActionEnd # Needed to TestCase run

        ALVerbose.unitTest( )



    def testALActionSlots( self ):
        ALVerbose.unitTest( "Test ALAction SLots" )

        oClassBT = ClassBT()

        oalAction = ALAction()
        oalAction.setName( "Action in Thread 01" )
        oalAction.connectPreProcess(   oClassBT.methodPrintInit )
        oalAction.connectPostProcess(  oClassBT.methodPrintEnd  )
        oalAction.connectProcess(      oClassBT.methodPrintRun  )

        oalAction.executeAction()

        ALVerbose.unitTest( )



##############################################################################
# Module ALActionSetAsynchronous

class TestCaseALActionSetAsynchronousSynchronous( ALTestCase ):
    def testALActionSetSynchronous( self ):
        ALVerbose.unitTest( "Test ALActionSet Synchronous" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "Action in Thread 01" )
        oalAction1.setTimeOut( 3 )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "Action in Thread 02" )
        oalAction2.setTimeOut( 3 )
        oalAction2.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction2.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction2.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalActionSet = ALActionSetSynchronous()
        oalActionSet.setName( "MainActionSet" )
        oalActionSet.setTimeOut( 2 )
        oalActionSet.addAction( oalAction1 )
        oalActionSet.addAction( oalAction2 )

        oalActionSet.execute()
#        ALTime.sleep( 2 )
#        oalActionSet.pause()
#        ALTime.sleep( 1 )
#        oalActionSet.resume()
#        ALTime.sleep( 2 )
#        oalActionSet.abort()
#

        ALVerbose.unitTest( )

    def testALActionSetAsynchronous( self ):
        ALVerbose.unitTest( "Test ALActionSet ASynchronous" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "Action in Thread 01" )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "Action in Thread 02" )
        oalAction2.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction2.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction2.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalActionSet = ALActionSetAsynchronous()
        oalActionSet.setName( "MainActionSet" )
        oalActionSet.addAction( oalAction1 )
        oalActionSet.addAction( oalAction2 )

        oalActionSet.execute()
        ALTime.sleep( 2 )
        oalActionSet.pause()
        ALTime.sleep( 1 )
        oalActionSet.resume()
        ALTime.sleep( 2 )
        oalActionSet.abort()

        #oalActionSet.waitActionEnd # Needed to TestCase run

        ALVerbose.unitTest( )



#===============================================================================
#Plugins for multiaction application and ApplicationActionControl Test
#===============================================================================

class TestPlugin01( ALPlugin ):

    def __init__( self ):
        ALPlugin.__init__( self )

    def process( self, oalObject = None ):
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(), "Running TestPlugin01010101" )
        self.synchronizeOff()



class TestPlugin02( ALPlugin ):

    def __init__( self ):
        ALPlugin.__init__( self, "tokenPlugin02" )

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(), "Running TestPlugin02020202" )
        self.synchronizeOff()


    def methodTest( self, _oalObject = None ):
        ALVerbose.screenID( self.getID(), "Running TestPlugin02020202---TEST" )


class TestPlugin03( ALPlugin ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        ALVerbose.screenID( self.getID(), "Running TestPlugin   ||||" + self.getToken() )
        self.synchronizeOff()


class TestPlugin04( TestPlugin03 ):
    pass


class TestPlugin05( ALPlugin ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()

        self.synchronizeOff()

        ALVerbose.screenID( self.getID(), "Running TestPlugin   ||||" + self.getToken() )

        self.synchronizeOn()
        for i in range(0, 10000):
            j = i
        ALVerbose.screenID( self.getID(), "Total Iteraction 1: " + ALString( j ) )
        for i in range(0, 100000):
            j = i
        ALVerbose.screenID( self.getID(), "Total Iteraction 2: " + ALString( j ) )
        for i in range(0, 100000):
            j = i
        ALVerbose.screenID( self.getID(), "Total Iteraction 3: " + ALString( j ) )
        self.synchronizeOff()




class TestPlugin06( ALPlugin ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 6" )
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 6" )
        self.synchronizeOff()



class TestPlugin07( ALPlugin ):

    def process( self, _oalObject = None ):
        self.synchronizeOn()
        oalPlugin = TestPlugin06()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 6 - in 7" )
        oalPlugin = TestPlugin05()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 5 - in 7" )
        oalPlugin = TestPlugin06()
        oalPlugin.executePlugin()
        ALVerbose.unitTest( "Running Plugin 6 - in 7" )
        self.synchronizeOff()



##############################################################################
# Module ALActionMethodTimer

def printLine( _oalObject = None ):
    ALVerbose.screen( " printing line on screen" )


def printMethod( _oalObject = None ):
    ALVerbose.screen( "from printMethod() = " + ALString( _oalObject ) )


class TestCaseALActionMethodTimer( ALTestCase ):
    def testALActionMethodTimer( self ):
        ALVerbose.unitTest( "Test ALActionMethodTimer" )


        oalActionTimer = ALActionMethodTimer( printLine, 1, 5 )
        # Execution
        oalActionTimer.executeAction( )


#        ALTime.sleep( 10.0 )


        oalActionTimer = ALActionMethodTimer( printLine )
        # Execution
        oalActionTimer.executeAction( )
        ALTime.sleep( 3.0 )
        oalActionTimer.close()

        # Forcing an ERROR - Python avoid the multiples thread instance initialization
#        ALVerbose.warning( "Forcing an ERROR - Python avoid the multiples thread instance initialization" )
#        oalPlugin7 = TestPlugin07()
#        oalActionTimer = ALActionMethodTimer( oalPlugin7.executePlugin, 0.2, 2.0 )
#        oalActionTimer.executeAction( )

        # Designed to run fine
        oalPlugin7 = TestPlugin07()
        oalActionTimer = ALActionMethodTimer( oalPlugin7.executePluginTimer, 0.2, 2.0 )
        oalActionTimer.executeAction( )


        ALVerbose.unitTest( )




class TestCaseALAction( ALTestCase ):


    def testALAction( self ):
        ALVerbose.unitTest( "Test ALAction" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "Thread 01" )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "Thread 02" )
        oalAction2.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction2.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction2.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )

        oalAction3 = TestALActionScreen( 3, 0.01 )
        oalAction3.setName( "Thread 03" )
        oalAction3.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction3.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction3.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )


        oalAction1.executeAction()
        oalAction2.executeAction()


        ALTime.sleep( 1 )
        oalAction1.abort()

        oalAction3.executeAction()


        oalAction2.pause()
        ALTime.sleep( 0.5 )
        oalAction2.resume()
        ALTime.sleep( 0.5 )
        oalAction2.abort()
        oalAction3.abort()
        oalAction2.synchronize()
        oalAction3.synchronize()

        del oalAction1
        del oalAction2
        del oalAction3

        ALVerbose.unitTest( )



    def testALActionVerbose( self ):
        ALVerbose.unitTest( "Test ALAction Verbose" )

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.connectScreen( printMethod )
        oalAction1.connectDEBUG( printMethod )


        oalAction1.screen( "Testing the verbose SCREEN - It must be out 2 times" )
        oalAction1.DEBUG( "Testing the verbose DEBUG - It must be out 2 times if --DEBUG option is on" )


        oalAction1.executeAction()

        del oalAction1

        ALVerbose.unitTest( )




    def testALActionSUCCESS( self ):
        ALVerbose.unitTest( "Test ALAction-SUCESS Workflow" )

        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "ALAction01" )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )
        # Individual Failure control
        oalAction1.connectFAILURE( self.methodControlAbortFAILURE )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "ALAction02" )
        oalAction2.connectPreProcess( oClassBT.methodPrintInit )
        oalAction2.connectPostProcess(  oClassBT.methodPrintEnd  )
        oalAction2.connectProcess(  oClassCT.methodPrintRun  )
        # Individual Failure control
        oalAction2.connectFAILURE( self.methodControlAbortFAILURE )

        oalAction3 = TestALActionScreen( 3, 0.01 )
        oalAction3.setName( "ALAction03" )
        oalAction3.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction3.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction3.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )
        # Individual Failure control
        oalAction3.connectFAILURE( self.methodControlAbortFAILURE )


        # Connecting the workflow
        oalAction1.connectSUCCESS( oalAction2.executeAction )
        oalAction2.connectSUCCESS( oalAction3.executeAction )
        oalAction3.connectSUCCESS( self.methodControlFinalSUCCESS )


        # Init control variable
        self.methodControlStartup()

        # Executing the workflow
        oalAction1.executeAction()


        # Controling the end of teh workflow
        self.methodControlLoopToWaitToClose( 10.0 )


        ALVerbose.unitTest( )


    def testALActionMassiveParallelSUCCESS( self ):
        ALVerbose.unitTest( "Test ALAction-SUCCESS Workflow - Massive Parallel" )

        oClassCT = ClassCT()

        i = 0
        ioActionOrig = TestALActionScreen( 1, 0.1 )
        ioActionOrig.setName( "TestALAction-" + ALString( i ) )
        ioActionOrig.connectProcess( oClassCT.methodPrintObjectExecution  )
        ioActionOrig.connectSUCCESS( oClassCT.methodPrintEndExecution )
        ioActionOrig.connectFAILURE( self.methodControlAbortFAILURE )




        ioActionStart = ioActionOrig

        for ioAction in range(1,10):
            i = i + 1
            ioAction = TestALActionScreen( 1, 0.1 )
            ioAction.setName( "TestALAction-" + ALString( i ) )
            ioAction.connectProcess( oClassCT.methodPrintObjectExecution  )
            ioAction.connectSUCCESS( oClassCT.methodPrintEndExecution )
            ioAction.connectFAILURE( self.methodControlAbortFAILURE )

            ioActionOrig.connectSUCCESS( ioAction.executeAction )
            ioActionOrig = ioAction

        ioActionOrig.connectSUCCESS( self.methodControlFinalSUCCESS )

        # Init control variable
        self.methodControlStartup()

        # Executing the workflow
        ioActionStart.executeAction()


        # Controling the end of teh workflow
        self.methodControlLoopToWaitToClose( 60.0 )


        ALVerbose.unitTest( )


    def testALActionFAILURE( self ):
        ALVerbose.unitTest( "Test ALAction-FAILURE Workflow" )
        oClassBT = ClassBT()
        oClassCT = ClassCT()

        oalAction1 = TestALActionScreen( 1, 0.1 )
        oalAction1.setName( "ALAction01" )
        oalAction1.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction1.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction1.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )
        # Individual Failure control
        oalAction1.connectFAILURE( self.methodControlAbortFAILURE )

        oalAction2 = TestALActionScreen( 2, 0.3 )
        oalAction2.setName( "ALAction02" )
        oalAction2.connectPreProcess( oClassBT.methodPrintInit )
        oalAction2.connectPostProcess(  oClassBT.methodPrintEnd  )
        oalAction2.connectProcess(  oClassCT.methodPrintRun  )
        #Simulating an error ##############################################
        oalAction2.connectProcess(  oClassCT.methodSimulateAnError )
        #Simulating an error ##############################################
        # Individual Failure control
        oalAction2.connectFAILURE( self.methodControlAbortFAILURE )

        oalAction3 = TestALActionScreen( 3, 0.01 )
        oalAction3.setName( "ALAction03" )
        oalAction3.m_oalSlotInit.connect( oClassBT.methodPrintInit )
        oalAction3.m_oalSlotEnd.connect(  oClassBT.methodPrintEnd  )
        oalAction3.m_oalSlotRun.connect(  oClassCT.methodPrintRun  )
        # Individual Failure control
        oalAction3.connectFAILURE( self.methodControlAbortFAILURE )


        # Connecting the workflow
        oalAction1.connectSUCCESS( oalAction2.executeAction )
        oalAction2.connectSUCCESS( oalAction3.executeAction )
        oalAction3.connectSUCCESS( self.methodControlFinalSUCCESS )



        self.methodControlStartup()

        # Executing the workflow
        oalAction1.executeAction()


        # Controling the end of teh workflow
        self.methodControlLoopToWaitToClose( 10.0 )


        ALVerbose.unitTest( )



    def methodControlFinalSUCCESS( self, _oalObject = None ):
        self.m_oBoolStatusClose = True
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " ====================== Workflow is finished with SUCCESS"  )
        ALVerbose.unitTest( " SUCCESS control from object: "  + ALString( _oalObject ) )



    def methodControlFinalFAILURE( self, _oalObject = None ):
        self.m_oBoolStatusClose = True
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " ====================== Workflow is finished with FAILURE"  )
        ALVerbose.unitTest( " Failure control from object: "  + ALString( _oalObject ) )


    # Not implemented the abort yet
    def methodControlAbortFAILURE( self, _oalObject = None ):
        self.m_oBoolStatusClose = True
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " "  )
        ALVerbose.unitTest( " ====================== Workflow is Aborted with FAILURE"  )
        ALVerbose.unitTest( " ABORT control from object: "  + ALString( _oalObject ) )


    def methodControlStartup( self, _oalObject = None ):
        self.m_oBoolStatusClose = False



    def methodControlLoopToWaitToClose( self, _oalTimeOut = 10.0 ):
        oalTimeOut = _oalTimeOut
        oalTime = ALTime()
        while (self.m_oBoolStatusClose == False):
            ALTime.sleep( 0.5 )
            ALVerbose.unitTest( " ####### Waiting to Stop The Loop [s]: " + ALString( oalTime.getDuration() ) )
            if (oalTimeOut < oalTime.getDuration() ):
                ALVerbose.unitTest( "              TimeOut: " + ALString( oalTime.getDuration() ) )
                ALVerbose.unitTest( " Aborting the plugins: " + ALString( oalTime.getDuration() ) )
                self.m_oBoolStatusClose = True



class ClassALActionController( ALActionController ):

    def process( self, _oalObject = None ):
        self.setClusterSize( 4 )
        for iAction in range( 0, 12 ):
             oalAction = TestALActionScreen( 1, 0.1 )
             oalAction.setName( "Thread-" + ALString( iAction ) )
             oalAction.connectSUCCESS( self.controlActionSuccess )
             oalAction.connectFAILURE( self.controlActionFailure )
             oalAction.executeAction()

             # Only 4 simultaneous actions running at the same time
             # The standart value is 64
             self.synchronizeCluster()

        # It waits the end of all actions
        self.synchronizeController()





class TestCaseALActionController( ALTestCase ):
    def testALActionController( self ):
        ALVerbose.unitTest( "Test ALActionController" )

        oclassActionController = ClassALActionController()
        oclassActionController.executeAction()
        oclassActionController.synchronize()

        ALVerbose.unitTest( )



class TestCaseALActionCluster( ALTestCase ):
    def testALActionCluster( self ):
        ALVerbose.unitTest( "Test ALActionCluster" )

        oActionCluster = ALActionCluster()

        #Creating 12 parallel different actions
        for iAction in range( 0, 12 ):
             oActionCluster.addAction( TestALActionScreen( 1, 0.1 ) )

        # Only 2 simultaneous actions running at the same time
        # The standart value is 64
        oActionCluster.setClusterSize( 2 )
        # It executes until the end of the last action
        oActionCluster.executeAction()
        # It waits to close the active action
        oActionCluster.synchronize()

        ALVerbose.unitTest( )

##############################################################################

class ALTestCaseALAction( ALTestCase ):
    """
    Total compatible with Jython
    """
    def process( self ):
#        self.addTestMethod( TestCaseALAction().testALAction )
        self.addTestMethod( TestCaseALAction().testALActionVerbose )
#        self.addTestMethod( TestCaseALAction().testALActionSUCCESS )
#        self.addTestMethod( TestCaseALAction().testALActionFAILURE )
#        self.addTestMethod( TestCaseALActionController().testALActionController )
#        self.addTestMethod( TestCaseALActionCluster().testALActionCluster )
#        self.addTestMethod( TestCaseALAction().testALActionMassiveParallelSUCCESS )
#        self.addTestMethod( TestCaseALActionMethod().testALActionMethod )
#        self.addTestMethod( TestCaseALActionMethodTimer().testALActionMethodTimer )
#        self.addTestMethod( TestCaseALActionSet().testALActionSet )
#        self.addTestMethod( TestCaseALActionSetAsynchronousSynchronous().testALActionSetSynchronous )
#        self.addTestMethod( TestCaseALActionSetAsynchronousSynchronous().testALActionSetAsynchronous )
#        self.addTestMethod( TestCaseALActionSet().testALActionSlots )




##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALAction = ALTestCaseALAction( "TestCase Action Classes" )
    oalTestCaseALAction.execute()

