#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *



class ALTestCaseALManagerProcess( ALTestCase ):
    """
    TestSuite for ALManagerProcess
    """


    def testKillProcess( self, _oalStrCommandLine ):
        oalStrCommand = _oalStrCommandLine
        ALVerbose.unitTest( "Kill the Process" )
        ALVerbose.unitTest( "Looking for the command line::[ " + oalStrCommand + " ]")

        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.killProcess( oalStrCommand )
        oStrStatus = oalManagerProcess.getStatus( oalStrCommand )
        ALVerbose.unitTest( "The Process Status is: " + oStrStatus )
        del oalManagerProcess



    def testStatus( self, _oalStrCommandLine ):
        oalStrCommand = _oalStrCommandLine
        ALVerbose.unitTest( "Getting The Status" )
        ALVerbose.unitTest( "Looking for the command line::[ " + oalStrCommand + " ]")

        oalManagerProcess = ALManagerProcess()
        oStrStatus = oalManagerProcess.getStatus( oalStrCommand )
        ALVerbose.unitTest( "The Process Status is: " + oStrStatus )
        del oalManagerProcess



    def testIsRunning( self, _oalStrCommandLine ):
        oalStrCommand = _oalStrCommandLine
        ALVerbose.unitTest( "Getting IsRUNNING" )
        ALVerbose.unitTest( "Looking for the command line::[ " + oalStrCommand + " ]")

        oalManagerProcess = ALManagerProcess()
        oBoolStatus = oalManagerProcess.isRunning( oalStrCommand )
        if (oBoolStatus==True):
            ALVerbose.unitTest( "[ok] The Process is Running" )
        else:
            ALVerbose.unitTest( "[FAIL] The Process is NOT Running or stopped!" )
        del oalManagerProcess


    def testPIDCommandLine( self, _oalStrCommandLine ):
        oalStrCommand = _oalStrCommandLine
        ALVerbose.unitTest( "Getting the PID" )
        ALVerbose.unitTest( "Looking for the command line::[ " + oalStrCommand + " ]")

        oalManagerProcess = ALManagerProcess()
        oiPid = oalManagerProcess.getPid( oalStrCommand )
        if (oiPid!=None):
            ALVerbose.unitTest( "[ok] The PID is: " + ALString( oiPid ) )
        else:
            ALVerbose.unitTest( "[WARNING] :: The PID is NOT Found or the process is not running!" )
        del oalManagerProcess



    def testALManagerProcessSoftware( self ):
        ALVerbose.unitTest( "Test ALManagerProcess Software" )

        oalStringCommand  = ALString( "firefox" )

        if (ALSystem.getOSName()=="Windows"):
             oalStringCommand = ALString( "C:\\Program Files\\Mozilla Firefox\\firefox.exe" )

#        oalSubProcess = ALSubProcess( oalStringCommand )
#        oalSubProcess.executeProcess()
#        ALTime.sleep( 1.0 ) # To wait the start up of the external process
#        oalListOutput = oalSubProcess.getOutput()
#        ALVerbose.unitTest( "Output of the firefox startup" )
#        for iLine in oalListOutput:
#            ALVerbose.unitTest( ALString( iLine ) )


        self.testPIDCommandLine( "firefox -help -bla --buuu" )
        self.testPIDCommandLine( "firefox.exe" )
        self.testPIDCommandLine( "firefox" )


        self.testIsRunning( "firefox -help -bla --buuu" )
        self.testIsRunning( "firefox.exe" )
        self.testIsRunning( "firefox" )


        self.testStatus( "firefox -help -bla --buuu" )
        self.testStatus( "firefox.exe" )
        self.testStatus( "firefox" )



        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  "firefox -help -bla --buuu" )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: firefox -help -bla --buuu is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: firefox -help -bla --buuu " )


        self.testKillProcess( "firefox -help -bla --buuu" )
        self.testKillProcess( "firefox.exe" )
        self.testKillProcess( "firefox" )




        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  "firefox -help -bla --buuu" )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: firefox -help -bla --buuu is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: firefox -help -bla --buuu " )




        ALVerbose.unitTest( "Printing the Process TABLE" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.control()
        oalManagerProcess.outputTable()
        oalManagerProcess.output( "testOutputDataProcessManager.xml" )




        ALVerbose.unitTest( "Testing the kill idiot Command" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.killProcessPid( 999999 )

        ALVerbose.unitTest( )



    def testALManagerProcessConsolePython( self ):
        ALVerbose.unitTest( "Test ALManagerProcess Console" )

        oalStringCommand  = ALString( "python -h" )

        if (ALSystem.getOSName()=="Windows"):
             oalStringCommand = ALString( "C:\\Program-Files\\Automation\\win32\\Program-Files\\Python-2.4.3\\python.exe -h" )

        oalSubProcess = ALSubProcess( oalStringCommand )
        oalSubProcess.executeProcess()
        ALTime.sleep( 1.0 ) # To wait the start up of the external process
        oalListOutput = oalSubProcess.getOutput()
        ALVerbose.unitTest( "Output of the python startup" )
        for iLine in oalListOutput:
            ALVerbose.unitTest( ALString( iLine ) )


        self.testKillProcess( oalStringCommand  )
        self.testKillProcess( "python.exe" )
        self.testKillProcess( "python" )



        self.testPIDCommandLine( oalStringCommand )
        self.testPIDCommandLine( "python.exe" )
        self.testPIDCommandLine( "python" )


        self.testIsRunning( oalStringCommand )
        self.testIsRunning( "python.exe" )
        self.testIsRunning( "python" )


        self.testStatus( oalStringCommand )
        self.testStatus( "python.exe" )
        self.testStatus( "python" )



        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  oalStringCommand )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: " + oalStringCommand + "  is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: " + oalStringCommand   )



        self.testKillProcess( oalStringCommand  )
        self.testKillProcess( "python.exe" )
        self.testKillProcess( "python" )


        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  oalStringCommand )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: " + oalStringCommand + "  is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: " + oalStringCommand   )



        ALVerbose.unitTest( "Printing the Process TABLE" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.control()
        oalManagerProcess.outputTable()
        oalManagerProcess.output( "testOutputDataProcessManager.xml" )




        ALVerbose.unitTest( "Testing the kill idiot Command" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.killProcessPid( 999999 )

        ALVerbose.unitTest( )


    def testALManagerProcessConsole( self ):
        ALVerbose.unitTest( "Test ALManagerProcess Console" )

        oalStringCommand  = ALString( "python -h" )

        if (ALSystem.getOSName()=="Windows"):
             oalStringCommand = ALString( "C:\\Program-Files\\Automation\\win32\\Program-Files\\Python-2.4.3\\python.exe -h" )

        oalSubProcess = ALSubProcess( oalStringCommand )
        oalSubProcess.executeProcess()
        ALTime.sleep( 1.0 ) # To wait the start up of the external process
        oalListOutput = oalSubProcess.getOutput()
        ALVerbose.unitTest( "Output of the python startup" )
        for iLine in oalListOutput:
            ALVerbose.unitTest( ALString( iLine ) )


        self.testKillProcess( oalStringCommand  )
        self.testKillProcess( "python.exe" )
        self.testKillProcess( "python" )



        self.testPIDCommandLine( oalStringCommand )
        self.testPIDCommandLine( "python.exe" )
        self.testPIDCommandLine( "python" )


        self.testIsRunning( oalStringCommand )
        self.testIsRunning( "python.exe" )
        self.testIsRunning( "python" )


        self.testStatus( oalStringCommand )
        self.testStatus( "python.exe" )
        self.testStatus( "python" )



        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  oalStringCommand )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: " + oalStringCommand + "  is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: " + oalStringCommand   )



        self.testKillProcess( oalStringCommand  )
        self.testKillProcess( "python.exe" )
        self.testKillProcess( "python" )


        ALVerbose.unitTest( "Getting The ALDataProcess Object" )
        oalManagerProcess = ALManagerProcess()
        oalDataProcess = oalManagerProcess.getDataProcess(  oalStringCommand )
        if (oalDataProcess!=None):
            oalDataProcess.outputFile( "testDataProcess.xml" )
            ALVerbose.unitTest( "Data Object for the Process: " + oalStringCommand + "  is Saved on the file testDataProcess.xml" )
        else:
            ALVerbose.unitTest( "No Data Object for the Process: " + oalStringCommand   )



        ALVerbose.unitTest( "Printing the Process TABLE" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.control()
        oalManagerProcess.outputTable()
        oalManagerProcess.output( "testOutputDataProcessManager.xml" )




        ALVerbose.unitTest( "Testing the kill idiot Command" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.killProcessPid( 999999 )

        ALVerbose.unitTest( )





    def testALManagerProcessConsole2( self ):
        ALVerbose.unitTest( "Test ALManagerProcess Console 2" )
        ALVerbose.unitTest( "Printing the Process TABLE" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.control()
     #   oalManagerProcess.outputTable()
        oalManagerProcess.output( "testOutputDataProcessManager2.xml" )


        ALVerbose.unitTest( "Testing the kill idiot Command" )
        oalManagerProcess = ALManagerProcess()
        oalManagerProcess.killProcessPid( 999999 )

        self.testIsRunning( "firefox" )
        self.testKillProcess( "firefox" )


        ALVerbose.unitTest( )


##############################################################################

    """
    Total compatible with Jython
    """
    def process( self ):
#        self.addTestMethod( self.testALManagerProcessSoftware )
#        self.addTestMethod( self.testALManagerProcessConsolePython )
#        self.addTestMethod( self.testALManagerProcessConsole )
        self.addTestMethod( self.testALManagerProcessConsole2 )


##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALManagerProcess = ALTestCaseALManagerProcess( "TestCase ALManagerProcess" )
    oalTestCaseALManagerProcess.execute()
