#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportAction      import *
from ALImportKernel      import *
from ALImportMath        import *
from ALImportSystem      import *


##############################################################################
# ALSubProcess

class ALTestCaseALSubProcess( ALTestCase ):

    def testALSubProcess(self):
        ALVerbose.unitTest( "UnitTest ALSubProcess" )

        oalStrPython = "C:/Program-Files/Automation/win32/Program-Files/python-2.5.2/python.exe"
        if (ALSystem.getOSName()!="Windows"):
            oalStrPython = "python"
        else:
            if (ALDiskExplorer.existFile(oalStrPython)==False):
                ALVerbose.warning(" Python install not found: " + oalStrPython )


        if(ALSystem.getInterpreter()=="Jython"):
            oalStrPython = "java.exe"
            oalStrCode   = " -h"
            if (ALSystem.getOSName()=="Linux"):
                oalStrPython = "java"
        else:
            oalStrCode   = ALDiskExplorer.getAbsolutePath( "./DataForTests/demoApplication.py" )


        oalCommand   = oalStrPython  + " " + oalStrCode


        ALVerbose.unitTest( "Test SubProcess" )

        ALVerbose.unitTest( "Please Wait - The demon run during 5 seconds" )
        oalSubProcess = ALSubProcess(  oalCommand, "./PythonJythonTestDataFor/SubProcess"  )
        oalSubProcess.executeProcess()

        if(ALSystem.getInterpreter()=="Jython"):
            ALTime.wait( 2.0 )


        oalListOutput2      = oalSubProcess.getOutput( )
        oalListOutputError2 = oalSubProcess.getOutputError()
        oalListInput2       = oalSubProcess.getInput()


        ALVerbose.unitTest( "Input to the Child Process: " )
        for i in oalListInput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output from the Child Process: " )
        for i in oalListOutput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output Error from the Child Process: " )
        for i in oalListOutputError2:
            ALVerbose.unitTest( i )


        oalStringReturnCode = oalSubProcess.getReturnCode()
        ALVerbose.unitTest( "Printing Return CODE: " + ALString(  oalStringReturnCode ) )


        del oalSubProcess
        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



    def testALSubProcessJavaFails(self):
        ALVerbose.unitTest( "UnitTest ALSubProcessJavaFails" )

        oalStrPython = "C:/Program-Files/Automation/win32/Program-Files/python-2.5.2/python.exe"
        if (ALSystem.getOSName()!="Windows"):
            oalStrPython = "python"
        else:
            if (ALDiskExplorer.existFile(oalStrPython)==False):
                ALVerbose.warning(" Python install not found: " + oalStrPython )

        oalStrCode   = ALDiskExplorer.getAbsolutePath( "./DataForTests/demoApplication.py" )
        oalCommand   = oalStrPython  + " " + oalStrCode


        ALVerbose.unitTest( "Test SubProcessJavaFails" )

        oalSubProcess = ALSubProcess(  oalCommand, "./PythonJythonTestDataFor/SubProcessJavaFails"  )
        oalSubProcess.executeProcess()

        if(ALSystem.getInterpreter()=="Jython"):
            ALTime.wait( AL_TIME_SECOND )


        oalListOutput2      = oalSubProcess.getOutput( )
        oalListOutputError2 = oalSubProcess.getOutputError()
        oalListInput2       = oalSubProcess.getInput()


        ALVerbose.unitTest( "Input to the Child Process: " )
        for i in oalListInput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output from the Child Process: " )
        for i in oalListOutput2:
            ALVerbose.unitTest( i )
        ALVerbose.unitTest( "Output Error from the Child Process: " )
        for i in oalListOutputError2:
            ALVerbose.unitTest( i )


        oalStringReturnCode = oalSubProcess.getReturnCode()
        ALVerbose.unitTest( "Printing Return CODE: " + ALString(  oalStringReturnCode ) )


        del oalSubProcess
        ALVerbose.unitTest()
        ALVerbose.unitTest("\n\n\n")



##############################################################################

    def process( self ):
        self.addTestMethod( self.testALSubProcess )
        self.addTestMethod( self.testALSubProcessJavaFails )



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    ALCompiler.accelerator()

    oalTestCaseALSubProcess = ALTestCaseALSubProcess( "TestCase ALSubProcess" )
    oalTestCaseALSubProcess.execute()
