#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportSystem      import *
from ALImportKernel      import *
from ALImportAction      import *
from ALImportMath        import *
from ALImportJython      import *

import java.applet                       as JyPackageApplet
import java.applet.Applet                as JyApplet
import java.awt                          as JyPackageAwt

"""
These Jython examples are extracted from the standart Jython distribution
source code. They are used here to test the integration of the PyAALib and Jython 2.2
"""
class TempConverter( JyObject ):
    """
    Python Inheritance from Java class
    """
    def __init__(self):
        self.setFahrenheit( 0.0 )

    def setFahrenheit(self, degrees):
        "@sig public void setFahrenheit(double degrees)"
        self.f = degrees
        self.c = (degrees-32.)/1.8

    def getFahrenheit(self):
        "@sig public double getFahrenheit()"
        return self.f

    def setCelsius(self, degrees):
        "@sig public void setCelsius(double degrees)"
        self.c = degrees
        self.f = degrees*1.8+32.

    def getCelsius(self):
        "@sig public double getCelsius()"
        return self.c

    def __repr__(self):
        return '<%.2g degrees fahrenheit == %.2g celsius>' % ( self.f, self.c )



class HelloWorld( JyApplet ):
    """
    Python Inheritance from Java class
    """
    def paint( self, g ):
        oString = ALSystem.getVersion()
        g.drawString( " Hello from Jython %s!" % oString, 100 , 100 )


def TestApplet( panel, size=None, name='AWT Tester' ):
    """
    Python Interface
    """
    import sys

    f = JyPackageAwt.Frame( name, windowClosing=lambda event: sys.exit(0) )
    if hasattr(panel, 'init'):
        panel.init()

    f.add('Center', panel)
    f.pack()
    if size is not None:
        f.setSize( apply( JyPackageAwt.Dimension, size ))
    f.setVisible(1)
    return f




##############################################################################
# Jython

class ALTestCaseJython( ALTestCase ):

    def testJython(self):
        ALVerbose.unitTest( "UnitTest Jython" )

        c = TempConverter()
        ALVerbose.unitTest( ALString( c ) )
        c.setCelsius(100)
        ALVerbose.unitTest( ALString( c ) )
        c.setCelsius(0)
        ALVerbose.unitTest( ALString( c ) )
        c.setFahrenheit(212)
        ALVerbose.unitTest( ALString( c ) )

        ALVerbose.unitTest( "Test import and print date information from java" )
        oStrFileName = ALString( "./DataForTests/ClassPluginFile.py" )
        ojFile = JyFile( oStrFileName )
        if (ojFile.exists()==True):
            ALVerbose.unitTest( "Test JyFile: The file exist " + oStrFileName )
        else:
            ALVerbose.unitTest( "Test JyFile: The file Does NOT exist " + oStrFileName )



        oStrFileName = ALString( "./DataForTests/BANANE.FILE" )
        ojFile = JyFile( oStrFileName )
        if (ojFile.exists()==True):
            ALVerbose.unitTest( "Test JyFile: The file exist - " + oStrFileName )
        else:
            ALVerbose.unitTest( "Test JyFile: The file Does NOT exist " + oStrFileName )

        ALVerbose.unitTest( )



    def testJythonApplet( self ):
        ALVerbose.unitTest( "Test testJythonApplet" )
        TestApplet( HelloWorld() )
        ALVerbose.unitTest( )



    def testJythonApplication( self ):
        ALVerbose.unitTest( "UnitTest ALFactoryPlugin - ALApplication:" )

        #Workaround to test the application log file in the GLOBAL UnitTest
        oalLogFileTmp = ALVerbose.m_oalLogFile


        # Workaround to have an command line to test the stuffs
        oalCommandLineArgument = ALCommandLineArgument( [ "--DEBUG", "--jython01", "--jython03", "--jython04", "--jython02" ] )


        # Testing Main Framework
        oalApplication = ALApplication( "Application-Test-JythonPlugins", "0.9" )
        oalApplication.setCommandLine( "--DEBUG --jython02 --jython03 --test1S --test --jython04 --jython01" )

        #Plugin search and load from files especified in this location
        oalApplication.addPluginFolder( "./DataForTests/FolderPlugins" )


        # Using a file to define the command line execution and mapping
        oalApplication.setCommandLineInterfaceFile( "./DataForTests/FolderPlugins/commandLineInterfaceJython.xml"  )


        # Test
        oalApplication.execute()


        # Workaround to test the application log file in the GLOBAL UnitTest - end
        ALVerbose.m_oalLogFile = oalLogFileTmp
        ALVerbose.unitTest()






##############################################################################

    def process( self ):
        self.addTestMethod( self.testJython )
#        self.addTestMethod( self.testJythonApplet )
        self.addTestMethod( self.testJythonApplication )



##############################################################################


if __name__ == '__main__':


    oalTestCaseJython = ALTestCaseJython( "TestCase Jython" )
    oalTestCaseJython.execute()
