#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALImportPython    import  PySys
from ALDefinition      import  *
from ALTime            import  ALTime
from ALVerbose         import  ALVerbose
from ALSystem          import  ALSystem
from ALString          import  ALString
from ALAssert          import  ALAssert
from ALTrace           import  ALTrace
from ALDebug           import  ALDebug
from ALLogFile         import  ALLogFile
from ALTest            import  ALTest
from ALManagerTest     import  ALManagerTest
from ALInformationTest import  ALInformationTest



class ALTestSuite( ALTest ):
    """
    Interface for Tests Suite

    # To be modified by the user with its methods
    def process( self ):
    def preProcess( self ):
    def postProcess( self ):

    """
    def __init__( self, _oalStringTestName = None ):
        ALTest.__init__( self,  _oalStringTestName )
        self.m_oNumberTest        = 0
        self.m_oNumberTestTotal   = 0
        self.m_oNumberTestSuccess = 0
        self.m_oNumberTestFailure = 0
        if (_oalStringTestName != None ):
            oalStringNameLog = self.m_oalStringTestName + ALString( ".log" )
            ALDebug.setDebugLogFile( oalStringNameLog )
        self.controlCommandLine()
        self.oalTime = ALTime()
        ALAssert.setAssertionModeOn()



    def __patchEclipseConsolePipe( self ):
        """
        Fix to Eclipse/Python console output not close the pipe quickly
        """
        ALTime.wait(1.0)



    def outputReport(self):
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( self.getTestName() + " - END Status [OK]" )
        ALManagerTest.create()
        ALManagerTest.outputScreen()
        ALVerbose.unitTest( "###################################################################" )



    def outputReportCumulate(self):
        ALVerbose.outputActionLog()
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "###################################################################" )
        ALVerbose.unitTest( "Final Summary Report:" )
        ALVerbose.unitTest( "                     Test Suite Core: [ PyAALib-JyAALib " + ALString( AL_VERSION ) +" ]" )
        ALVerbose.unitTest( "                            Compiler: [ " + ALString( ALSystem.getCompilerName() ) +" ]" )
        ALVerbose.unitTest( "                     Virtual Machine: [ " + ALString( ALSystem.getMachineVirtuel() ) +" ]" )
        ALVerbose.unitTest( "                              System: [ " + ALSystem.getOSName()+" "+ALSystem.getOSType()+" "+ALSystem.getOSVersion()+" "+ALSystem.getMachineVersion()+" ]" )
        ALVerbose.unitTest( "                                Date: " + ALTime.getTimeString() )
        ALVerbose.unitTest( "                     Total TestCases: " + ALString( ALManagerTest.getNumberTestCumulative() ) )
        ALVerbose.unitTest( "           Total TestCases [SUCCESS]: " + ALString( ALManagerTest.getNumberTestSuccessCumulative() ) )
        ALVerbose.unitTest( "              Total TestCases [FAIL]: " + ALString( ALManagerTest.getNumberTestFailureCumulative() ) )
        ALVerbose.unitTest( "                 [Total TestMethods]: " + ALString( ALManagerTest.getNumberTestMethodCumulative() ) )
        ALVerbose.unitTest( "          Acid Index [Success Ratio]: [ " + ALString( ALManagerTest.getAcidIndex() ) +"% ]" )

        ofTime = self.oalTime.getDuration()
        ALVerbose.unitTest( "                             Runtime: " + ALString( ofTime ) + " [s]" )
        ALVerbose.unitTest( "                                 Run: " + ALTime.getTimeStringPSP( ofTime )  )
        ALVerbose.screen(   " " )
        ALVerbose.screen(   " " )




    def processKernel( self ):
        ALVerbose.aalibDEBUG( "Execution: ALTestSuite.processKernel()" )
        ALManagerTest.create()
        self.m_oNumberTestTotal   = self.m_oalListTest.getNumberObjects( )
        self.m_oNumberTestSuccess = 0
        self.m_oNumberTestFailure = 0
        self.m_oNumberTest        = 0
        for oTestMethod in self.m_oalListTest:
            self.m_oNumberTest += 1
            oalInformationTest = ALInformationTest( self.patchMethodName( oTestMethod ), self.m_oNumberTest )
            ALVerbose.screen( " " )
            ALVerbose.unitTest( "###################################################################" )
            ALVerbose.unitTest( "TEST [ " + ALString( oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]" )
            ALVerbose.unitTest( " ")
            ALManagerTest.addInformationTest( oalInformationTest )
            try:
                try:
                    oTestMethod()
                    oalInformationTest.setTerminate()
                    oalInformationTest.setSuccess()
                    ALVerbose.unitTest( "TEST [" + ALString(  oalInformationTest.getTestNumber() ) + " ] : [SUCCESS]" )
                except Exception, opyException:
                    oalInformationTest.setTerminate()
                    oalInformationTest.setFailure()
                    oalInformationTest.setException( ALString(opyException) )
                    ALVerbose.unitTest( "Test Failure = [ " + oalInformationTest.getException() + " ] Raised"  )
                    ALVerbose.unitTest( "TEST [ " + ALString(  oalInformationTest.getTestNumber() ) + " ] : [ " + oalInformationTest.getTestName() + " ]: [FAILURE]" )
                    self.raiseControl()

            finally:
                ALVerbose.unitTest( "###################################################################" )
                ofTime = oalInformationTest.getRunTime()
                ALVerbose.unitTest( "Test - " + ALString( oalInformationTest.getTestNumber() ) + " : Module Runtime: " + ALString( ofTime ) + " [s]" )
                ALVerbose.screen( " " )

        self.m_oNumberTestSuccess = ALManagerTest.getNumberTestSuccess()
        self.m_oNumberTestFailure = ALManagerTest.getNumberTestFailure()



    def execute( self ):
        ALVerbose.unitTest( "STARTING::" + self.getTestName() )
        ALVerbose.unitTest( "###################################################################" )
        try:
            try:
                self.executeKernel()
            except:
                self.raiseControl()
        finally:
            self.outputReport()
            self.outputReportCumulate()
            self.outputProfile()
            self.__patchEclipseConsolePipe()


##############################################################################