#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDefinition                  import *
from ALString                      import ALString
from ALVerbose                     import ALVerbose
from ALSlot                        import ALSlot
from ALSignal                      import ALSignal
from ALThread                      import ALThread
from ALDebug                       import ALDebug
from ALTrace                       import ALTrace
from ALTime                        import ALTime
from ALExceptionSignal             import ALExceptionSignal
from ALApplicationCommunicator     import ALApplicationCommunicator


class ALThreadControl( ALThread ):
    """
    Interface for AL thread control objects.
    """
    def __init__( self ):
        ALThread.__init__( self )
        self.m_oalSlotInit      = ALSlot()
        self.m_oalSlotRun       = ALSlot()
        self.m_oalSlotEnd       = ALSlot()
        self.m_oalSlotPause     = ALSlot()
        self.m_oalSlotResume    = ALSlot()
        self.m_oalSlotAbort     = ALSlot()
        self.m_oalSlotException = ALSlot()
        self.m_oalSlotError     = ALSlot()


        # Basic Event Callback Connections
        self.m_oalSlotEnd.connect( self.endOfAction )
        self.m_oalSlotAbort.connect( self.verboseAbort, ALSignal.abort() )
        self.m_oalSlotException.connect( self.verboseException, ALSignal.exception() )

        self.m_bIsEnded = False
        self.m_bIsAbort = False
        self.m_bIsPause = False

        # To be controled by the ALApplication framework
        self.m_bRegisterAction = True

        self.m_oalRunTimeOut        = AL_TIME_SECOND * 6000.0  #(100 minutes)
        self.m_oalTimeDuration      = 0.0
        self.m_oalTime              = ALTime()



    def getTimeDuration(self):
        self.m_oalTimeDuration = self.m_oalTime.getDuration()
        return self.m_oalTimeDuration


    def getTimeRun(self):
        return self.m_oalTime


    def getRegisterAction( self ):
        return self.m_bRegisterAction


    def setRegisterAction( self ):
        self.synchronizeOn()
        self.m_bRegisterAction = True
        self.synchronizeOff()

    def setNoRegisterAction( self ):
        self.synchronizeOn()
        self.m_bRegisterAction = False
        self.synchronizeOff()


    def synchronizeOn( self, _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            ALVerbose.aalibDEBUGID( self.getID(), "ALAction::synchronizeOn - " + ALString( _oalObject.getID() ) )
            ALVerbose.aalibDEBUGID( self.getID(), "ALAction::synchronizeOn - Semaphore is ON" )
        self.m_opySemaphore.acquire()


    def synchronizeOff( self, _oalObject = None ):
        # Option used to debug thread locks
        if (_oalObject!=None):
            ALVerbose.aalibDEBUGID( self.getID(), "ALAction::synchronizeOff - " + ALString( _oalObject.getID() ) )
            ALVerbose.aalibDEBUGID( self.getID(), "ALAction::synchronizeOff - Semaphore is OFF" )
        self.m_opySemaphore.release()



    def connectInit( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotInit.connect( _oMethod )
        self.synchronizeOff()

    def connectPause( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotOause.connect( _oMethod )
        self.synchronizeOff()

    def connectResume( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotResume.connect( _oMethod )
        self.synchronizeOff()

    def connectAbort( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotAbort.connect( _oMethod, ALSignal.abort() )
        self.synchronizeOff()

    def connectException( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotException.connect( _oMethod )
        self.synchronizeOff()

    def connectError( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotError.connect( _oMethod )
        self.synchronizeOff()

    def connectEnd( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotEnd.connect( _oMethod )
        self.synchronizeOff()

    def connectExecute( self, _oMethod ):
        self.synchronizeOn()
        if (_oMethod != None):
            self.m_oalSlotRun.connect( _oMethod )
        self.synchronizeOff()

    def connectRun( self, _oMethod ):
        self.connectExecute( _oMethod )



    def threadControl( self ):
        # Patch for Java-Jython
        oTime    = float( self.getTimeDuration() )
        oTimeOut = float( self.getTimeOut() )
        # controls the timeOut method
        if ( oTime>oTimeOut ):
        #if (self.getTimeDuration()>self.getTimeOut()): -> It can not run on Jython
            ALVerbose.aalibDEBUGID( self.getID(), ALString( "Thread::STOPPED under control in threadControl() - TimeOUT: " + self.getName() + " " + self.getID() )  )
            raise ALExceptionSignal( ALSignal.abort())
        if (self.isAbort()==True):
            ALVerbose.aalibDEBUGID( self.getID(), ALString( "Thread::STOPPED under control: " + self.getName() + " " + self.getID() )  )
            raise ALExceptionSignal( ALSignal.abort() )
        if (self.isPause()==True):
            ALVerbose.aalibDEBUGID( self.getID(), ALString( "Thread::PAUSED under control: " + self.getName() + " " + self.getID() )  )
            ALTime.wait( AL_TIME_MINIMUM * 10.0 )
        # Experimental
        oalApplicationSignal = ALApplicationCommunicator.getSignal()
        if (oalApplicationSignal is not None):
            oalSignal = oalApplicationSignal.getSignal()
            if (isinstance( oalSignal, ALSignal)):
                oalStringInformation = oalApplicationSignal.getInformation()
                if ( oalSignal.getSignal()==AL_ACTION_SIGNAL_ABORT or oalSignal.getSignal()==AL_ACTION_SIGNAL_ERROR ) :
                    ALVerbose.aalibDEBUGID( self.getID(), ALString( oalSignal.getSignal() ) + "::" + oalStringInformation )
                    raise ALExceptionSignal( oalSignal )
            else:
                pass



    def verboseAbort( self, _oalObject = None ):
        self.m_bIsAbort = True



    def verboseException( self, _oalObject = None ):
        self.m_bIsAbort = True




    def threadControlTimer( self, _oalTimeDelay ):
        """
        Only to be used in internal run kernels
        Otherwise it generates an exception to
        stop the flow
        """
        if (self.isAlive()!=True):
            ALTime.wait( AL_TIME_MINIMUM * 2.0 )
        oalTime = ALTime()
        obStatus = self.isAlive()
        while ( obStatus==True ):
            self.threadControl()
            ALTime.wait( AL_TIME_MINIMUM )
            # Patch for Java-Jython
            oTime    = float( oalTime.getDuration() )
            oTimeOut = float( _oalTimeDelay )
            if ( oTime>oTimeOut ):
                obStatus = False
            if (self.isAlive()!=True):
                obStatus = False
#            if (oalTime.getDuration()>_oalTimeDelay):
#                obStatus = False



    def synchronizeUntilTimeOut( self, _oalTimeDelay ):
        self.threadControlTimer( _oalTimeDelay )
        
        


    def synchronize( self, _oalObject = None ):
        # Delay to wait lazy threads
        if (self.isRunning()!=True):
            ALTime.wait( AL_TIME_MINIMUM * 5.0 )

        oalTime = ALTime()
        while( self.isRunning()==True ):
            ALTime.wait( AL_TIME_MINIMUM )
            # Patch for Java-Jython
            oTime    = float( oalTime.getDuration() )
            oTimeOut = float( self.getTimeOut() )
            if ( oTime>oTimeOut ):
                self.close()
#            if (oalTime.getDuration()>self.getTimeOut()):
#                self.close()




    def abort( self, _oalObject = None ):
        self.m_bIsAbort = True


    def stop( self, _oalObject = None ):
        self.abort()


    def close( self, _oalObject = None ):
        self.stop()


    def endOfAction( self, _oalObject = None ):
        self.m_bIsEnded = True


# To be deprecated
    def resume( self, _oalObject = None ):
        self.m_oalSlotResume.call()
        self.m_bIsPause = False


    def pause( self, _oalObject = None ):
        self.m_oalSlotPause.call()
        self.m_bIsPause = True


    def isAbort( self ):
        return self.m_bIsAbort


    def isPause( self ):
        return self.m_bIsPause


    def isRunning( self ):
        return self.isAlive()


    def isEnded( self ):
        return self.m_bIsEnded


    def isExecute( self ):
        return self.isRunning()



    def setTimeOut( self, _ofTime ):
        self.m_oalRunTimeOut = float( _ofTime )


    def getTimeOut( self ):
        return self.m_oalRunTimeOut


##############################################################################

