#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALDebug                   import ALDebug
from ALImportPython            import PyMinidom
from ALImportPython            import PyXmlNode
from ALList                    import ALList
from ALObject                  import ALObject
from ALSlot                    import ALSlot
from ALStreamFile              import ALStreamFile
from ALString                  import ALString
from ALThreadingCondition      import ALThreadingCondition
from ALToken                   import ALToken
from ALTrace                   import ALTrace
from ALVerbose                 import ALVerbose
from ALXmlNode                 import ALXmlNode


class ALXml( ALObject ):
    """
    Interface for ALXml objects.
    ATTENTION:: must be transformed to thread safety
    """

    m_oalThreadingConditionXml = ALThreadingCondition()


    def __init__( self, _oalString = "document" ):
        ALObject.__init__( self )
        self.m_oPyXmlDocument = None
        oPyImplementation = PyMinidom.getDOMImplementation()
        self.m_oPyXmlDocument = oPyImplementation.createDocument(None, _oalString, None)
        self.m_oPyXmlDocumentRoot = self.m_oPyXmlDocument.documentElement


    def __controlRaise( self ):
        oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
        ALDebug.controlDebug( self, oBoolStatus )


    def getXmlDocument( self ):
        return self.m_oPyXmlDocument


    def output( self, _oalStringName, _bPrintType = False, _oalStringEncodingType = "ISO-8859-1" ): # or using "utf-8"
        if (self.m_oPyXmlDocument==None):
            return
        self.synchronizeOn()
        oalStreamFile = ALStreamFile( _oalStringName, "w" )
        # Here to bypass error in the python xml module when compiled with py2exe
        try:
            if (_bPrintType == False ) :
                oalStreamFile.write( self.m_oPyXmlDocument.toxml( _oalStringEncodingType )  )
            else:
                oalStreamFile.write( self.m_oPyXmlDocument.toprettyxml( "\t", "\n", _oalStringEncodingType )  ) # it creates a bug in komodo kpf file - tests
        except:
            oalStreamFile.write( self.m_oPyXmlDocument.toxml() )

        oalStreamFile.close()
        self.synchronizeOff()



    def input( self,  _oalStrFileName ):
        self.synchronizeOn()
        self.m_oPyXmlDocument = ALXml.__parseXmlFile( _oalStrFileName )
        self.synchronizeOff()


    def createXml( _oalStrFileName ):
        return ALXml.__parseXmlFile( _oalStrFileName )
    createXml = staticmethod( createXml )


    def createXmlString( _oalString ):
        return ALXml.__parseXmlString( _oalString )
    createXmlString = staticmethod( createXmlString )



    def __parseXmlFile( _oalStrFileName ):
        oXml = None
        ####################################################################
#        # Nasty error on Jython  Java sax implementation....It waits a str pure string!!!!
#          [ERROR]: File  :: C:/Program-Files/Automation/win32/Program-Files/jython-2.2rc3/Lib/xml/sax/drivers2/drv_javasax.py
#          [ERROR]: Method:: parse
#          [ERROR]: Line  :: 141
#          [ERROR]: Object:: self._parser.parse(JyInputSourceWrapper(source))
#          [ERROR]: java.net.MalformedURLException: java.net.MalformedURLException: unknown protocol: c
#
        oalString = ALString( _oalStrFileName )
        oPyString = oalString.getPythonString()
        #####################################################################
        try:
            oXml = PyMinidom.parse( oPyString )
        except:
            ALVerbose.error( "ALXml::__parseXmlFile() = parser Failed" )
            ALVerbose.error( "File Parser Error :: " + ALString( _oalStrFileName ) )
            oBoolStatus =  ALDebug.isDebugModeOn()
            ALDebug.controlDebug( None, oBoolStatus )
        return oXml
    __parseXmlFile = staticmethod( __parseXmlFile )


    def __parseXmlString( _oalString ):
        oXml = None
        try:
            oXml = PyMinidom.parseString( _oalString )
        except:
            ALVerbose.error( "ALXml::__parseXmlString() = parser Failed" )
            oBoolStatus =  ALDebug.isDebugModeOn()
            ALDebug.controlDebug( None, oBoolStatus )
        return oXml
    __parseXmlString = staticmethod( __parseXmlString )


    def addComment( self, _oalString ):
        self.synchronizeOn()
        oPyXmlComment = self.m_oPyXmlDocument.createComment( ALString( _oalString ) )
        self.m_oPyXmlDocumentRoot.appendChild(oPyXmlComment)
        self.synchronizeOff()



    def addText( self, _oalString ):
        self.synchronizeOn()
        oPyXmlText = self.m_oPyXmlDocument.createTextNode( ALString( _oalString ) )
        self.m_oPyXmlDocumentRoot.appendChild(oPyXmlText)
        self.synchronizeOff()



    def close( self ):
        self.synchronizeOn()
        if (self.m_oPyXmlDocument!=None):
            self.m_oPyXmlDocument.unlink()
        self.synchronizeOff()


    def destroy( self ):
        self.close()



    def addObject( self, _oalObject ):
        self.synchronizeOn()
        _oalObject.outputXml( self, self.m_oPyXmlDocumentRoot )
        self.synchronizeOff()

    # Clone methods from Document() xml class
    def createElement( self, _oToken ):
        return self.m_oPyXmlDocument.createElement( _oToken )

    def createTextNode( self, _oTextNode ):
        return self.m_oPyXmlDocument.createTextNode( _oTextNode )


    def rootNode( self ):
        return self.m_oPyXmlDocumentRoot





    def addTokenValue( self, _oalToken, _oalString ):
        self.synchronizeOn()
        if ( _oalString!=None ):
            oalXmlNodeString = self.m_oPyXmlDocument.createTextNode( ALString( _oalString ) )
            oalXmlNode = self.m_oPyXmlDocument.createElement( ALToken( _oalToken ) )
            oalXmlNode.appendChild( oalXmlNodeString )
            self.m_oPyXmlDocumentRoot.appendChild( oalXmlNode  )
        self.synchronizeOff()




    # Only in the main branch of the tree
    # WARNING: returns every time the first object!!!
    def getTokenValue( self, _oalToken ):
        return ALXml.getElementTokenValue( self.m_oPyXmlDocumentRoot, _oalToken )




    # Looking for ONE object XmlElements in the main branch of the tree
    # WARNING: returns every time the first object!!!
    def getElement( self, _oalToken ):
        self.synchronizeOn()
        oalXmlElement = None

        # Trick to bypass bad Xml documents
        if ALToken( self.m_oPyXmlDocumentRoot.tagName ) == ALToken( _oalToken ):
            oalXmlElement = self.m_oPyXmlDocumentRoot
            self.synchronizeOff()
            return oalXmlElement
        else :
            self.synchronizeOff()
            return ALXml.getElementObject( self.m_oPyXmlDocument, _oalToken )

        return oalXmlElement




    def removeElement( self, _oalToken ):
        oalXmlElement = self.getElement( _oalToken )
        self.synchronizeOn()
        if (oalXmlElement != None):
            self.m_oPyXmlDocumentRoot.removeChild( oalXmlElement  )
        self.synchronizeOff()



    # Looking for ONE XmlElement from any start node (sub-points)
    # WARNING: returns every time the first object!!!
    def getElementObject( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalXmlElement = None
        for inode in _oalXmlElement.getElementsByTagName( ALToken( _oalToken ) ):
            oalXmlElement = inode

            # Bad....
            ALXml.m_oalThreadingConditionXml.synchronizeOff()
            return oalXmlElement
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalXmlElement
    getElementObject = staticmethod( getElementObject )






    # Looking for a list of XmlElements in the main branch of the tree
    def getElementList( self, _oalToken ):
        self.synchronizeOn()
        oalListXmlElement = None
        # Trick to bypass bad Xml documents
        if ALToken( self.m_oPyXmlDocumentRoot.tagName ) == ALToken( _oalToken ):
            oalListXmlElement = ALList()
            oalListXmlElement.add( self.m_oPyXmlDocumentRoot )
        else :
            self.synchronizeOff()
            return ALXml.getElementListObject( self.m_oPyXmlDocumentRoot, _oalToken )
        self.synchronizeOff()
        return oalListXmlElement




    # Looking for VALUE of XmlElement from any start node (sub-points)
    # WARNING: returns every time the first object!!!
    def getElementTokenValue( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalStringValue = None
        for inode in _oalXmlElement.childNodes:
            if inode.nodeType==PyXmlNode.ELEMENT_NODE:
                if ( ALToken( inode.nodeName ) == ALToken( _oalToken ) ):
                    for jnode in inode.childNodes:
                        if jnode.nodeType==PyXmlNode.TEXT_NODE:
                            oalStringValue = ALString( jnode.data )
                            # cleaning the xml errors from the python code!!!!!!
                            oalStringValue = oalStringValue.cleanXml()
                            # BAD....but runs
                            ALXml.m_oalThreadingConditionXml.synchronizeOff()
                            return oalStringValue
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalStringValue
    getElementTokenValue = staticmethod( getElementTokenValue )


    # Looking for a list of XmlElements from any start node (sub-points)
    def getElementListObject( _oalXmlElement, _oalToken ):
        ALXml.m_oalThreadingConditionXml.synchronizeOn()
        oalListXmlElement = None
        for inode in _oalXmlElement.childNodes:
            if inode.nodeType==ALXmlNode.ELEMENT_NODE:
                if ( ALToken( inode.nodeName ) == ALToken( _oalToken ) ):
                    if ( oalListXmlElement==None ):
                        oalListXmlElement = ALList()
                    oalListXmlElement.add( inode )
        ALXml.m_oalThreadingConditionXml.synchronizeOff()
        return oalListXmlElement
    getElementListObject = staticmethod( getElementListObject )




    #Used to fix the "m_"  of AALib class attribute scheme
    def getAtributeToken( _oalToken ):
        oalToken = ALToken( _oalToken )
        if oalToken[0]=="m" and oalToken[1]=="_" :
            #oalToken.replace( "m_", "" ) # It can generate a problem in complex names
            oalTokenNew = oalToken[2:]
            oalToken = oalTokenNew
        return oalToken
    getAtributeToken = staticmethod( getAtributeToken )






##############################################################################
