#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################

from ALObject        import ALObject
from ALVerbose       import ALVerbose
from ALString        import ALString
from ALTime          import ALTime
from ALDiskExplorer  import ALDiskExplorer
from ALStreamFile    import ALStreamFile
from ALList          import ALList

import ALExternalPackGenerateDS  as ALPackXsd


class ALXsd( ALObject ):
    """
    Interface for Xsd objects.
    """

    def __init__( self ):
        ALObject.__init__( self )


    def generateModule( self, _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace = "xs:" ):
        oalXsdFileNameTemp = self.setSchemaLocation( _oalXsdFileNameInput, _nameSpace )
        ALPackXsd.generateClassDefinition( _oalPyFileNameOutput, oalXsdFileNameTemp, "xsd:" )
        ALDiskExplorer.deleteFile( oalXsdFileNameTemp )




    def generateModuleBuffer( self, _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace = "xs:" ):
        """
        Not yet Full Compatible with Jython2.2
        """
        try:
            oalXsdStringTemp = self.setSchemaLocationBuffer( _oalXsdFileNameInput, _nameSpace )
            #Path to fix possible bug on Java-Jython: some lazy process or disk writing???
            ALTime.wait(0.1)
            ALPackXsd.generateClassDefinitionBuffer( _oalPyFileNameOutput, oalXsdStringTemp, "xsd:" )
        except:
            ALVerbose.aalibDEBUG( "[Warning] ALXsd.generateModuleBuffer() - Jython2.2 Limitation" )
            self.generateModule( _oalPyFileNameOutput, _oalXsdFileNameInput, _nameSpace )






    def setSchemaLocationBuffer( self, _oalXsdFileName, _nameSpace ):
        oalXsdString = ALString.empty()

        oalFolder = ALDiskExplorer.getFolderName( _oalXsdFileName )

        oalStream = ALStreamFile( _oalXsdFileName, "r" )
        oalList = oalStream.getLines()
        oalStream.close()

        oalListList = ALList()

        oalListLine = ALList()
        for i in oalList:
            istr = ALString( i ).replace( "xs:", "xsd:" )
            if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                oalListLine.append( istr )
            if (istr.existSubString("schemaLocation")==True):
                oalListList.append( self.__getSchemeLocationData( istr, oalFolder ) )


        oalXsdString = ALString( "<?xml version=\"1.0\"?>\n" )
        oalXsdString = oalXsdString + ALString( "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n" )
        for i in oalListLine:
            oalXsdString = oalXsdString + ALString( i )
        for iList in oalListList:
            oalXsdString = oalXsdString + ALString( "\n\n" )
            for j in iList:
                oalXsdString = oalXsdString + ALString(  j )
        oalXsdString = oalXsdString + ALString( "\n" )
        oalXsdString = oalXsdString + ALString( "</xsd:schema>" )

        return oalXsdString




    def setSchemaLocation( self, _oalXsdFileName, _nameSpace ):
        oalXsdFileName = ALTime.getTimeName() + ALString( "-xsdTemp.xsd" )

        oalFolder = ALDiskExplorer.getFolderName( _oalXsdFileName )

        oalStream = ALStreamFile( _oalXsdFileName, "r" )
        oalList = oalStream.getLines()
        oalStream.close()

        oalListList = ALList()

        oalListLine = ALList()
        for i in oalList:
            istr = ALString( i ).replace( "xs:", "xsd:" )
            if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                oalListLine.append( istr )
            if (istr.existSubString("schemaLocation")==True):
                oalListList.append( self.__getSchemeLocationData( istr, oalFolder ) )

        oalStream = ALStreamFile( oalXsdFileName )
        oalStream.write( "<?xml version=\"1.0\"?>\n" )
        oalStream.write( "<xsd:schema xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\">\n\n" )
        for i in oalListLine:
            oalStream.write( i )
        for iList in oalListList:
            oalStream.write( "\n\n" )
            for j in iList:
                oalStream.write( j )
        oalStream.write( "\n" )
        oalStream.write( "</xsd:schema>" )
        oalStream.close()

        return oalXsdFileName




    def __getSchemeLocationData( self, _oalXsdLine, _oalFolder ):
        oalString = ALString( _oalXsdLine )
        oalList = oalString.split( "\"" )
        if (_oalFolder==ALString.empty()):
            _oalFolder = "."

        oalListLine = ALList()
        oalString = _oalFolder + "/" + ALString( oalList[1] )

        if (ALDiskExplorer.existFile(oalString)==True):
            oalStream = ALStreamFile( oalString, "r" )
            oalList = oalStream.getLines()
            oalStream.close()
            oalListList = ALList()
            for i in oalList:
                istr = ALString( i ).replace( "xs:", "xsd:" )
                if ( istr.existSubString("xml version=")==False and istr.existSubString("schema")==False and istr.existSubString("schemaLocation")==False):
                    oalListLine.append( istr )
                if (istr.existSubString("schemaLocation")==True):
                    oalListList.append( self.__getSchemeLocationData( istr, _oalFolder ) )

            for iList in oalListList:
                for j in iList:
                    oalListLine.append( j )
        else:
            ALVerbose.warning( "ALXsd::__getSchemeLocationData = File not found: ", oalString )

        return oalListLine



##############################################################################