#############################################################################
#
#    Project:  PyAALib-JyAALib Asynchronous Action Library Project
#              http://pyaalib.sourceforge.net/
#              http://jyaalib.sourceforge.net/
#
#
#    License:  PyAALib-JyAALib Asynchronous Action Library
#    Copyright (C) 2005, 2006, 2007, 2008, 2009  Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#
#    This program library is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#
#############################################################################
#
#       File: 
#    Version: 20090428 - Candidate Release 7
#     Author: Romeu Andre' Pieritz (romeu.pieritz@gmail.com)
#
#############################################################################


from ALObject        import ALObject
from ALVerbose       import ALVerbose
from ALString        import ALString
from ALTime          import ALTime
from ALDebug         import ALDebug
from ALDiskExplorer  import ALDiskExplorer
from ALFile          import ALFile
from ALXsd           import ALXsd


class ALXsdDataBinding( ALXsd ):
    """
    Interface for Xsd objects.
    """

    def __init__( self, _oalStrXsdFileName, _oalStrModuleFileName = None ):
        ALXsd.__init__( self )
        self.m_oalStrXsdFileName    = ALDiskExplorer.normalize(  _oalStrXsdFileName )
        if (_oalStrModuleFileName==None):
            oalNameModule = ALFile.getBaseName( self.m_oalStrXsdFileName )
            self.m_oalStrModuleFileName = ALFile.setExtension( oalNameModule, ".py" )
        else:
            self.m_oalStrModuleFileName =  ALFile.setExtension( _oalStrModuleFileName, ".py" )



    def generateModuleEngine( self, _oalStrModuleFileName, _oalStrXsdFileName ):
        self.generateModuleBuffer( _oalStrModuleFileName, _oalStrXsdFileName )



    def getObject( self, _oalStrXmlFileName = None, _oalStrModuleFileName = None, _oStringNameClass = None ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """
        return self.getObjectParser( _oalStrXmlFileName, _oalStrModuleFileName, _oStringNameClass )



    def getObjectClass( self, _oStringNameClass ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """
        return self.getObjectParser( None, None, _oStringNameClass )




    def getObjectParser( self, _oalStrXmlFileName = None,
                         _oalStrModuleFileName = None, _oStringNameClass = None ):
        """
        inputs: ( xml file name, module.py file name, class to import )
        """

        oalObject = None

        if (_oalStrModuleFileName==None):
            oalStrModuleFileName = self.m_oalStrModuleFileName
        else:
            oalNameModule = ALFile.getBaseName( _oalStrModuleFileName )
            oalStrModuleFileName = ALFile.setExtension( oalNameModule, ".py" )

        if (ALDiskExplorer.existFile( oalStrModuleFileName )==True):
           # print oalStrModuleFileName
            ALDiskExplorer.deleteFile( oalStrModuleFileName )


        try:
            self.generateModuleEngine( oalStrModuleFileName, self.m_oalStrXsdFileName )
            oNameModule = ALFile.getBaseName( oalStrModuleFileName )
            oModule = __import__( oNameModule )

            if (_oStringNameClass is not None):
                oStringNameClass = _oStringNameClass
            else:
                oStringNameClass = oNameModule


            if (_oalStrXmlFileName!=None):
                oalStrXmlFileName = ALDiskExplorer.normalize( _oalStrXmlFileName )
                if (ALDiskExplorer.existFile( oalStrXmlFileName )==True):
                    oalObject = oModule.unMarshalling( oalStrXmlFileName )
                else:
                    oalObject = oModule.__dict__[ oStringNameClass ]()
            else:
                oalObject = oModule.__dict__[ oStringNameClass ]()

        except:
            raise
            ALVerbose.warning("ALXsdDataBinding::getObject = problem to generate the module: " + ALString( oalStrModuleFileName ) )
            oBoolStatus =  ALDebug.isDebugModeOn() # bug in jython version
            ALDebug.controlDebug( self, oBoolStatus )



        return oalObject


##############################################################################