#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteBestv10.py 752 2009-06-04 09:51:15Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath




class EDTestCasePluginExecuteBestv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginBestv10", "EDPluginBest-v1.0", _oalStringTestName )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataBestv10Input_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataBestv10Output_reference.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        edStringBestVersion = self.getPlugin().getStringVersion()

        if( edStringBestVersion.existSubString( "3.1.0" )) :
            self.setReferenceDataOutputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataBestv10Output_reference.xml"))
        elif( edStringBestVersion.existSubString( "3.2.0" )) :
            self.setReferenceDataOutputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataBestv10Output_reference_BESTv320.xml"))
        
        # Checks the expected result
        strExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )    
        EDVerbose.DEBUG("Checking obtained result...") 
                
        from XSDataBestv10 import XSDataBestOutput
        xsDataOutputExpected = XSDataBestOutput.parseString( strExpectedOutput ) 
        xsDataOutputObtained = XSDataBestOutput.parseString( strObtainedOutput )
        
        EDAssert.equal( xsDataOutputExpected.marshal(), xsDataOutputObtained.marshal() )
        
        
    def postProcess( self ):
        """
        """
        EDTestCasePluginExecute.postProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteBestv10 = EDTestCasePluginExecuteBestv10( "EDTestCasePluginExecuteBestv10" )
    edTestCasePluginExecuteBestv10.execute()