#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitBestv10.py 752 2009-06-04 09:51:15Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDImportLib                         import EDDict
from EDImportLib                         import EDDictionary
from EDImportLib                         import EDList

from EDAssert                            import EDAssert
from EDUtilsFile                         import EDUtilsFile
from EDTestCasePluginUnit                import EDTestCasePluginUnit
from EDUtilsTest                         import EDUtilsTest


class EDTestCasePluginUnitBestv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _oalStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginBestv10", "EDPluginBest-v1.0", _oalStringTestName )
        self.m_oedStrDataPath = self.getPluginTestsDataHome()
        
        self.m_oedObtainedInputFile = "XSDataInputBestv10.xml"
        self.m_oedObtainedInputFile2 = "XSDataInputBestv10FromObject_02.xml"
        self.m_oedReferenceInputFile = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataBestv10Input_reference.xml")
        self.m_oedReferenceInputFile2 = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSDataBestv10Input_reference_02.xml")
        
        self.m_oedReferenceScriptLogFileName = EDDiskExplorer.mergePath( self.m_oedStrDataPath, "EDPluginBestv10.log" )
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile2 )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginBest = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration.xml" ))
        oedPluginBest.setConfiguration( xsPluginItemGood01 )
        oedPluginBest.setScriptExecutable( "cat" )
        oedPluginBest.configure()
        EDAssert.equal("/bin/bash", oedPluginBest.getScriptShell())
        EDAssert.equal("cat", oedPluginBest.getScriptExecutable())
        EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/include/ccp4.setup-bash.orig", oedPluginBest.getSetupCCP4())
        EDAssert.equal("Version of Best to be tested", oedPluginBest.getStringVersion())
        #EDAssert.equal(600, oedPluginMessage.getTimeOut())
        EDAssert.equal("/home/sweet/home", oedPluginBest.getBestHome())
        EDAssert.equal("export besthome=/home/sweet/home", oedPluginBest.getCommandBestHome())
        self.cleanUp( oedPluginBest )
        
        
    def testSetDataModelInput( self ):
        """
        """
        oedPluginBest = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "XSConfiguration.xml" ))
        oedPluginBest.setConfiguration( xsPluginItemGood01 )
        oedPluginBest.setScriptExecutable( "cat" )
        oedPluginBest.configure()
             
        from XSDataBestv10 import XSDataBestInput
        xsDataBestInput = XSDataBestInput()
        
        from XSDataCommon import XSDataAbsorbedDoseRate
        from XSDataCommon import XSDataFloat
        from XSDataCommon import XSDataString
        from XSDataCommon import XSDataTime
        from XSDataCommon import XSDataFile
        from XSDataCommon import XSDataSpeed
        from XSDataCommon import XSDataString
        from XSDataCommon import XSDataAngle
        
        xsDataBestInput.setCrystalAbsorbedDoseRate( XSDataAbsorbedDoseRate( 0.22E+06 ))
        xsDataBestInput.setCrystalShape( XSDataFloat(1) )
        xsDataBestInput.setCrystalSusceptibility( XSDataFloat(1.5) )
        xsDataBestInput.setDetectorType( XSDataString("q210-2x") )
        xsDataBestInput.setBeamExposureTime( XSDataTime(1) )
        xsDataBestInput.setBeamMaxExposureTime( XSDataTime(10000) )
        xsDataBestInput.setBeamMinExposureTime( XSDataTime(0.1) )
        xsDataBestInput.setGoniostatMinRotationWidth( XSDataAngle(0.1) )
        xsDataBestInput.setGoniostatMaxRotationSpeed( XSDataSpeed(10) )
        xsDataBestInput.setAimedResolution( XSDataFloat(2) )
        xsDataBestInput.setAimedRedundancy( XSDataFloat(6.5) )
        xsDataBestInput.setAimedCompleteness( XSDataFloat(0.9) )
        xsDataBestInput.setAimedIOverSigma( XSDataFloat(3) )
        xsDataBestInput.setComplexity( XSDataString("min") )
        fileDirectory = oedPluginBest.getWorkingDirectory()
        
        bestFileContentDat = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.dat" ))
        xsDataBestInput.setBestFileContentDat( XSDataString( bestFileContentDat) )
        
        bestFileContentPar = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile.par" ))
        xsDataBestInput.setBestFileContentPar( XSDataString( bestFileContentPar ))

        bestFileContentHKL = EDUtilsFile.readFile( EDDiskExplorer.mergePath( self.m_oedStrDataPath, "bestfile1.hkl" ))
        listBestFileContentHKL = EDList()
        listBestFileContentHKL.add( XSDataString( bestFileContentHKL ))
        xsDataBestInput.setBestFileContentHKL( listBestFileContentHKL )
        
        xsDataBestInput.outputFile( self.m_oedObtainedInputFile )
        
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile ) 
        
        xsDataInputExpected = XSDataBestInput.parseString( strExpectedInput ) 
        xsDataInputObtained = XSDataBestInput.parseString( strObtainedInput )
        
        EDAssert.equal( xsDataInputExpected.marshal(), xsDataInputObtained.marshal() ) 
        
        self.cleanUp( oedPluginBest )


##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testSetDataModelInput )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitBestv10 = EDTestCasePluginUnitBestv10( "EDTestCasePluginUnitBestv10" )
    edTestCasePluginUnitBestv10.execute()

