#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMGeneratePredictionv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDCompiler
from EDImportLib import EDString
from EDImportLib import EDVerbose
from EDImportLib import EDDiskExplorer
from EDImportLib import EDList

from EDAssert             import EDAssert
from EDTestCasePluginUnit import EDTestCasePluginUnit
from EDUtilsTest          import EDUtilsTest
from EDUtilsPath          import EDUtilsPath

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataString
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataWavelength
from XSDataCommon import XSDataFile


class EDTestCasePluginUnitMOSFLMGeneratePredictionv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMGeneratePredictionv10", "EDPluginGroupMOSFLM-v1.0", _edStringTestName )
        
        edStrDataImageDir = "images"
        self.m_edStringDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), edStrDataImageDir )

        edStringPluginTestDataHome = self.getPluginTestsDataHome()
        self.m_edStringUnitTestDataHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "unitTest" ) )

        self.m_edStringReferenceInputFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMv10InputGeneratePrediction_reference.xml")
        self.m_edStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMOutputGeneratePrediction_reference.xml")
                
            
    
    def generateDataMOSFLMInputGeneratePrediction( self ):
        """
        """
        from XSDataMOSFLMv10 import XSDataMOSFLMBeamPosition
        xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeamPosition.setX( XSDataLength( 102.5 ) )
        xsDataMOSFLMBeamPosition.setY( XSDataLength( 104.7 ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMDetector
        xsDataMOSFLMDetector = XSDataMOSFLMDetector()
        xsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMImage
        xsDataMOSFLMImage1 = XSDataMOSFLMImage()
        xsDataMOSFLMImage1.setNumber( XSDataInteger( 1 ) )
        xsDataMOSFLMImage1.setRotationAxisStart( XSDataAngle( 0.0 ) )
        xsDataMOSFLMImage1.setRotationAxisEnd( XSDataAngle( 1.0 ) )
                
        from XSDataMOSFLMv10 import XSDataMOSFLMInputGeneratePrediction
        xsDataMOSFLMInputGeneratePrediction = XSDataMOSFLMInputGeneratePrediction()
        xsDataMOSFLMInputGeneratePrediction.setDistance( XSDataLength( 198.4 ) )
        xsDataMOSFLMInputGeneratePrediction.setWavelength( XSDataWavelength( 0.9340 ) )
        xsDataMOSFLMInputGeneratePrediction.setBeam( xsDataMOSFLMBeamPosition )
        xsDataMOSFLMInputGeneratePrediction.setDetector( xsDataMOSFLMDetector )
        xsDataMOSFLMInputGeneratePrediction.setDirectory( XSDataString( self.m_edStringDataImagePath ) )
        xsDataMOSFLMInputGeneratePrediction.setTemplate( XSDataString( "ref-testscale_1_###.img" ) )
        xsDataMOSFLMInputGeneratePrediction.setImage( xsDataMOSFLMImage1 )
        
        return xsDataMOSFLMInputGeneratePrediction
        
        
    def testGenerateMOSFLMCommands( self ):
        """
        """
        pluginGeneratePrediction = self.createPlugin()
        pluginGeneratePrediction.setScriptExecutable("cat")
        pluginGeneratePrediction.configure()
        xsDataMOSFLMInputGeneratePrediction = self.generateDataMOSFLMInputGeneratePrediction()
        pluginGeneratePrediction.setDataInput( xsDataMOSFLMInputGeneratePrediction )
        pluginGeneratePrediction.generateMOSFLMCommands()
        edListCommandsReference = EDList( ['WAVELENGTH 0.934', 'DISTANCE 198.4', 'BEAM 102.5 104.7', 'DETECTOR ADSC', 'DIRECTORY ' + self.m_edStringDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'XGUI ON', 'IMAGE 1 PHI 0.000000 TO 1.000000', 'GO', 'PREDICT_SPOTS', 'CREATE_IMAGE PREDICTION ON BINARY TRUE FILENAME ' + pluginGeneratePrediction.getPredictionImageFileName(), 'RETURN', 'EXIT'] )
        edListCommands = pluginGeneratePrediction.getListCommandExecution()
        #print edListCommandsReference
        #print edListCommands
        EDAssert.equal( edListCommandsReference, edListCommands )
        
        self.cleanUp( pluginGeneratePrediction )

    def testCreateDataMOSFLMOutputGeneratePrediction( self ):
        """
        """
        pluginGeneratePrediction = self.createPlugin()
        pluginGeneratePrediction.setScriptExecutable("cat")
        pluginGeneratePrediction.configure()
        pluginGeneratePrediction.setPredictionImageFileName( "prediction.jpg" )
        edStringBaseName = pluginGeneratePrediction.createBaseName()
        pluginGeneratePrediction.setScriptBaseName( edStringBaseName )
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMGeneratePredictionv10_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginGeneratePrediction.getWorkingDirectory(), edStringBaseName + "_dnaTables.xml" ) ) 
        edStringNewmatFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMv10_autoindexMat_ok.txt" )
        pluginGeneratePrediction.setNewmatFileName( edStringNewmatFile )
        from XSDataMOSFLMv10 import XSDataMOSFLMInputGeneratePrediction
        xsDataMOSFLMInputGeneratePrediction = XSDataMOSFLMInputGeneratePrediction.parseFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMInputGeneratePrediction_reference.xml" ) )
        pluginGeneratePrediction.setDataInput( xsDataMOSFLMInputGeneratePrediction )
        xsDataMOSFLMOutputGeneratePrediction = pluginGeneratePrediction.createDataMOSFLMOutputGeneratePrediction()
        edStringImagePath = xsDataMOSFLMOutputGeneratePrediction.getPredictionImage().getPath().getValue()
        #xsDataMOSFLMOutputGeneratePrediction.outputFile( "XSDataMOSFLMOutputGeneratePrediction_reference.xml" )
        edStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataOutputFile )
        from XSDataMOSFLMv10 import XSDataMOSFLMOutputGeneratePrediction
        xsDataMOSFLMOutputGeneratePredictionReference = XSDataMOSFLMOutputGeneratePrediction.parseString( edStringReferenceXML )
        xsDataImage = xsDataMOSFLMOutputGeneratePredictionReference.getPredictionImage()
        xsDataImage.setPath( XSDataString( edStringImagePath ))
        xsDataMOSFLMOutputGeneratePredictionReference.setPredictionImage( xsDataImage )
        EDAssert.equal( xsDataMOSFLMOutputGeneratePredictionReference.marshal(), xsDataMOSFLMOutputGeneratePrediction.marshal() )
        
        self.cleanUp( pluginGeneratePrediction )

    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputGeneratePrediction )        



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMGeneratePredictionv10 = EDTestCasePluginUnitMOSFLMGeneratePredictionv10( "EDTestCasePluginUnitMOSFLMGeneratePredictionv10" )
    edTestCasePluginUnitMOSFLMGeneratePredictionv10.execute()
