#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMIndexingv10.py 731 2009-05-25 10:31:15Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDCompiler
from EDImportLib import EDString
from EDImportLib import EDVerbose
from EDImportLib import EDDiskExplorer
from EDImportLib import EDList

from EDAssert             import EDAssert
from EDTestCasePluginUnit import EDTestCasePluginUnit
from EDUtilsTest          import EDUtilsTest
from EDUtilsPath          import EDUtilsPath

from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataInteger
from XSDataCommon import XSDataString
from XSDataCommon import XSDataFloat
from XSDataCommon import XSDataWavelength

from XSDataMOSFLMv10 import XSDataMOSFLMOutputIndexing

class EDTestCasePluginUnitMOSFLMIndexingv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMIndexingv10", "EDPluginGroupMOSFLM-v1.0", _edStringTestName )
        
        edStringDataImageDir = "images"
        self.m_edStringDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), edStringDataImageDir )

        edStringPluginTestDataHome = self.getPluginTestsDataHome()
        self.m_edStringUnitTestDataHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "unitTest" ) )

        self.m_edStringReferenceDataInputFile  = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMInputIndexing_reference.xml")
        self.m_edStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMOutputIndexing_reference.xml")
        
        
            
    
    def generateDataMOSFLMInputIndexing( self ):
        """
        """
        from XSDataMOSFLMv10 import XSDataMOSFLMBeamPosition
        xsDataMOSFLMBeamPosition = XSDataMOSFLMBeamPosition()
        xsDataMOSFLMBeamPosition.setX( XSDataLength( 102.5 ) )
        xsDataMOSFLMBeamPosition.setY( XSDataLength( 104.7 ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMDetector
        xsDataMOSFLMDetector = XSDataMOSFLMDetector()
        xsDataMOSFLMDetector.setType( XSDataString( "ADSC" ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMImage
        xsDataMOSFLMImage1 = XSDataMOSFLMImage()
        xsDataMOSFLMImage1.setNumber( XSDataInteger( 1 ) )
        xsDataMOSFLMImage1.setRotationAxisStart( XSDataAngle( 0.0 ) )
        xsDataMOSFLMImage1.setRotationAxisEnd( XSDataAngle( 1.0 ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMImage
        xsDataMOSFLMImage2 = XSDataMOSFLMImage()
        xsDataMOSFLMImage2.setNumber( XSDataInteger( 2 ) )
        xsDataMOSFLMImage2.setRotationAxisStart( XSDataInteger( 90.0 ) )
        xsDataMOSFLMImage2.setRotationAxisEnd( XSDataInteger( 91.0 ) )
        
        from XSDataMOSFLMv10 import XSDataMOSFLMInputIndexing
        xsDataMOSFLMInputIndexing = XSDataMOSFLMInputIndexing()
        xsDataMOSFLMInputIndexing.setDistance( XSDataLength( 198.4 ) )
        xsDataMOSFLMInputIndexing.setWavelength( XSDataWavelength( 0.9340 ) )
        xsDataMOSFLMInputIndexing.setBeam( xsDataMOSFLMBeamPosition )
        xsDataMOSFLMInputIndexing.setDetector( xsDataMOSFLMDetector )
        xsDataMOSFLMInputIndexing.setDirectory( XSDataString( self.m_edStringDataImagePath ) )
        xsDataMOSFLMInputIndexing.setTemplate( XSDataString( "ref-testscale_1_###.img" ) )
        xsDataMOSFLMInputIndexing.addImage( xsDataMOSFLMImage1 )
        xsDataMOSFLMInputIndexing.addImage( xsDataMOSFLMImage2 )
        
        return xsDataMOSFLMInputIndexing
        
        
    def testGenerateMOSFLMCommands( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.setScriptExecutable("cat")
        pluginIndexing.configure()
        xsDataMOSFLMInputIndexing = self.generateDataMOSFLMInputIndexing()
        pluginIndexing.setDataInput( xsDataMOSFLMInputIndexing )
        pluginIndexing.generateMOSFLMCommands()
        edListCommandsReference = EDList( ['WAVELENGTH 0.934', 'DISTANCE 198.4', 'BEAM 102.5 104.7', 'DETECTOR ADSC', 'DIRECTORY ' + self.m_edStringDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'NEWMAT ' + pluginIndexing.getNewmatFileName(), 'AUTOINDEX DPS REFINE IMAGE 1 PHI 0.0 1.0', 'AUTOINDEX DPS REFINE IMAGE 2 PHI 90.0 91.0', 'GO', 'MOSAIC ESTIMATE 1', 'GO'] )
        edListCommands = pluginIndexing.getListCommandExecution()
        EDAssert.equal( edListCommandsReference, edListCommands )        
        self.cleanUp( pluginIndexing )


    def testCreateDataMOSFLMOutputIndexing( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.setScriptExecutable("cat")
        pluginIndexing.configure()
        edStringBaseName = pluginIndexing.createBaseName()
        pluginIndexing.setScriptBaseName( edStringBaseName )
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMIndexingv10_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginIndexing.getWorkingDirectory(), edStringBaseName + "_dnaTables.xml" ) ) 
        edStringNewmatFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMv10_autoindexMat_ok.txt" )
        pluginIndexing.setNewmatFileName( edStringNewmatFile )
        xsDataMOSFLMOutputIndexing = pluginIndexing.createDataMOSFLMOutputIndexing()
        xsDataMOSFLMOutputIndexing.outputFile( "XSDataMOSFLMOutputIndexing_reference.xml" )
        edStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataOutputFile )
        xsDataMOSFLMOutputIndexingReference = XSDataMOSFLMOutputIndexing.parseString( edStringReferenceXML )
        #print len( edStringReferenceXML )
        #print len( xsDataMOSFLMOutputIndexing.marshal() )
        EDAssert.equal( xsDataMOSFLMOutputIndexingReference.marshal(), xsDataMOSFLMOutputIndexing.marshal() )
        self.cleanUp( pluginIndexing )


    def testGenerateExecutiveSummary( self ):
        """
        """
        pluginIndexing = self.createPlugin()
        pluginIndexing.setScriptExecutable("cat")
        pluginIndexing.configure()
        from XSDataMOSFLMv10 import XSDataMOSFLMInputIndexing
        from XSDataMOSFLMv10 import XSDataMOSFLMOutputIndexing
        edStringMOSFLMInputIndexingXML  = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataInputFile )
        edStringMOSFLMOutputIndexingXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataOutputFile )
        xsDataMOSFLMInputIndexing  = XSDataMOSFLMInputIndexing.parseString( edStringMOSFLMInputIndexingXML )
        xsDataMOSFLMOutputIndexing = XSDataMOSFLMOutputIndexing.parseString( edStringMOSFLMOutputIndexingXML )
        pluginIndexing.setDataInput( xsDataMOSFLMInputIndexing )
        pluginIndexing.setDataOutput( xsDataMOSFLMOutputIndexing )
        pluginIndexing.generateExecutiveSummary( pluginIndexing )
        pluginIndexing.verboseScreenExecutiveSummary()
        self.cleanUp( pluginIndexing )

    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputIndexing )
        self.addTestMethod( self.testGenerateExecutiveSummary )  



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMIndexingv10 = EDTestCasePluginUnitMOSFLMIndexingv10( "EDTestCasePluginUnitMOSFLMIndexingv10" )
    edTestCasePluginUnitMOSFLMIndexingv10.execute()
