#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitMOSFLMIntegrationv10.py 675 2009-05-07 08:42:36Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    Contributing author:    Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib         import EDCompiler
from EDImportLib         import EDString
from EDImportLib         import EDVerbose
from EDImportLib         import EDDiskExplorer
from EDImportLib         import EDList

from EDAssert             import EDAssert
from EDTestCasePluginUnit import EDTestCasePluginUnit
from EDUtilsTest          import EDUtilsTest
from EDUtilsPath          import EDUtilsPath

from XSDataCommon import XSDataFile
from XSDataCommon import XSDataString

from XSDataMOSFLMv10 import XSDataMOSFLMOutputIntegration

class EDTestCasePluginUnitMOSFLMIntegrationv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginMOSFLMIntegrationv10", "EDPluginGroupMOSFLM-v1.0", _edStringTestName )

        edStringPluginTestDataHome = self.getPluginTestsDataHome()
        self.m_edStringUnitTestDataHome = EDUtilsPath.mergePath( edStringPluginTestDataHome, EDString( "unitTest" ) )

        self.m_edStringReferenceDataInputFile  = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMInputIntegration_reference.xml")
        self.m_edStringReferenceDataOutputFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "XSDataMOSFLMOutputIntegration_reference.xml")
        
        edStringDataImageDir = "images"
        self.m_edStringDataImagePath = EDDiskExplorer.mergePath( self.getTestsDataHome(), edStringDataImageDir )
        
        
    def preProcess( self ):
        """
        """  
        EDVerbose.DEBUG ("*** EDTestCaseEDPluginMOSFLMIntegrationv10.preProcess")
    
    
                
    def testGenerateMOSFLMIntegrationCommands( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.setScriptExecutable("cat")
        pluginIntegration.configure()
        strXMLInputData = EDUtilsTest.readAndParseFile ( self.m_edStringReferenceDataInputFile )
        pluginIntegration.setDataInput( strXMLInputData )
        pluginIntegration.generateMOSFLMCommands()
        edListCommandExecution = pluginIntegration.getListCommandExecution()
        #edListCommandReference = ['SYMMETRY P222', 'MATRIX ' + EDString( pluginIntegration.getScriptBaseName() + "_newmat.mat" ), 'MOSAIC 0.75', 'WAVELENGTH 0.934', 'DISTANCE 198.441', 'BEAM 102.478996 104.8862', 'DETECTOR ADSC', 'DIRECTORY '+self.m_edStrDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'HKLOUT ' + EDString( pluginIntegration.getScriptBaseName() ) + '_process_1_1.mtz', 'PROCESS 1 TO 1 START 0.000000 ANGLE 1.000000', 'BEST ON', 'GO', 'BEST OFF']
        edListCommandReference = ['WAVELENGTH 0.934', 'DISTANCE 198.440994', 'BEAM 102.478996 104.8862', 'DETECTOR ADSC', 'DIRECTORY '+self.m_edStringDataImagePath, 'TEMPLATE ref-testscale_1_###.img', 'SYMMETRY P222', 'MATRIX ' + EDString( pluginIntegration.getScriptBaseName() ) + '_matrix.mat', 'MOSAIC 0.75', 'HKLOUT ' + EDString( pluginIntegration.getScriptBaseName() ) + '_process_1_1.mtz', 'PROCESS 1 TO 1 START 0.000000 ANGLE 1.000000', 'BEST ON', 'GO', 'BEST OFF']
        #print edListCommandReference
        #print edListCommandExecution
        EDAssert.equal(edListCommandReference, edListCommandExecution)
        

    def testCreateDataMOSFLMOutputIntegration( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.setScriptExecutable("cat")
        pluginIntegration.configure()
        edStringBaseName = pluginIntegration.getBaseName()
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMIntegrationv10_bestfileDat_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.dat") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMIntegrationv10_bestfilePar_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.par") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMIntegrationv10_bestfileHKL_ok.txt"), EDDiskExplorer.mergePath ( pluginIntegration.getWorkingDirectory(), "bestfile.hkl") ) 
        EDDiskExplorer.copyFile( EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMIntegrationv10_outputDnaTables_ok.xml"), EDDiskExplorer.mergePath( pluginIntegration.getWorkingDirectory(), edStringBaseName + "_dnaTables.xml" ) ) 
        edStringMatrixFile = EDDiskExplorer.mergePath( self.m_edStringUnitTestDataHome, "EDPluginMOSFLMv10_autoindexMat_ok.txt" )
        pluginIntegration.setMatrixFileName( edStringMatrixFile )
        xsDataMOSFLMIntegrationOutput = pluginIntegration.createDataMOSFLMOutputIntegration()
        # Fix problem with absolute path by replacing it with a fixed one
        xsDataFile = XSDataFile()
        xsDataFile.setPath( XSDataString( "/tmp/EDPluginMOSFLMIntegrationv10_process_1_1.mtz" ) )
        xsDataMOSFLMIntegrationOutput.setGeneratedMTZFile( xsDataFile )
        edStringReferenceXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataOutputFile )
        #xsDataMOSFLMIntegrationOutput.outputFile( "test.xml" )
        xsDataMOSFLMIntegrationOutputReference = XSDataMOSFLMOutputIntegration.parseString( edStringReferenceXML )
        EDAssert.equal( xsDataMOSFLMIntegrationOutputReference.marshal(), xsDataMOSFLMIntegrationOutput.marshal() )
        

    def testGenerateExecutiveSummary( self ):
        """
        """
        pluginIntegration = self.createPlugin()
        pluginIntegration.setScriptExecutable("cat")
        pluginIntegration.configure()
        from XSDataMOSFLMv10 import XSDataMOSFLMInputIntegration
        from XSDataMOSFLMv10 import XSDataMOSFLMOutputIntegration
        edStringMOSFLMInputIntegrationXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataInputFile )
        edStringMOSFLMOutputIntegrationXML = EDUtilsTest.readAndParseFile( self.m_edStringReferenceDataOutputFile )
        xsDataMOSFLMInputIntegration = XSDataMOSFLMInputIntegration.parseString( edStringMOSFLMInputIntegrationXML )
        xsDataMOSFLMOutputIntegration = XSDataMOSFLMOutputIntegration.parseString( edStringMOSFLMOutputIntegrationXML )
        pluginIntegration.setDataInput( xsDataMOSFLMInputIntegration )
        pluginIntegration.setDataOutput( xsDataMOSFLMOutputIntegration )
        pluginIntegration.generateExecutiveSummary( pluginIntegration )
        pluginIntegration.verboseScreenExecutiveSummary()
        self.cleanUp( pluginIntegration )



    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateMOSFLMIntegrationCommands )
        self.addTestMethod( self.testCreateDataMOSFLMOutputIntegration )        
        self.addTestMethod( self.testGenerateExecutiveSummary )  
        
        
    def postProcess( self ):
        """
        """
        
        # check if in the next AALib release, the MOSFLM plugin directory is added only once in the PYTHONPATH
        EDVerbose.DEBUG ( "*** EDTestCaseEDPluginMOSFLMIntegrationv10.postProcess" )



##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()

    edTestCasePluginUnitMOSFLMIntegrationv10 = EDTestCasePluginUnitMOSFLMIntegrationv10( "EDTestCasePluginUnitMOSFLMIntegrationv10" )
    edTestCasePluginUnitMOSFLMIntegrationv10.execute()
