#
#    Project: mxPluginExec
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitRaddosev10.py 753 2009-06-04 09:54:13Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Lesser General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    and the GNU Lesser General Public License  along with this program.  
#    If not, see <http://www.gnu.org/licenses/>.
#



from EDImportLib                     import EDCompiler
from EDImportLib                     import EDVerbose
from EDImportLib                     import EDString
from EDImportLib                     import EDDiskExplorer
from EDImportLib                     import EDDict
from EDImportLib                     import EDDictionary

from EDAssert                        import EDAssert
from EDUtilsPath                     import EDUtilsPath
from EDTestCasePluginUnit            import EDTestCasePluginUnit
from EDUtilsTest                     import EDUtilsTest


class EDTestCasePluginUnitRaddosev10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _pyStrTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginRaddosev10", "EDPluginRaddose-v1.0", _pyStrTestName )

        self.m_oedStrDataPath = self.getPluginTestsDataHome()
        self.m_oedObtainedInputFile = "XSDataInputRaddosev10.xml"
        self.m_oedObtainedInputFile2 = "XSDataInputRaddosev10FromObject_02.xml"
        self.m_oedReferenceInputFile = EDUtilsPath.mergePath( self.m_oedStrDataPath, "XSDataRaddosev10Input_reference.xml")
        self.m_oedReferenceInputFile2 = EDUtilsPath.mergePath( self.m_oedStrDataPath, "XSDataRaddosev10Input_reference_02.xml")
        
        self.m_oedReferenceScriptLogFileName = EDUtilsPath.mergePath( self.m_oedStrDataPath, "EDPluginRaddosev10.log" )
        self.m_oedReferenceScriptLogFileNamev2 = EDUtilsPath.mergePath( self.m_oedStrDataPath, "EDPluginRaddosev10_Raddosev2.log" )
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile )
        EDDiskExplorer.deleteFile( self.m_oedObtainedInputFile2 )
        
        
    def testConfigureOK( self ):
        """
        """
        oedPluginRaddose = self.createPlugin()
        edStringConfigurationFile = EDUtilsPath.mergePath( self.m_oedStrDataPath, "XSConfiguration.xml" )
        xsPluginItemGood01 = self.getPluginConfiguration( edStringConfigurationFile )
        oedPluginRaddose.setConfiguration( xsPluginItemGood01 )
        oedPluginRaddose.setScriptExecutable( "cat" )
        oedPluginRaddose.configure()
        EDAssert.equal("/bin/bash", oedPluginRaddose.getScriptShell())
        EDAssert.equal("cat", oedPluginRaddose.getScriptExecutable())
        EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/include/ccp4.setup-bash.orig", oedPluginRaddose.getSetupCCP4())
        EDAssert.equal("Verion of Raddose to be tested", oedPluginRaddose.getStringVersion())
        #EDAssert.equal(600, oedPluginMessage.getTimeOut())
        
        self.cleanUp( oedPluginRaddose )
        
        
    def testSetDataModelInput( self ):
        """
        """
        # Crystal         
        from XSDataRaddosev10 import XSDataRaddoseInput
        xsDataRaddoseInput = XSDataRaddoseInput()
        
        from XSDataCommon import XSDataString
        from XSDataCommon import XSDataFloat
        from XSDataCommon import XSDataAngle
        from XSDataCommon import XSDataLength
        from XSDataCommon import XSDataSize
        from XSDataCommon import XSDataInteger
        from XSDataRaddosev10 import XSDataCell
        from XSDataRaddosev10 import XSDataAtom
        from XSDataRaddosev10 import XSDataAtomicComposition

        xsDataAtomSulfur = XSDataAtom()
        xsDataAtomSulfur.setNumberOf( XSDataFloat(4) )
        xsDataAtomSulfur.setSymbol( XSDataString("S"))
        xsDataAtomSelenium = XSDataAtom()
        xsDataAtomSelenium.setNumberOf( XSDataFloat(4) )
        xsDataAtomSelenium.setSymbol( XSDataString("Se"))
        
        xsDataAtomicComposition = XSDataAtomicComposition()
        xsDataAtomicComposition.addAtom( xsDataAtomSulfur )
        xsDataAtomicComposition.addAtom( xsDataAtomSelenium )
        xsDataRaddoseInput.setCrystalPATM( xsDataAtomicComposition )
        xsDataRaddoseInput.setCrystalNRES( XSDataInteger(295) )
        xsDataRaddoseInput.setCrystalNMON( XSDataInteger(8) )
        xsDataCell = XSDataCell( XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataAngle(90.0),
                                 XSDataLength(78.9),
                                 XSDataLength(95.162),
                                 XSDataLength(104.087) )
        
        xsDataSizeCrystal = XSDataSize( XSDataLength(0.1),
                                        XSDataLength(0.1),
                                        XSDataLength(0.1))
        
        xsDataRaddoseInput.setCrystalCell( xsDataCell )
        xsDataRaddoseInput.setCrystalSize( xsDataSizeCrystal )
        
        # Beam
        from XSDataCommon import XSDataFlux
        from XSDataCommon import XSDataWavelength
        from XSDataCommon import XSDataTime
                
        xsDataSize = XSDataSize( XSDataLength(0.1),
                                 XSDataLength(0.1))
        
        xsDataRaddoseInput.setBeamSize( xsDataSize )
        xsDataRaddoseInput.setBeamFlux( XSDataFlux(1e+12) )
        xsDataRaddoseInput.setBeamWavelength( XSDataWavelength(2.41) )
        xsDataRaddoseInput.setBeamExposureTime( XSDataTime(1))
 
        from XSDataCommon import XSDataInteger

        xsDataRaddoseInput.setNumberOfImages( XSDataInteger(1) )
        
        xsDataRaddoseInput.outputFile( self.m_oedObtainedInputFile )
        
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedInputFile ) 
        
        xsDataRaddoseInputExpected = XSDataRaddoseInput.parseString( strExpectedInput ) 
        xsDataRaddoseInputObtained = XSDataRaddoseInput.parseString( strObtainedInput )
        
        EDAssert.equal( xsDataRaddoseInputExpected.marshal(), xsDataRaddoseInputObtained.marshal() )
             
        
    def testSetDataInput( self ):
        """
        """
        
        from XSDataRaddosev10 import XSDataRaddoseInput
        oedPluginRaddose = self.createPlugin()
        
        EDAssert.raiseException( RuntimeError, oedPluginRaddose.setDataInput, EDDict())
        
        strXMLInputData = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        oedPluginRaddose.setDataInput( EDString( strXMLInputData ) )
        oedPluginRaddose.setScriptExecutable("cat")
        oedPluginRaddose.preProcess()
        
        xsDataInput = oedPluginRaddose.getDataInput()
        xsDataInput.outputFile( "XSDataInputRaddosev10FromObject.xml" )
        strExpectedInput = EDUtilsTest.readAndParseFile ( self.m_oedReferenceInputFile )
        strObtainedInput = EDUtilsTest.readAndParseFile ( "XSDataInputRaddosev10FromObject.xml" )
        
        xsDataRaddoseInputExpected = XSDataRaddoseInput.parseString( strExpectedInput ) 
        xsDataRaddoseInputObtained = XSDataRaddoseInput.parseString( strObtainedInput )
        
        EDAssert.equal( xsDataRaddoseInputExpected.marshal(), xsDataRaddoseInputObtained.marshal() )
        
        EDAssert.equal( "CELL 78.9 95.162 104.087 90.0 90.0 90.0", oedPluginRaddose.getCommandCrystalCell() )
        
        EDAssert.equal( "NRES 295", oedPluginRaddose.getCommandCrystalNRES() )
        EDAssert.equal( "NMON 8", oedPluginRaddose.getCommandCrystalNMON() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalNDNA() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalNRNA() )
        EDAssert.equal( "S", xsDataInput.getCrystalPATM().getAtom()[0].getSymbol().getValue() )
        EDAssert.equal( 4, xsDataInput.getCrystalPATM().getAtom()[0].getNumberOf().getValue() )
        EDAssert.equal( "Se", xsDataInput.getCrystalPATM().getAtom()[1].getSymbol().getValue() )
        EDAssert.equal( 4, xsDataInput.getCrystalPATM().getAtom()[1].getNumberOf().getValue() )
        EDAssert.equal( "PATM S 4.0 Se 4.0", oedPluginRaddose.getCommandCrystalPATM() )
        EDAssert.equal( None, xsDataInput.getCrystalSATM() )
        EDAssert.equal( None, oedPluginRaddose.getCommandCrystalSATM() )
        EDAssert.equal( "CRYSTAL 0.1 0.1 0.1", oedPluginRaddose.getCommandCrystalSize() )
        
        EDAssert.equal( "BEAM 0.1 0.1", oedPluginRaddose.getCommandBeamSize() )
        EDAssert.equal( "PHOSEC 1e+12", oedPluginRaddose.getCommandBeamFlux() )
        EDAssert.equal( "WAVELENGTH 2.41", oedPluginRaddose.getCommandBeamWavelength() )
        
        EDAssert.equal( "EXPOSURE 1.0", oedPluginRaddose.getCommandExposureTime() )
        EDAssert.equal( "IMAGES 1", oedPluginRaddose.getCommandImages() )
        
        self.cleanUp( oedPluginRaddose )
        
        
    def testDictionnaryResults( self ):
        """
        """
        
        # Tests with Raddose v? (old version, no Raddose version management...)
        edPluginRaddose = self.createPlugin()
        expectedDictionnaryv1 = EDDict()
        expectedDictionnaryv1.add("Dose in Grays", "0.22E+06")
        expectedDictionnaryv1.add("Total absorbed dose (Gy)", None)
        expectedDictionnaryv1.add("Solvent Content (%)", "59.2")
        obtainedDictionnaryv1 = edPluginRaddose.analyseScriptLogFileName( self.m_oedReferenceScriptLogFileName )
        EDAssert.equal( expectedDictionnaryv1, obtainedDictionnaryv1 )
        
        # Tests with Raddose v2
        edPluginRaddose = self.createPlugin()
        expectedDictionnaryv2 = EDDict()
        expectedDictionnaryv2.add("Dose in Grays", None)
        expectedDictionnaryv2.add("Total absorbed dose (Gy)", "0.223E+06")
        expectedDictionnaryv2.add("Solvent Content (%)", "59.2")
        obtainedDictionnaryv2 = edPluginRaddose.analyseScriptLogFileName( self.m_oedReferenceScriptLogFileNamev2 )
        EDAssert.equal( expectedDictionnaryv2, obtainedDictionnaryv2 )
        
        
    def testSymmetryTableTest( self ):
        """
        """
        
        import ConfigParser

        config = ConfigParser.ConfigParser()
        config.read("table_symmetry.txt")

        print "id:", config.get("3", "id")
        print "Number of Symmetry Operators:", config.get("3", "Number of Symmetry Operators")
        print "Symmetry:", config.get("3", "symmetry")
        
        from XSDataRaddosev01 import XSDataSymmetry
        from XSDataRaddosev01 import XSDataFloat
        from XSDataRaddosev01 import XSDataString
        
        xsDataSymmetry = XSDataSymmetry( XSDataString("1"), XSDataFloat() )
        
        
        
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        self.addTestMethod( self.testSetDataModelInput )
        self.addTestMethod( self.testSetDataInput )
        self.addTestMethod( self.testDictionnaryResults )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitRaddosev10 = EDTestCasePluginUnitRaddosev10( "EDTestCasePluginUnitRaddosev10" )
    edTestCasePluginUnitRaddosev10.execute()

