#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitControlIndexingv10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDImportLib                         import EDDict
from EDImportLib                         import EDDictionary
from EDImportLib                         import EDList

from EDAssert                            import EDAssert
from EDUtilsFile                         import EDUtilsFile
from EDTestCasePluginUnit                import EDTestCasePluginUnit
from EDUtilsTest                         import EDUtilsTest

from XSDataMXv1 import XSDataIndexingInput

class EDTestCasePluginUnitControlIndexingv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginControlIndexingv10", "EDPluginControlIndexing-v1.0", _edStringTestName )
        self.m_edReferenceInputFile = EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataIndexingInput_reference.xml")
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginUnit.preProcess( self )
        #EDDiskExplorer.deleteFile( self.m_edObtainedInputFile )
        
        
    def testConfigureOK( self ):
        """
        """
        edPluginControlIndexingv10 = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginControlIndexingv10.setConfiguration( xsPluginItemGood01 )
        edPluginControlIndexingv10.configure()
        #EDAssert.equal("/opt/pxsoft/ccp4-6.0.2/lib/data", edPluginControlIndexingv10.getSymopHome())
        
        self.cleanUp( edPluginControlIndexingv10 )
    
    def testSetDataModelInput( self ):
        from XSDataIndexingv01 import XSDataIndexingv01Input
        from XSDataIndexingv01 import XSDataImage
        from XSDataIndexingv01 import XSDataExperimentPoint
        from XSDataIndexingv01 import XSDataBeam
        from XSDataIndexingv01 import XSDataWavelength
        
        wavelength = XSDataWavelength()
        wavelength.setValue( 0.934000 )
        
        beam = XSDataBeam()
        beam.setWavelength( wavelength )
        
        experiment = XSDataExperimentPoint()
        experiment.setBeam( beam )
        
        image = XSDataImage()
        image.setExperiment( experiment )

        xsDataIndexingv01Input = XSDataIndexingv01Input()
        xsDataIndexingv01Input.addReferenceImage( image )
        #xsDataIndexingv01Input.setAnomalousStrategy( True )

        xmlString = xsDataIndexingv01Input.marshal()
        xsDataIndexingv01Input2 = XSDataIndexingv01Input.parseString( xmlString )

        
        
    def testSetDataInput( self ):
        """
        """
        edPluginControlIndexingv10 = self.createPlugin()
        xsPluginItemGood01 = self.getPluginConfiguration( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSConfiguration.xml" ))
        edPluginControlIndexingv10.setConfiguration( xsPluginItemGood01 )
        edPluginControlIndexingv10.configure()

        xmlInput = EDUtilsTest.readAndParseFile( self.m_edReferenceInputFile )
        edPluginControlIndexingv10.setDataInput( xmlInput )

        xsDataIndexingv01Input = edPluginControlIndexingv10.getDataInput()

        xsDataIndexingInputReference = XSDataIndexingInput.parseString( xmlInput )
        EDAssert.equal( xsDataIndexingInputReference.marshal(), xsDataIndexingv01Input.marshal() )
        
        self.cleanUp( edPluginControlIndexingv10 )
        

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testConfigureOK )
        #self.addTestMethod( self.testSetDataModelInput )
        self.addTestMethod( self.testSetDataInput )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    EDTestCasePluginUnitControlIndexingv10 = EDTestCasePluginUnitControlIndexingv10( "EDTestCasePluginUnitControlIndexingv10" )
    EDTestCasePluginUnitControlIndexingv10.execute()

