#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteStrategyv10WithoutChemComposition.py 728 2009-05-20 14:02:00Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDAssert                            import EDAssert

from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath



class EDTestCasePluginExecuteStrategyv10WithoutChemComposition( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginControlStrategyv10", "EDPluginControlStrategy-v1.0", _edStringTestName )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataStrategyv10Input_reference_03.xml" ) )
        
        self.setReferenceDataOutputFile( EDUtilsPath.mergePath( self.getPluginTestsDataHome(), "XSDataStrategyv10Output_reference_03.xml"))
        self.m_oedObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
    def preProcess( self ):
        """
        """
        EDTestCasePluginExecute.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_oedObtainedOutputDataFile )
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages
        
        plugin = self.getPlugin()

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        # Checks the expected result
        strExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        strObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_oedObtainedOutputDataFile )    
        EDVerbose.DEBUG("Checking obtained result...") 
        
        from XSDataMXv1 import XSDataStrategyResult
        
        xsDataOutputExpected = XSDataStrategyResult.parseString( strExpectedOutput ) 
        xsDataOutputObtained = XSDataStrategyResult.parseString( strObtainedOutput )
        
        # To be changed to not compare the XML but certain parts in the XML, see bug #294
        #EDAssert.equal( xsDataOutputExpected.marshal(), xsDataOutputObtained.marshal() )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteStrategyv01WithoutChemComposition = EDTestCasePluginExecuteStrategyv01WithoutChemComposition( "EDTestCasePluginExecuteStrategyv01WithoutChemComposition" )
    edTestCasePluginExecuteStrategyv01WithoutChemComposition.execute()