#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteISPyBv10.py 583 2008-12-05 12:06:52Z levik $"
#
#    Copyright (C) 2008 Diamond Light Source
#                       Chilton, Didcot, UK
#
#    Principal author:      Karl Levik (karl.levik@diamond.ac.uk)
#
#    Contributing authors:  Marie-Francoise Incardona (incardon@esrf.fr)
#                           Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from EDImportLib                      import EDCompiler
from EDImportLib                      import EDVerbose
from EDImportLib                      import EDString
from EDAssert                         import EDAssert
from EDImportLib                      import EDDiskExplorer
from EDTestCasePluginExecute          import EDTestCasePluginExecute
from EDUtilsTest                      import EDUtilsTest

from XSDataISPyBv10                   import XSDataResultISPyB
from XSDataCommon                     import XSDataString 

class EDTestCasePluginExecuteISPyBv10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        Sets config file + input and output reference files. 
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginISPyBv10", "EDPluginGroupISPyB-v0.1", _edStringTestName )
        
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataISPyBv10Input_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataResultISPyB_reference.xml"))
        self.m_edObtainedOutputDataFile = self.getPluginName() + "_output.xml"

    def preProcess( self ):
        """
        Deletes the obtained output from the previous run. 
        """
        EDTestCasePluginExecute.preProcess( self )
        EDDiskExplorer.deleteFile( self.m_edObtainedOutputDataFile )

    def testExecute( self ):
        """
        Runs the plugin and then compares expected output with obtained output to verify that it executed correctly. 
        """ 
        self.run()

        # Checks that there are no error or warning messages
        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        # Checks the expected result
        edStrOutputObtained = EDUtilsTest.readAndParseFile( self.m_edObtainedOutputDataFile )    
        xsDataResultISPyBObtained = XSDataResultISPyB.parseString( edStrOutputObtained )
        xsDataResultStatusListObtained = xsDataResultISPyBObtained.getResultStatus()
        
        EDVerbose.DEBUG("Checking obtained result...") 
        for xsDataResultStatusObtained in xsDataResultStatusListObtained:
            EDAssert.equal( xsDataResultStatusObtained.getCode().getValue(), "ok" )

##############################################################################

    def process( self ):
        """
        Adds the plugin execute test methods
        """
        self.addTestMethod( self.testExecute )
        
##############################################################################

if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteISPyBv10 = EDTestCasePluginExecuteISPyBv10( "EDTestCasePluginExecuteISPyBv10" )
    edTestCasePluginExecuteISPyBv10.execute()