#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginExecuteSubWedgeMergev10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal author:      Olof Svensson (svensson@esrf.fr)
#
#    Contributing authors:   Marie-Francoise Incardona (incardon@esrf.fr)
#                            Gleb Bourenkov (Gleb.Bourenkov@embl-hamburg.de)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer

from EDAssert                            import EDAssert
from EDTestCasePluginExecute             import EDTestCasePluginExecute
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath


class EDTestCasePluginExecuteSubWedgeMergev10( EDTestCasePluginExecute ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginExecute.__init__( self, "EDPluginSubWedgeMergev10", "EDPluginGroupInterface-v1.0", _edStringTestName )
        self.setConfigurationFile( self.getRefConfigFile() )
        self.setDataInputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataInputSubWedgeMerge_reference.xml" ) )
        
        self.setReferenceDataOutputFile( EDDiskExplorer.mergePath( self.getPluginTestsDataHome(), "XSDataResultSubWedgeMerge_reference.xml"))
        self.m_edObtainedOutputDataFile = self.getPluginName() + "_output.xml"
        
        
        
        
    def testExecute( self ):
        """
        """ 
        self.run()
        
        # Checks that there are no error messages

        EDVerbose.DEBUG("Checking error messages...")
        EDAssert.equal( 0, self.getErrorMessages().getNumberObjects() )
            
        EDVerbose.DEBUG("Checking warning messages...")
        EDAssert.equal( 0, self.getWarningMessages().getNumberObjects() )
        
        # Checks the expected result
        edStringExpectedOutput = EDUtilsTest.readAndParseFile ( self.getReferenceDataOutputFile() )
        edStringObtainedOutput = EDUtilsTest.readAndParseFile ( self.m_edObtainedOutputDataFile ) 
        from XSDataMXv1 import XSDataSubWedge
        xsDataSubWedgeObtained  = XSDataSubWedge.parseString( edStringObtainedOutput )  
        xsDataSubWedgeReference = XSDataSubWedge.parseString( edStringExpectedOutput )  
        EDVerbose.DEBUG("Checking obtained result...") 
        EDAssert.equal( xsDataSubWedgeReference.marshal(), xsDataSubWedgeObtained.marshal() )
        EDDiskExplorer.deleteFile( self.m_edObtainedOutputDataFile )



##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testExecute )

        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginExecuteSubWedgeMergev10 = EDTestCasePluginExecuteSubWedgeMergev10( "EDTestCasePluginExecuteSubWedgeMergev10" )
    edTestCasePluginExecuteSubWedgeMergev10.execute()