#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDPluginExecReadImageHeaderMARCCDv10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDDiskExplorer
from EDImportLib import EDDictionary

from EDImportSystem import PyStruct
from EDMessage      import EDMessage
from EDUtilsImage   import EDUtilsImage

from EDPluginExec import EDPluginExec

from XSDataCommon import XSDataFile
from XSDataCommon import XSDataWavelength
from XSDataCommon import XSDataImage
from XSDataCommon import XSDataAngle
from XSDataCommon import XSDataLength
from XSDataCommon import XSDataTime
from XSDataCommon import XSDataString
from XSDataCommon import XSDataInteger

from XSDataMXv1 import XSDataExperimentalCondition
from XSDataMXv1 import XSDataSubWedge
from XSDataMXv1 import XSDataDetector
from XSDataMXv1 import XSDataBeam
from XSDataMXv1 import XSDataGoniostat
from XSDataMXv1 import XSDataInputReadImageHeader
from XSDataMXv1 import XSDataResultReadImageHeader



class EDPluginExecReadImageHeaderMARCCDv10( EDPluginExec ):


    def __init__( self ):
        """
        """
        EDPluginExec.__init__( self )
        self.setXSDataInputClass( XSDataInputReadImageHeader )
        self.m_xsDataResultReadImageHeader = None
    

    def checkParameters( self ):
        """
        Checks the mandatory parameters.
        """
        EDVerbose.DEBUG( "*** EDPluginExecReadImageHeaderMARCCDv10.checkParameters")
        self.checkMandatoryParameters( self.getDataInput(),                 "Data Input is None" )
        
        
    def process( self, _edObject = None ):
        """
        """
        EDPluginExec.process( self )
        EDVerbose.DEBUG( "*** EDPluginExecReadImageHeaderMARCCDv10.process" )
        xsDataInputReadImageHeader = self.getDataInput() 
        xsDataFile = xsDataInputReadImageHeader.getImage()
        edStringPath = EDString( xsDataFile.getPath().getValue() )
        edDictictionaryMARCCDHeader = self.readHeaderMarccd( edStringPath )
        if ( edDictictionaryMARCCDHeader is None ):
            errorMessage = EDMessage.ERROR_CANNOT_READ_FILE_02 % ( "EDPluginExecReadImageHeaderMARCCDv10.process", edStringAbsolutePath )
            EDVerbose.error( errorMessage )
            self.addErrorMessage( errorMessage )
            raise RuntimeError, errorMessage          
        else:
            xsDataExperimentalCondition = XSDataExperimentalCondition()
            xsDataDetector = XSDataDetector()  

            xsDataDetector.setBeamPositionX(       XSDataLength(  EDString( edDictictionaryMARCCDHeader[ "beam_x" ] ).atof()/1000.0 ) )
            xsDataDetector.setBeamPositionY(       XSDataLength(  EDString( edDictictionaryMARCCDHeader[ "beam_y" ] ).atof()/1000.0 ) )
            xsDataDetector.setDistance(            XSDataLength(  EDString( edDictictionaryMARCCDHeader[ "xtal_to_detector"   ] ).atof()/1000.0 ) )
            dPixelSizeX = EDString( edDictictionaryMARCCDHeader[ "pixelsize_x"   ] ).atof()/1000.0
            xsDataDetector.setPixelSizeX(          XSDataLength(  dPixelSizeX ) )
            dPixelSizeY = EDString( edDictictionaryMARCCDHeader[ "pixelsize_y"   ] ).atof()/1000.0
            xsDataDetector.setPixelSizeY(          XSDataLength(  dPixelSizeY ) )
            xsDataDetector.setTwoTheta(            XSDataAngle(   EDString( edDictictionaryMARCCDHeader[ "end_twotheta"   ] ).atof()/1000.0 ) )
            xsDataDetector.setNumberBytesInHeader( XSDataInteger( EDString( edDictictionaryMARCCDHeader[ "header_size"   ] ).atof()  ) )
            #xsDataDetector.setSerialNumber(        XSDataInteger( EDString( edDictictionaryMARCCDHeader[ "DETECTOR_SN"   ] ).atof() ) )
            xsDataDetector.setNumberPixelX(        XSDataInteger( EDString( edDictictionaryMARCCDHeader[ "nslow"   ] ).atof() ) )
            xsDataDetector.setNumberPixelY(        XSDataInteger( EDString( edDictictionaryMARCCDHeader[ "nfast"   ] ).atof() ) )
            #xsDataDetector.setBin(                 XSDataString(  EDString( edDictictionaryMARCCDHeader[ "BIN" ] ) ) )
            #xsDataDetector.setDataType(            XSDataString(  EDString( edDictictionaryMARCCDHeader[ "TYPE" ] ) ) )
            #xsDataDetector.setByteOrder(           XSDataString(  EDString( edDictictionaryMARCCDHeader[ "BYTE_ORDER" ] ) ) )
            xsDataDetector.setImageSaturation(     XSDataString(  EDString( edDictictionaryMARCCDHeader[ "saturation_level" ] ) ) )
            # Determine type of detector...
            iNoPixelsX = xsDataDetector.getNumberPixelX().getValue()
            iNoPixelsY = xsDataDetector.getNumberPixelY().getValue()
            if ( iNoPixelsX == 2048 and iNoPixelsY == 2048 ):
                xsDataDetector.setName( XSDataString( "MAR CCD 165" ) )
                xsDataDetector.setType( XSDataString( "mar165" ) )
            elif ( iNoPixelsX == 3072 and iNoPixelsY == 3072 ):
                xsDataDetector.setName( XSDataString( "MAR CCD 225" ) )
                xsDataDetector.setType( XSDataString( "mar225" ) )
            elif ( iNoPixelsX == 4096 and iNoPixelsY == 4096 ):
                xsDataDetector.setName( XSDataString( "MAR CCD 325" ) )
                xsDataDetector.setType( XSDataString( "mar325" ) )
            else:
                errorMessage = EDMessage.ERROR_DATA_HANDLER_02 % ("EDPluginReadImageHeaderv10.process", "Unknown detector type" )
                EDVerbose.error( errorMessage )
                self.addErrorMessage( errorMessage )
                raise RuntimeError, errorMessage

            xsDataExperimentalCondition.setDetector( xsDataDetector )

            # Beam object
            
            xsDataBeam = XSDataBeam()
            xsDataBeam.setWavelength(   XSDataWavelength( EDString( edDictictionaryMARCCDHeader[ "source_wavelength" ] ).atof()/100000.0 ) )
            xsDataBeam.setExposureTime( XSDataTime(       EDString( edDictictionaryMARCCDHeader[ "exposure_time" ] ).atof()/1000.0 ) )               
            xsDataExperimentalCondition.setBeam( xsDataBeam )
            
            # Goniostat object
            xsDataGoniostat = XSDataGoniostat()
            dRotationAxisStart = EDString( edDictictionaryMARCCDHeader[ "start_phi" ] ).atof()/1000.0
            dOscillationWidth = EDString( edDictictionaryMARCCDHeader[ "rotation_range" ] ).atof()/1000.0
            xsDataGoniostat.setRotationAxisStart( XSDataAngle( dRotationAxisStart ) )
            xsDataGoniostat.setRotationAxisEnd(   XSDataAngle( dRotationAxisStart + dOscillationWidth ) )
            xsDataGoniostat.setOscillationWidth(  XSDataAngle( dOscillationWidth ) )
            xsDataExperimentalCondition.setGoniostat( xsDataGoniostat )
            
            # Create the image object
            xsDataImage = XSDataImage()
            xsDataImage.setPath( XSDataString( edStringPath ))
            edStringTimeStamp = EDString( edDictictionaryMARCCDHeader[ "acquire_timestamp" ] )
            xsDataImage.setDate( XSDataString( edStringTimeStamp ) )
            edStringFileName = EDDiskExplorer.getFileName( edStringPath )
            iImageNumber = EDUtilsImage.getImageNumber( edStringPath )
            xsDataImage.setNumber( XSDataInteger( iImageNumber ) ) 
            
            xsDataSubWedge = XSDataSubWedge()
            xsDataSubWedge.setExperimentalCondition( xsDataExperimentalCondition )
            xsDataSubWedge.addImage( xsDataImage )

            self.m_xsDataResultReadImageHeader = XSDataResultReadImageHeader()
            self.m_xsDataResultReadImageHeader.setSubWedge( xsDataSubWedge )



    def postProcess( self, _edObject = None ):
        """
        """
        EDPluginExec.postProcess( self )
        EDVerbose.DEBUG( "*** EDPluginReadImageHeaderv10.postProcEDUtilsImage.readHeaderADSC( ess")
        if ( self.m_xsDataResultReadImageHeader is not None ):
            self.setDataOutput( self.m_xsDataResultReadImageHeader )
        

    def readHeaderMarccd( self, _edStringFileName ):
        """
        Returns an EDDictionary with the contents of a MAR CCD image header.
        """
        #
        #  Created by Krister Larsson on 08/11/5.
        #  Modified by Olof Svensson 20090401
        #
        pyStrUint32 = 'I'
        pyStrChar   = 's'
        pyStrInt16  = 'H'
        pyStrInt32  = 'i'
    
        iMAXIMAGES=9
    
        pyListHeader=[
        #File/header format parameters (256bytes)
        [pyStrUint32,'header_type'],
        [pyStrChar,  'header_name',16],
        [pyStrUint32,'header_major_version'],
        [pyStrUint32,'header_minor_version'],
        [pyStrUint32,'header_byte_order'],
        [pyStrUint32,'data_byte_order'],
        [pyStrUint32,'header_size'],
        [pyStrUint32,'frame_type'],
        [pyStrUint32,'magic_number'],
        [pyStrUint32,'compression_type'],
        [pyStrUint32,'compression1'],
        [pyStrUint32,'compression2'],
        [pyStrUint32,'compression3'],
        [pyStrUint32,'compression4'],
        [pyStrUint32,'compression5'],
        [pyStrUint32,'compression6'],
        [pyStrUint32,'nheaders'],
        [pyStrUint32,'nfast'],
        [pyStrUint32,'nslow'],
        [pyStrUint32,'depth'],
        [pyStrUint32,'record_length'],
        [pyStrUint32,'signif_bits'],
        [pyStrUint32,'data_type'],
        [pyStrUint32,'saturated_value'],
        [pyStrUint32,'sequence'],
        [pyStrUint32,'nimages'],
        [pyStrUint32,'origin'],
        [pyStrUint32,'orientation'],
        [pyStrUint32,'view_direction'],
        [pyStrUint32,'overflow_direction'],
        [pyStrUint32,'over_8_bits'],
        [pyStrUint32,'over_16_bits'],
        [pyStrUint32,'multiplexed'],
        [pyStrUint32,'nfastimages'],
        [pyStrUint32,'nslowimages'],
        [pyStrUint32,'darkcurrent_applied'],
        [pyStrUint32,'bias_applied'],
        [pyStrUint32,'flatfield_applied'],
        [pyStrUint32,'distortion_applied'],
        [pyStrUint32,'original_header_type'],
        [pyStrUint32,'file_saved'],
        [pyStrUint32,'n_valid_pixels'],
        [pyStrUint32,'defectmap_applied'],
        [pyStrUint32,'subimage_nfast'],
        [pyStrUint32,'subimage_nslow'],
        [pyStrUint32,'subimage_origin_fast'],
        [pyStrUint32,'subimage_origin_slow'],
        [pyStrUint32,'readout_pattern'],
        [pyStrUint32,'saturation_level'],
        [pyStrUint32,'orientation_code'],
        [pyStrUint32,'frameshift_multiplexed'],
        [pyStrUint32,'precsan_nfast'],
        [pyStrUint32,'prescan_nslow'],
        [pyStrUint32,'postscan_nfast'],
        [pyStrUint32,'postscan_nslow'],
        [pyStrUint32,'prepost_trimmed'],
        [pyStrChar,  'reserve1',20],
        #Data statistics (128 bytes)
        [pyStrUint32,'total_counts',2],
        [pyStrUint32,'special_counts1',2],
        [pyStrUint32,'special_counts2',2],
        [pyStrUint32,'min'],
        [pyStrUint32,'max'],
        [pyStrUint32,'mean'],
        [pyStrUint32,'rms'],
        [pyStrUint32,'n_zeros'],
        [pyStrUint32,'n_saturated'],
        [pyStrUint32,'stats_uptodate'],
        [pyStrUint32,'pixel_noise', iMAXIMAGES],
        [pyStrChar,  'reserve2', ( 19 - iMAXIMAGES ) * 4 ],
    
        #More statistics (256 bytes)
        #[uint16,'percentile',128],
        #Percentile is substituted by Sample changer info
        [pyStrChar,       'barcode',16],
        [pyStrUint32,'barcode_angle'],
        [pyStrUint32,'barcode_status'],
        [pyStrChar,  'reserve2a',232],
        #Goniostat parameters (128 bytes)
        [pyStrInt32,'xtal_to_detector'],
        [pyStrInt32,'beam_x'],
        [pyStrInt32,'beam_y'],
        [pyStrInt32,'integration_time'],
        [pyStrInt32,'exposure_time'],
        [pyStrInt32,'readout_time'],
        [pyStrInt32,'nreads'],
        [pyStrInt32,'start_twotheta'],
        [pyStrInt32,'start_omega'],
        [pyStrInt32,'start_chi'],
        [pyStrInt32,'start_kappa'],
        [pyStrInt32,'start_phi'],
        [pyStrInt32,'start_delta'],
        [pyStrInt32,'start_gamma'],
        [pyStrInt32,'start_xtal_to_detector'],
        [pyStrInt32,'end_twotheta'],
        [pyStrInt32,'end_omega'],
        [pyStrInt32,'end_chi'],
        [pyStrInt32,'end_kappa'],
        [pyStrInt32,'end_phi'],
        [pyStrInt32,'end_delta'],
        [pyStrInt32,'end_gamma'],
        [pyStrInt32,'end_xtal_to_detector'],
        [pyStrInt32,'rotation_axis'],
        [pyStrInt32,'rotation_range'],
        [pyStrInt32,'detector_rotx'],
        [pyStrInt32,'detector_roty'],
        [pyStrInt32,'detector_rotz'],
        [pyStrInt32,'total_dose'],
        [pyStrChar, 'reserve3',12],
        #Detector parameters (128 bytes)
        [pyStrInt32,'detector_type'],
        [pyStrInt32,'pixelsize_x'],
        [pyStrInt32,'pixelsize_y'],
        [pyStrInt32,'mean_bias'],
        [pyStrInt32,'photons_per_100adu'],
        [pyStrInt32,'measured_bias', iMAXIMAGES],
        [pyStrInt32,'measured_temperature', iMAXIMAGES],
        [pyStrInt32,'measured_pressure', iMAXIMAGES],
        #X-ray source and optics parameters (128 bytes)
        [pyStrInt32,'source_type'],
        [pyStrInt32,'source_dx'],
        [pyStrInt32,'source_dy'],
        [pyStrInt32,'source_wavelength'],
        [pyStrInt32,'source_power'],
        [pyStrInt32,'source_voltage'],
        [pyStrInt32,'source_current'],
        [pyStrInt32,'source_bias'],
        [pyStrInt32,'source_polarization_x'],
        [pyStrInt32,'source_polarization_y'],
        [pyStrInt32,'source_intensity_0'],
        [pyStrInt32,'source_intensity_1'],
        [pyStrChar, 'reserve_source',8],
        [pyStrInt32,'optics_type'],
        [pyStrInt32,'optics_dx'],
        [pyStrInt32,'optics_dy'],
        [pyStrInt32,'optics_wavelength'],
        [pyStrInt32,'optics_dispersion'],
        [pyStrInt32,'optics_crossfire_x'],
        [pyStrInt32,'optics_crossfire_y'],
        [pyStrInt32,'optics_angle'],
        [pyStrInt32,'optics_polarization_x'],
        [pyStrInt32,'optics_polarization_y'],
        [pyStrChar,'reserve_optics',16],
        [pyStrChar,'reserve5',16],
        #File parameters
        [pyStrChar,'filetitle',128],
        [pyStrChar,'filepath',128],
        [pyStrChar,'filename',64],
        [pyStrChar,'acquire_timestamp',32],
        [pyStrChar,'header_timestamp',32],
        [pyStrChar,'save_timestamp',32],
        [pyStrChar,'file_comment',512],
        [pyStrChar,'reserve6',96],
        [pyStrChar,'dataset_comments',512],
        [pyStrChar,'user_data',512]
        ]
    
        pyFile=None
        edDictionary=None
        try:
            pyFile=open( _edStringFileName, "rb")
        except:
            EDVerbose.warning( "*** EDUtilsImage.readHeaderMarccd: couldn't open file: " + _edStringFileName )
        if ( pyFile is not None ):
            EDVerbose.DEBUG( "*** EDUtilsImage.readHeaderMarccd: Reading header from image " + _edStringFileName )
            edDictionary=EDDictionary()
            #Move to marccd part of header
            pyFile.seek( 1024 )
            for pyListRow in pyListHeader:
                if len( pyListRow ) == 3:
                    edStringFormat = EDString( pyListRow[2] ) + EDString( pyListRow[0] )
                else:
                    edStringFormat = EDString( pyListRow[0] )
                edStringKeyword = EDString( pyListRow[1] )
                iReadSize = PyStruct.calcsize( edStringFormat )
                pyRawData = pyFile.read( iReadSize )
                pyStrValue = PyStruct.unpack( edStringFormat, pyRawData)
                if len( pyStrValue ) == 1:
                    pyStrValue = pyStrValue[0]
                if pyListRow[0] == pyStrChar:
                    pyStrValue = pyStrValue.strip( '\x00' )
                edDictionary.add( edStringKeyword, EDString( pyStrValue ) )
            pyFile.close()
    
        return edDictionary

