#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCasePluginUnitPluginControlReadImageHeaderv10.py 698 2009-05-11 10:43:08Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal authors:      Marie-Francoise Incardona (incardon@esrf.fr)
#                            Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDString

from EDAssert                            import EDAssert
from EDTestCasePluginUnit                import EDTestCasePluginUnit


class EDTestCasePluginUnitPluginControlReadImageHeaderv10( EDTestCasePluginUnit ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCasePluginUnit.__init__( self, "EDPluginControlReadImageHeaderv10", "EDPluginGroupReadImageHeader-v0.1", _edStringTestName )

        self.m_edStringDataPath = self.getPluginTestsDataHome()       
        

    def testDetermineImageType( self ):
        """
        """
        edPluginControlReadImageHeaderv10 = self.createPlugin()
        # Test 1 : ADSC 
        edStringImage1 = EDString( "test_1_001.img" )
        edStringType1  = edPluginControlReadImageHeaderv10.determineImageType( edStringImage1 )
        EDAssert.equal( edStringType1, EDString( "ADSC" ) )
        # Test 2 : MAR CCD
        edStringImage2 = EDString( "test_1_001.mccd" )
        edStringType2  = edPluginControlReadImageHeaderv10.determineImageType( edStringImage2 )
        EDAssert.equal( edStringType2, EDString( "MARCCD" ) )
        # Test 3 : MAR CCD
        edStringImage3 = EDString( "test_1_001.marccd" )
        edStringType3  = edPluginControlReadImageHeaderv10.determineImageType( edStringImage3 )
        EDAssert.equal( edStringType3, EDString( "MARCCD" ) )
        # Test 4 : Unknown
        edStringImage4 = EDString( "test_1_004.unknown" )
        bException = False
        try:
            edStringType4  = edPluginControlReadImageHeaderv10.determineImageType( edStringImage4 )
        except:
            bException = True
        EDAssert.equal( True, bException )
            
            
    def testDetermineExecReadImageHeaderPluginName( self ):
        edPluginControlReadImageHeaderv10 = self.createPlugin()
        # Test 1 : ADSC 
        edStringType1       = EDString( "ADSC" )
        edStringPluginName1 = edPluginControlReadImageHeaderv10.determineExecReadImageHeaderPluginName( edStringType1 )
        EDAssert.equal( edStringPluginName1, "EDPluginExecReadImageHeaderADSCv10" )
        # Test 2 : MAR CCD 
        edStringType2       = EDString( "MARCCD" )
        edStringPluginName2 = edPluginControlReadImageHeaderv10.determineExecReadImageHeaderPluginName( edStringType2 )
        EDAssert.equal( edStringPluginName2, "EDPluginExecReadImageHeaderMARCCDv10" )
        # Test 3 : UNKNOWN 
        edStringType3       = EDString( "UNKNOWN" )
        bException = False
        try:
            edStringPluginName3 = edPluginControlReadImageHeaderv10.determineExecReadImageHeaderPluginName( edStringType3 )
        except:
            bException = True
        EDAssert.equal( True, bException )
        


             

##############################################################################

    def process( self ):
        """
        """
        self.addTestMethod( self.testDetermineImageType )
        self.addTestMethod( self.testDetermineExecReadImageHeaderPluginName )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCasePluginUnitPluginControlReadImageHeaderv10 = EDTestCasePluginUnitPluginControlReadImageHeaderv10( "EDTestCasePluginUnitPluginControlReadImageHeaderv10" )
    edTestCasePluginUnitPluginControlReadImageHeaderv10.execute()

