#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDApplicationDeprecated.py 586 2008-12-11 10:28:54Z incardon $"
#
#    Copyright (C) 2008 European Synchrotron Radiation Facility
#                       Grenoble, France
#
#    Principal authors: Marie-Francoise Incardona (incardon@esrf.fr)
#                       Olof Svensson (svensson@esrf.fr) 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib import EDObject
from EDImportLib import EDVerbose
from EDImportLib import EDString
from EDImportLib import EDTime
from EDImportLib             import EDCommandLine
from EDImportLib             import EDCommandLineArgument
from EDImportLib             import EDLogFile
from EDImportLib import EDSemaphore

from EDConfiguration import EDConfiguration
from EDFactoryPlugin import EDFactoryPlugin
from EDApplication import EDApplication


class EDApplicationDeprecated( EDObject ):

    m_edStrCONFIGURATION_PARAM_LABEL = "--conf"
    m_edConfiguration = None
    m_edSemaphore = None
    m_edFactoryPlugin = None 

    
    """
    This is a Configurable ALApplication
    """
    def __init__( self, _edStringName, _edStringVersion, _edStringConfigurationFileName = None ):
        """
        """
        EDObject.__init__( self )
        self.m_edStringName = _edStringName
        self.m_edStringVersion = _edStringVersion
        self.m_edCommandLine = EDCommandLine( EDCommandLineArgument() )
        # If edStringConfigurationFileName is None, this means that it has not been given to the constructor\
        # It has been given by the command line\
        if( _edStringConfigurationFileName == None ):
            self.m_edStrConfigurationFileName = self.getCommandLineArgument( EDApplicationDeprecated.m_edStrCONFIGURATION_PARAM_LABEL )
        else:
            self.m_edStrConfigurationFileName = _edStringConfigurationFileName
        # Set the name of the log file
        EDVerbose.m_aolLogFile = EDLogFile( self.getApplicationName() + ".log")
        
    
    def setConfiguration( self, _edConfiguration ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplicationDeprecated.setConfiguration" )
        EDApplicationDeprecated.synchronizeOn()
        if ( _edConfiguration == None ):
            EDVerbose.warning( "*** EDApplicationDeprecated.setConfiguration: Configuration is None!" )
        else:             
            EDApplicationDeprecated.m_edConfiguration = _edConfiguration
            EDApplication.m_edConfiguration = _edConfiguration
        EDApplicationDeprecated.synchronizeOff()
        
        
    def getPluginConfiguration( _pluginName ):
        """
        """
        EDVerbose.DEBUG( "*** EDApplicationDeprecated.getPluginConfiguration" )
        EDApplicationDeprecated.synchronizeOn()
        pluginConfiguration = None
        if ( EDApplicationDeprecated.m_edConfiguration != None ):
            pluginConfiguration = EDApplicationDeprecated.m_edConfiguration.getPluginItem( _pluginName )
        else:
            EDVerbose.warning( "*** EDApplicationDeprecated.getPluginConfiguration: No configuration at all!" ) 
        EDApplicationDeprecated.synchronizeOff()
        return pluginConfiguration
    getPluginConfiguration = staticmethod( getPluginConfiguration )
        
    
    def preProcess( self ):
        """
        Reads the command line and retrieves the configuration file path, the plugin name and the dataset file name
        """
        #EDVerbose.DEBUG( "*** EDApplicationDeprecated.preProcess: Getting Configuration File Path from command..." )        
        #edStrConfigurationFile = self.getCommandLineArgument( EDApplicationDeprecated.m_edStrCONFIGURATION_PARAM_LABEL )
        #EDVerbose.DEBUG( "*** EDApplicationDeprecated.preProcess: Configuration File Path: " + edStrConfigurationFile)

        EDVerbose.DEBUG( "*** EDApplicationDeprecated.preProcess: Loading Configuration if not already loaded..." )
        edConfiguration = EDConfiguration( self.m_edStrConfigurationFileName )
        edConfiguration.load()
        self.setConfiguration( edConfiguration )        
        EDVerbose.DEBUG( "*** EDApplicationDeprecated.preProcess: Checking... Number of plugins...: " + EDString( EDApplicationDeprecated.m_edConfiguration.getPluginListSize()) )
        
        
    def process( self ):
        """
        Calls the Plugin to be executed
        """
        EDVerbose.DEBUG( "*** EDApplicationDeprecated.process: I'm processing" )            
        
                    
    def init( self, _bInit = True ):
        """
        This is the Init method in the ALAppDefinitionInit.
        """
        EDVerbose.DEBUG( "Execution: ALAppDefinitionInit.init()" )
        oBoolStatus = True
        edString = EDString( self.m_edCommandLine.getCommandLine() )
        EDVerbose.log( edString )
        EDVerbose.setVerboseOff()

        if (self.m_edCommandLine.existCommand( "--verbose" )):
            EDVerbose.setVerboseOn()

        if (self.m_edCommandLine.existCommand( "--quiet" )):
            EDVerbose.setVerboseOff()

        if (self.m_edCommandLine.existCommand( "--DEBUG" )):
            EDVerbose.setVerboseDebugOn()
            EDVerbose.DEBUG( "Debug Mode [ON]" )

        if (_bInit==True):
            #EDVerbose.screen( self.m_edApplicationName + "::Starting" )
            if (self.m_edCommandLine.existCommand( "--help" )
                or self.m_edCommandLine.existCommand( "-h" )):
                EDVerbose.setVerboseOn()
                self.usage()
                oBoolStatus = False

            if (self.m_edCommandLine.existCommand( "--version" ) or
                self.m_edCommandLine.existCommand( "-v" )):
                EDVerbose.setVerboseOn()
                EDVerbose.screen( "%s version %s" % ( self.m_edStringName, self.m_edStringVersion ) )
                oBoolStatus = False

            if (self.m_edCommandLine.existCommand( "--no-log" )):
                EDVerbose.setLogFileOff()

        # The OK to continue the application
        return oBoolStatus
    

#    def end( self ):
#        """
#        End The application framework...closing and registering the resources
#        """
#        self.__forceCloseActions()
#        self.__forceCloseTimerActions()
#        ALAppDefinitionEnd.setResource( AL_FRAMEWORK_LAST_RUN, EDTime.timeString() )
#        ALAppDefinitionEnd.outputResource()
#        self.__getActionLog()
#
#
#        ofAppRunTime     = self.getRunTime()
#        oalStrAppName    = ALAppDefinitionEnd.getApplicationName()
#        edStrAppRunTime = EDString( ofAppRunTime )
#        EDVerbose.outputActionLog()
#        EDVerbose.log( oalStrAppName + "::Closing" )
#        EDVerbose.screen( oalStrAppName + "::Runtime: " + edStrAppRunTime + " [s]" )
#        EDVerbose.log( oalStrAppName + "::Runtime: " + EDTime.getTimeStringPSP( ofAppRunTime ) )
#
#
#        # Used to generate a XML profile log file to analise the performance
#        ALAppDefinitionEnd.outputProfile()
#
#        # To be safety and close external pipes on remote control
#        #ALSystem.exit( 0 ) #Bug in Python 2.4 - it raises an exception !!!!!!!!!!

    def getCommandLine( self ):
        return self.m_edCommandLine.getCommandLine()


    def getCommandLineArguments( self ):
        EDApplicationDeprecated.synchronizeOn()
        edCommandLine = self.m_edCommandLine.getCommandLine()
        EDApplicationDeprecated.synchronizeOff()
        return edCommandLine

    def getCommandLineArgument( self, _edStringKey ):
        EDApplicationDeprecated.synchronizeOn()
        edStringCommandLineArgument = self.m_edCommandLine.getArgument( _edStringKey )
        EDApplicationDeprecated.synchronizeOff()
        return edStringCommandLineArgument
    
    def postProcess( self ):
        pass
    
    def execute( self ):
        if ( self.init() == True ):
            self.preProcess()
            self.process()
            self.postProcess()


    def synchronizeOn():
        if ( EDApplicationDeprecated.m_edSemaphore is None ):
            EDApplicationDeprecated.m_edSemaphore = EDSemaphore()
        EDApplicationDeprecated.m_edSemaphore.acquire()
    synchronizeOn = staticmethod( synchronizeOn )


    def synchronizeOff():
        if ( EDApplicationDeprecated.m_edSemaphore is None ):
            EDApplicationDeprecated.m_edSemaphore = EDSemaphore()
        EDApplicationDeprecated.m_edSemaphore.release()
    synchronizeOff = staticmethod( synchronizeOff )

        
    def getApplicationName( self ):
        return EDString( self.m_edStringName + "-" + self.m_edStringVersion )


    def getFactoryPlugin():
        EDApplicationDeprecated.synchronizeOn()
        if ( EDApplicationDeprecated.m_edFactoryPlugin is None ):
            EDApplicationDeprecated.m_edFactoryPlugin = EDFactoryPlugin()
        EDApplicationDeprecated.synchronizeOff()
        return EDApplicationDeprecated.m_edFactoryPlugin
    getFactoryPlugin = staticmethod( getFactoryPlugin )
    
    
    def loadPlugin( _edStringPluginName ):
        return EDApplicationDeprecated.getFactoryPlugin().loadPlugin( _edStringPluginName )
    loadPlugin = staticmethod( loadPlugin )

    
    def loadModule( _edStringModuleName ):
        EDApplicationDeprecated.getFactoryPlugin().loadModule( _edStringModuleName )
    loadModule = staticmethod( loadModule )
    