#
#    Project: EDNA MXv1
#             http://www.edna-site.org
#
#    File: "$Id: EDTestCaseEDHandlerLabelitv10.py 745 2009-05-29 10:26:40Z svensson $"
#
#    Copyright (C) 2008-2009 European Synchrotron Radiation Facility
#                            Grenoble, France
#
#    Principal author:       Olof Svensson (svensson@esrf.fr) 
#
#    Contributors:           Marie-Francoise Incardona (incardon@esrf.fr)
#                            Karl Levik (karl.levik@diamond.ac.uk)
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from EDImportLib                         import EDCompiler
from EDImportLib                         import EDVerbose
from EDImportLib                         import EDString
from EDImportLib                         import EDDiskExplorer
from EDImportLib                         import EDDict
from EDImportLib                         import EDDictionary
from EDImportLib                         import EDList

from EDAssert                            import EDAssert
from EDUtilsFile                         import EDUtilsFile
from EDTestCase                          import EDTestCase
from EDUtilsTest                         import EDUtilsTest
from EDUtilsPath                         import EDUtilsPath
from EDApplication                       import EDApplication


class EDTestCaseEDHandlerLabelitv10( EDTestCase ):
    """
    """
    
    def __init__( self, _edStringTestName = None):
        """
        """
        EDTestCase.__init__( self, _edStringTestName )
        edStringMXv1DataHome = EDUtilsTest.getPluginTestDataDirectory( self.getClassName() )
        edStringDataDir = "EDHandlerLabelitv10"
        self.m_edStringDataPath = EDUtilsPath.mergePath( edStringMXv1DataHome, edStringDataDir )        
        self.m_edStringReferenceInputFile = EDDiskExplorer.mergePath( self.m_edStringDataPath, "XSDataInputLabelit_reference.xml")
        self.m_edStringReferenceOutputFile = EDDiskExplorer.mergePath( self.m_edStringDataPath, "XSDataResultLabelit_reference.xml")
        
        
        
            
        
    
    
    def testGenerateXSDataInputLabelit( self, _edStringFileNameXSDataIndexingInput_reference, 
                                              _edStringFileNameXSDataLabelitInput_reference ):
        """
        This method is testing the generation of the XSDataInputLabelit object given a XSDataIndexingInput object.
        """
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  _edStringFileNameXSDataIndexingInput_reference )
        edStringXMLIndexingInput = EDUtilsTest.readAndParseFile( edStringPath )
        from XSDataMXv1 import XSDataIndexingInput
        xsDataIndexingInput = XSDataIndexingInput.parseString( edStringXMLIndexingInput  )
        from EDHandlerXSDataLabelitv10 import EDHandlerXSDataLabelitv10
        xsDataInputLabelit = EDHandlerXSDataLabelitv10.generateXSDataInputLabelit( xsDataIndexingInput )
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  _edStringFileNameXSDataLabelitInput_reference )
        edStringXMLInputLabelitReference = EDUtilsTest.readAndParseFile( edStringReferencePath )
        EDApplication.loadModule( "XSDataLabelitv10" )
        from XSDataLabelitv10 import XSDataInputLabelit
        xsDataInputLabelitReference = XSDataInputLabelit.parseString( edStringXMLInputLabelitReference )
        EDAssert.equal( xsDataInputLabelitReference.marshal(), xsDataInputLabelit.marshal() ) 


    def testGenerateXSDataInputLabelit_reference( self ):
        """
        This method tests the generation of an XSDataInputLabelit object given two reference 
        images collected more than 4 degrees apart.
        """
        self.testGenerateXSDataInputLabelit( "XSDataIndexingInput_reference.xml", 
                                             "XSDataInputLabelit_reference.xml" )

    
    def testGenerateXSDataInputLabelit_oneImage( self ):
        """
        This method tests the generation of an XSDataInputLabelit object given one reference image.
        """
        self.testGenerateXSDataInputLabelit( "XSDataIndexingInput_oneImage.xml", 
                                             "XSDataInputLabelit_oneImage.xml" )

    
    def testGenerateXSDataInputLabelit_twoImagesLessThan4degrees( self ):
        """
        This method tests the generation of an XSDataInputLabelit object given two reference 
        images collected less than 4 degrees apart.
        """
        self.testGenerateXSDataInputLabelit( "XSDataIndexingInput_twoImagesLessThan4degrees.xml", 
                                             "XSDataInputLabelit_oneImage.xml" )
               

    def testGenerateXSDataIndexingResult( self ):
        """
        This method tests the generation of an XSDataIndexingResult object given an XSDataResultLabelit object.
        """
        edStringPath = EDDiskExplorer.mergePath( self.m_edStringDataPath,  self.m_edStringReferenceOutputFile )
        edStringXMLResultLabelit = EDUtilsTest.readAndParseFile( edStringPath )
        EDApplication.loadModule( "XSDataLabelitv10" )
        from XSDataLabelitv10 import XSDataResultLabelit
        xsDataResultLabelit = XSDataResultLabelit.parseString( edStringXMLResultLabelit  )
        from EDHandlerXSDataLabelitv10 import EDHandlerXSDataLabelitv10
        xsDataIndexingResult = EDHandlerXSDataLabelitv10.generateXSDataIndexingResult( xsDataResultLabelit )
        edStringReferencePath = EDDiskExplorer.mergePath( self.m_edStringDataPath, "XSDataIndexingResult_reference.xml" )
        edStringIndexingResultReferenceXML = EDUtilsTest.readAndParseFile( edStringReferencePath )
        from XSDataMXv1 import XSDataIndexingResult
        xsDataIndexingResultReference = XSDataIndexingResult.parseString( edStringIndexingResultReferenceXML )
        EDAssert.equal( xsDataIndexingResultReference.marshal(), xsDataIndexingResult.marshal() ) 



##############################################################################
 
    def process( self ):
        """
        """
        self.addTestMethod( self.testGenerateXSDataInputLabelit_reference )
        self.addTestMethod( self.testGenerateXSDataInputLabelit_oneImage )
        self.addTestMethod( self.testGenerateXSDataInputLabelit_twoImagesLessThan4degrees )
        self.addTestMethod( self.testGenerateXSDataIndexingResult )
        
        
##############################################################################


if __name__ == '__main__':

    # JIT compiler accelerator
    EDCompiler.accelerator()
    
    edTestCaseEDHandlerLabelitv10 = EDTestCaseEDHandlerLabelitv10( "EDTestCaseEDHandlerLabelitv10" )
    edTestCaseEDHandlerLabelitv10.execute()

